# =============================================================================
#  This file is part of the ``HASDEL'' Toolset.
#  Copyright (C) 2014 by Fondazione Bruno Kessler
#  Copyright (C) 2014 by RWTH Aachen University
#
#
#  The HASDEL Toolset can be copied, modified and/or redistributed
#  under the terms of the HASDEL General Public License, either
#  version 1 of the License, or (at your option) any later version.
#
#  You should have received a copy of the HASDEL General Public
#  License along with this software. If not, please contact the
#  copyright owners.
#
#  The HASDEL Toolset is distributed in the hope that it will be
#  useful, but WITHOUT ANY WARRANTY; without even the implied warranty
#  of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  HASDEL General Public License for more details.
#
#  For more information on the HASDEL Project see the Project Web
#  site at <http://compassinternal.informatik.rwth-aachen.de/hasdel-site/> 
#  or email to <hasdel-support@lists.rwth-aachen.de>.  
#
#  Please report bugs to <hasdel-support@lists.rwth-aachen.de>.
# =============================================================================


# this file contains all the templates used by the adder example
# generator


import string

slim = string.Template("""
bus Bus
end Bus;

bus implementation Bus.I
end Bus.I;

----------------------------------------------------------------------
system Main
end Main;

system implementation Main.I
  subcomponents
    $gens
    $bits
    $adders

    --
    trig : system Trigger accesses abus;
    abus : bus Bus.I;

  connections
    $gens_bits
    $sources_adders

    -- synchronization: gens, bits, adders
    $gens_tick
    $bits_tick
    $adds_tick

end Main.I;


----------------------------------------------------------------------
-- General interface for a {0,1} dicrete values generator
----------------------------------------------------------------------
system Generator
  features
    tick : in event port;
    output: out data port [0..1] default 0;
end Generator;

----------------------------------------------------------------------
-- Generates a random discrete output when event tick is received
----------------------------------------------------------------------
system implementation Generator.random
  modes
    init: initial mode;
  transitions
    init -[tick then output := 0]-> init;
    init -[tick then output := 1]-> init;
end Generator.random;


----------------------------------------------------------------------
-- A Fault Tolerant Adder which contains a redundant pair of adders
-- and can switch between them
----------------------------------------------------------------------
system FTAdder
  features
    input1: in data port [0..1];
    input2: in data port [0..1];
    output: out data port [0..1] default 0;

    tick: in event port;
    switch: in event port;
end FTAdder;

system implementation FTAdder.I
  subcomponents       
    abus : bus Bus.I;
    adder1: system Adder accesses abus in modes (run1);
    adder2: system Adder accesses abus in modes (run2);

  connections
    data port input1        -> adder1.input1 in modes (run1);
    data port input2        -> adder1.input2 in modes (run1);
    data port adder1.output -> output in modes (run1);

    data port input1        -> adder2.input1 in modes (run2);
    data port input2        -> adder2.input2 in modes (run2);
    data port adder2.output -> output in modes (run2);

    event port tick         -> adder1.tick in modes (run1);
    event port tick         -> adder2.tick in modes (run2);

  modes
   run1 : initial mode;
   run2 : mode;

  transitions
   run1 -[switch]-> run2;
   run2 -[switch]-> run1;

end FTAdder.I;


----------------------------------------------------------------------
-- Performs the binary sum of the two input data, and present the
-- result to the output. There is no carry bit.
----------------------------------------------------------------------
system Adder
  features
    input1: in data port [0..1];
    input2: in data port [0..1];
    output: out data port [0..1] default 0;
    alarm1: out data port bool default false observable;
    tick: in event port;
end Adder;

system implementation Adder.I
  modes
    work : initial mode;

  transitions
    work -[tick then output := input1 + input2 ]-> work;

end Adder.I;


----------------------------------------------------------------------
-- Copies its input to the output
----------------------------------------------------------------------
system Bit
  features
    input: in data port [0..1];
    output: out data port [0..1] default 0;
    alarm1: out data port bool default false observable;
    tick: in event port;
end Bit;

system implementation Bit.I
  modes
    work : initial mode;

  transitions
    work -[tick then output := input ]-> work;

end Bit.I;

    
----------------------------------------------------------------------
-- This component is used to synch components
----------------------------------------------------------------------
system Trigger
  features
    gen_tick : out event port;
    bit_tick : out event port;
    add_tick : out event port;
    switch : in event port;
end Trigger;

system implementation Trigger.I
  modes
    on_gen: initial mode;
    on_bit: mode;
    on_add: mode;

    moff: mode;
    
  transitions
    on_gen -[gen_tick]-> on_bit;
    on_bit -[bit_tick]-> on_add;
    on_add -[add_tick]-> on_gen;
    
    * -[switch]-> moff;
    moff -[switch]-> on_gen;

end Trigger.I;


----------------------------------------------------------------------
-- SMV Properties (reported here as comments only for testing with
-- nusmv)
----------------------------------------------------------------------
$properties
""")


error = string.Template("""
error model AdderError
  features
    normal: initial state;
    stuck_at_zero: error state;
    stuck_at_one: error state;
end AdderError;

error model implementation AdderError.Impl
  events
    adder_stuck_at_zero_FM: error event;
    adder_stuck_at_one_FM : error event;
  transitions
    normal -[adder_stuck_at_zero_FM]-> stuck_at_zero;
    normal -[adder_stuck_at_one_FM] -> stuck_at_one;
end AdderError.Impl;


error model BitError
  features
    normal: initial state;
    inverted: error state;
end BitError;

error model implementation BitError.Impl
  events
    bit_inverted_FM: error event;
  transitions
    normal -[bit_inverted_FM]-> inverted;
end BitError.Impl;
""")


props = string.Template("""<?xml version='1.0' encoding='UTF-8'?>
<ns0:properties xmlns:ns0="http://compass.informatik.rwth-aachen.de/properties">
$properties
</ns0:properties>
""")

ctl = string.Template("""
  <ns0:property>
    <ns0:name>$name</ns0:name>
    <ns0:description> Whenever atomic proposition $precond holds, this is eventually responded with $response. </ns0:description>
    <ns0:ctlPatternInstance patternId="responseGlobal">
      <ns0:stateFormulaInstance xmlns:ns0="http://compass.informatik.rwth-aachen.de/ctl" placeholderId="P">
        <ns0:atomicProposition>$precond</ns0:atomicProposition>
      </ns0:stateFormulaInstance>
      <ns0:stateFormulaInstance xmlns:ns0="http://compass.informatik.rwth-aachen.de/ctl" placeholderId="S">
        <ns0:atomicProposition>$response</ns0:atomicProposition>
      </ns0:stateFormulaInstance>
    </ns0:ctlPatternInstance>
  </ns0:property>
""")

ltl = string.Template("""
  <ns0:property>
    <ns0:name>$name</ns0:name>
    <ns0:description> Whenever atomic proposition $precond holds, this is eventually responded with $response. </ns0:description>
    <ns0:ltlPatternInstance patternId="responseGlobal">
      <ns0:stateFormulaInstance xmlns:ns0="http://compass.informatik.rwth-aachen.de/ltl" placeholderId="P">
        <ns0:atomicProposition>$precond</ns0:atomicProposition>
      </ns0:stateFormulaInstance>
      <ns0:stateFormulaInstance xmlns:ns0="http://compass.informatik.rwth-aachen.de/ltl" placeholderId="S">
        <ns0:atomicProposition>$response</ns0:atomicProposition>
      </ns0:stateFormulaInstance>
    </ns0:ltlPatternInstance>
  </ns0:property>
""")


propos = string.Template("""
  <ns0:property>
    <ns0:name>$name</ns0:name>
    <ns0:description> $formula </ns0:description>
    <ns0:ctlPatternInstance patternId="propositional">
      <ns0:stateFormulaInstance xmlns:ns0="http://compass.informatik.rwth-aachen.de/ctl" placeholderId="P">
        <ns0:atomicProposition>$formula</ns0:atomicProposition>
      </ns0:stateFormulaInstance>
    </ns0:ctlPatternInstance>
  </ns0:property>
""")

slim_fi_adder_assoc = string.Template("""
  <ns0:association>
    <ns0:componentPath>$name.adder1</ns0:componentPath>
    <ns0:errorModel>
      <ns0:packageName>__default__</ns0:packageName>
      <ns0:implementationName>AdderError.Impl</ns0:implementationName>
    </ns0:errorModel>
    <ns0:faultEffectsSpecification>
      <ns0:faultEffects errorState="stuck_at_zero">
        <ns0:effect>output := 0</ns0:effect>
        <ns0:effect>alarm1 := true</ns0:effect>
      </ns0:faultEffects>
      <ns0:faultEffects errorState="stuck_at_one">
        <ns0:effect>output := 1</ns0:effect>
        <ns0:effect>alarm1 := true</ns0:effect>
      </ns0:faultEffects>
    </ns0:faultEffectsSpecification>
  </ns0:association>""")

slim_fi_bit_assoc = string.Template("""
  <ns0:association>
    <ns0:componentPath>$name</ns0:componentPath>
    <ns0:errorModel>
      <ns0:packageName>__default__</ns0:packageName>
      <ns0:implementationName>BitError.Impl</ns0:implementationName>
    </ns0:errorModel>
    <ns0:faultEffectsSpecification>
      <ns0:faultEffects errorState="inverted">
        <ns0:effect>output := case input = 0: 1 otherwise 0 end</ns0:effect>
        <ns0:effect>alarm1 := true</ns0:effect>
      </ns0:faultEffects>
    </ns0:faultEffectsSpecification>
  </ns0:association>
""")


slim_fi = string.Template("""<?xml version='1.0' encoding='UTF-8'?>
<ns0:faultInjection xmlns:ns0="http://compass.informatik.rwth-aachen.de/fault">
  $assocs
</ns0:faultInjection>
""")

