# =============================================================================
#  This file is part of the ``HASDEL'' Toolset.
#  Copyright (C) 2014 by Fondazione Bruno Kessler
#  Copyright (C) 2014 by GMV Aerospace and Defence  
#
#
#  The HASDEL Toolset can be copied, modified and/or redistributed
#  under the terms of the HASDEL General Public License, either
#  version 1 of the License, or (at your option) any later version.
#
#  You should have received a copy of the HASDEL General Public
#  License along with this software. If not, please contact the
#  copyright owners.
#
#  The HASDEL Toolset is distributed in the hope that it will be
#  useful, but WITHOUT ANY WARRANTY; without even the implied warranty
#  of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  HASDEL General Public License for more details.
#
#  For more information on the HASDEL Project see the Project Web
#  site at <http://compassinternal.informatik.rwth-aachen.de/hasdel-site/> 
#  or email to <hasdel-support@lists.rwth-aachen.de>.  
#
#  Please report bugs to <hasdel-support@lists.rwth-aachen.de>.
# =============================================================================

import os.path
import gtk
import gobject
from gtkmvc import Controller
from src.models.mission import PhaseOpModeCombinationEditAssociationDialogModel, \
    PhaseOpModeCombinationEditDefinitionDialogModel, PhaseOpModeListStore
from src.views.mission_phase_op_mode_combinations import \
    PhaseOpModeCombinationEditAssociationDialogView, \
    PhaseOpModeCombinationEditDefinitionDialogView
from delayedcall import DelayedCall
import hasdel_tools.src.functions as functions


class PhaseOpModeCombinationsCtrl (Controller):
    """Represents the Controller of PhaseOpModeCombinations."""

    def __init__(self, model, view):
        """Constructor Method"""
        Controller.__init__(self, model, view)
        return


    def register_view(self, view):
        """Sets the windows, dialog, buttons and other parts of View."""
        Controller.register_view(self, view)
        self.iter = None

        view['button_edit_association'].set_sensitive(False)
        view['button_edit_definition'].set_sensitive(False)

        tv = view['tv_phase_op_mode_combination']
        tv.set_model(self.model.get_phase_op_mode_combinations_tree())
        for col in self.model.get_phase_op_mode_combinations_cols():
            tv.append_column(col)

        self.model.register_observer(self)

        return


    def register_adapters(self):
        return


    def open_edit_association_window(self):
        """Sets the Confirm Window of a new association edited."""
        parent_combination = self.iter
        associated_op_modes = PhaseOpModeListStore()
        iter = self.model.get_phase_op_mode_combinations_tree().iter_children(self.iter)

        while iter is not None:
            comb = self.model.get_phase_op_mode_combinations_tree().get_value(iter, 0)
            associated_op_modes.append([comb.get_op_mode()])
            iter = self.model.get_phase_op_mode_combinations_tree().iter_next(iter)

        m = PhaseOpModeCombinationEditAssociationDialogModel(
            self.model, parent_combination, associated_op_modes)
        v = PhaseOpModeCombinationEditAssociationDialogView()
        c = PhaseOpModeCombinationEditAssociationDialogCtrl(m, v)

        response = v.run()

        if gtk.RESPONSE_APPLY == response:
            self.model.edit_phase_op_mode_associations(m, self.iter)

        return


    def open_edit_definition_window(self):
        """Sets the Confirm Window of a new definition edited."""
        phase_op_mode_combination = \
            self.model.get_phase_op_mode_combinations_tree().get_value(self.iter, 0)

        m = PhaseOpModeCombinationEditDefinitionDialogModel(self.model,
                                                  phase_op_mode_combination)
        v = PhaseOpModeCombinationEditDefinitionDialogView()
        c = PhaseOpModeCombinationEditDefinitionDialogCtrl(m, v)

        response = v.run()

        if gtk.RESPONSE_APPLY == response:
            self.model.edit_phase_op_mode_definition(m)

        return


    # ---------------------------------------------------------------
    #        Signal handlers
    # ---------------------------------------------------------------    
    def _clear_selections(self):
        """Clears the old phase_op_mode_combinations selected."""
        sel = self.view['tv_phase_op_mode_combination'].get_selection()
        sel.unselect_all()
        self.view['button_edit_association'].set_sensitive(False)
        self.view['button_edit_definition'].set_sensitive(False)
        return


    def on_button_edit_association_clicked(self, button):
        """Edits the association selected in the list.
           @param   button:  The Edit Button.
	   @type    button:  self.view['button_edit_association']
        """
        self.open_edit_association_window()
        return


    def on_association_definition_selected(self, tv, event):
        """Selects the association from the list.
           @param   tv:  The Tree View of the association (self.view['edit_association_tv_associated_opm']).
	   @type    tv:  gtk.TreeView
           @param   event:  The event associated
	   @type    event:  gtk.gdk.Event
        """
        if event.type == gtk.gdk._2BUTTON_PRESS: 
            self.iter = self.view['tv_phase_op_mode_combination'].get_selection().get_selected()[1]
            parent = self.model.get_phase_op_mode_combinations_tree().iter_parent(self.iter)

            if (self.iter is not None) and (parent is None):
                self.open_edit_association_window() 
            
            if (self.iter is not None) and (parent is not None):
                self.open_edit_definition_window()
            else: return
                  
        return 


    def on_button_edit_definition_clicked(self, button):
        """Edits the definition selected in the list.
           @param   button:  The Edit Button (self.view['button_edit_definition']).
	   @type    button:  gtk.Button
        """
        self.open_edit_definition_window()
        return


    def on_tv_phase_op_mode_combination_cursor_changed(self, treeview):
        """Changes the phase_op_mode_combination cursor selected.
           @param   treeview:  The Tree View of the phase_op_mode_combination (self.view['tv_phase_op_mode_combination']).
	   @type    treeview:  gtk.TreeView
        """
        self.iter = treeview.get_selection().get_selected()[1]

        if self.iter is not None:
            parent = self.model.get_phase_op_mode_combinations_tree().iter_parent(self.iter)

            """The selection of phase/op-mode combinations has been changed.
            IFF one phase is selected, button "Edit Association" must be
            sensitive"""
            self.view['button_edit_association'].set_sensitive(parent is None)
            
            """Otherwise, the "Edit Definition" button has to become enabled
            when one phase/op-mode combination is selected."""
            self.view['button_edit_definition'].set_sensitive(not parent is None)
        else:
            self.view['button_edit_association'].set_sensitive(False)
            self.view['button_edit_definition'].set_sensitive(False)

        return


    def phase_op_mode_combinations_busy_value_change(self, model, old, new):
	"""Busy level, in this case, is 0 if AUTOGEF is not running and
           1 otherwise. If AUTOGEF is active, all the informations contained
           in the "Mission" tab cannot be edited. This could otherwise
           lead to misinterpretations after AUTOGEF lead to a result.
           @param   model:  The Mission Model.
	   @type    model:  MissionModel
           @param   old:  The number of old value.
	   @type    old:  int
           @param   new:  The number of new value.
	   @type    new:  int
        """
	if model is self: return # not interested in introspection

	self.view['button_edit_association'].set_sensitive(new == 0)
	self.view['button_edit_definition'].set_sensitive(new == 0)
	return


    def property_phase_op_mode_combinations_tree_changed_signal_emit(self, model, args):
        """When a change occured in the PhaseOpModeCombinations, a signal celar the args selected.
           @param   model:  The PhaseOpModeCombinations Model.
	   @type    model:  PhaseOpModeCombinationsModel
           @param   args:  The arguments of PhaseOpModeCombinations Model.
	   @type    args:  object
        """
        self._clear_selections()
        return

pass # end of PhaseOpModeCombinationsCtrl class
# ----------------------------------------------------------------------


class PhaseOpModeCombinationEditAssociationDialogCtrl(Controller):
    """Represents the Dialog Class of PhaseOpModeCombinationEditAssociation"""

    def __init__(self, model, view, loading_phase_op_mode_combinations=False):
        """Constructor Method"""
        Controller.__init__(self, model, view)
        self.loading_phase_op_mode_combinations = loading_phase_op_mode_combinations

        model.register_observer(self)
        return


    def register_view(self, view):
        """Sets the windows, dialog, buttons and other parts of View."""
        tv = view['edit_association_tv_associated_opm']
        tv.set_model(self.model.get_associated_op_modes())
        tv.append_column(self.model.get_associated_op_modes_col())

        tv = view['edit_association_tv_not_associated_opm']
        tv.set_model(self.model.get_not_associated_op_modes())
        tv.append_column(self.model.get_not_associated_op_modes_col())

        # to control selection of associated op-modes
        sel = self.view['edit_association_tv_associated_opm'].get_selection()
        sel.set_mode(gtk.SELECTION_MULTIPLE)
        sel.connect('changed',
                    self.on_associated_opm_selection_changed)

        # to control selection of not associated op-modes
        sel = self.view['edit_association_tv_not_associated_opm'].get_selection()
        sel.set_mode(gtk.SELECTION_MULTIPLE)
        sel.connect('changed', 
                    self.on_not_associated_opm_selection_changed)

        view['edit_association_tv_associated_opm'].set_sensitive(True)
        view['edit_association_tv_not_associated_opm'].set_sensitive(True)
        view['button_edit_association_as_notas'].set_sensitive(False)
        view['button_edit_association_notas_as'].set_sensitive(False)
        view['button_edit_association_edit'].set_sensitive(True)
        view['button_edit_association_discard'].set_sensitive(True)

        return


    def register_adapters(self):
        return

    # -------------------------------------------------------------
    #         Signal handlers
    # -------------------------------------------------------------
    def _clear_selections(self):
        """Clears the old associations selected."""
        sel = self.view['edit_association_tv_associated_opm'].get_selection()
        sel.unselect_all()
        sel = self.view['edit_association_tv_not_associated_opm'].get_selection()
        sel.unselect_all()
        return


    def on_button_edit_association_edit_clicked(self, button):
        """Confirms the association of the PhasesOpModesCombinations selected in the list.
           @param   button:  The Confirm Button (self.view['button_edit_association_edit']).
	   @type    button:  gtk.Button
        """
        self.view['edit_association'].response(gtk.RESPONSE_APPLY)
        self.view['edit_association'].destroy()
        return


    def on_button_edit_association_discard_clicked(self, button):
        """Discards the association of the PhasesOpModesCombinations selected in the list.
           @param   button:  The Discard Button (self.view['button_edit_association_discard']).
	   @type    button:  gtk.Button
        """
        self.view['edit_association'].response(gtk.RESPONSE_CANCEL)
        self.view['edit_association'].destroy()
        return


    def on_button_edit_association_notas_as_clicked(self, button):
        """Transfers the association with not associated op_mode selected to the list 
           of association with associated op_modes.
           @param   button:  The Transfer Button (self.view['edit_association_tv_not_associated_opm']).
	   @type    button:  gtk.Button
        """
        sel = self.view['edit_association_tv_not_associated_opm'].get_selection()
        tree, paths = sel.get_selected_rows()
        self.model.notass_to_ass_op_modes_transfer(tree, paths)
        self._clear_selections()
        return


    def on_button_edit_association_as_notas_clicked(self, button):
        """Transfers the association with associated op_mode selected to the list 
           of association with not associated op_modes.
           @param   button:  The Transfer Button (self.view['edit_association_tv_associated_opm']).
	   @type    button:  gtk.Button
        """
        sel = self.view['edit_association_tv_associated_opm'].get_selection()
        tree, paths = sel.get_selected_rows()
        self.model.ass_to_notass_op_modes_transfer(tree, paths)
        self._clear_selections()
        return


    def on_associated_opm_selection_changed(self, selection):
        """Changes the association with associated op_mode of the PhasesOpModesCombinations selected.
           @param   selection:  The selection of association with associated op_mode.
	   @type    selection:  gtk.TreeSelection
        """
        self.view['button_edit_association_as_notas'].set_sensitive(
            selection.count_selected_rows() > 0)

        return


    def on_not_associated_opm_selection_changed(self, selection):
        """Changes the association with not associated op_mode of the PhasesOpModesCombinations selected.
           @param   selection:  The selection of association with not associated op_mode.
	   @type    selection:  gtk.TreeSelection
        """
        self.view['button_edit_association_notas_as'].set_sensitive(
            selection.count_selected_rows() > 0)

        return
    
pass # end of PhaseOpModeCombinationEditAssociationDialogCtrl class
# ----------------------------------------------------------------------


class PhaseOpModeCombinationEditDefinitionDialogCtrl(Controller):
    """Represents the Dialog Class of PhaseOpModeCombinationEditDefinition"""

    def __init__(self, model, view, loading_phase_op_mode_combinations=False):
        """Constructor Method"""
        Controller.__init__(self, model, view)
        self.loading_phase_op_mode_combinations = loading_phase_op_mode_combinations
	self.effect_valid = False
        return


    def register_view(self, view):
        """Sets the windows, dialog, buttons and other parts of View."""
        self.delay = DelayedCall(100, view.change_color_and_balloon)
        self.definition_correct = False

        definition = self.model.get_old_phase_op_mode_combination().get_definition()

        if not definition == "":
            self.view['entry_edit_definition'].set_text(definition)
            self.definition_correct = True

            pass

        view['entry_edit_definition'].connect(
            'changed', self.on_entry_edit_definition_changed)
	view['entry_edit_definition'].connect(
            "activate", self.on_activate_entry_effect)
        view['button_edit_definition_edit'].set_sensitive(self.definition_correct)

        return


    def register_adapters(self):
        return

    # -------------------------------------------------------------
    #         Signal handlers
    # -------------------------------------------------------------
    def on_button_edit_definition_edit_clicked(self, button):
        """Confirms the definition of the PhasesOpModesCombinations selected in the list.
           @param   button:  The Confirm Button (self.view['button_edit_definition_edit']).
	   @type    button:  gtk.Button
        """
        definition = \
            self.view['entry_edit_definition'].get_text().decode('utf_8')

        self.model.instantiate(definition)
        self.view['edit_definition'].response(gtk.RESPONSE_APPLY)
        self.view['edit_definition'].destroy()
        return


    def on_activate_entry_effect(self, editable):
        """Confirms the new definition selected in the list with the key ENTER.
           @param   editable:  The entry
	   @type    editable:  string
        """
	if self.effect_valid:
            # Simulate clicking of op_mode
            self.on_button_edit_definition_edit_clicked(None)
        return


    def on_button_edit_definition_discard_clicked(self, button):
        """Discards the definition of the PhasesOpModesCombinations selected in the list.
           @param   button:  The Discard Button (self.view['button_edit_definition_discard']).
	   @type    button:  gtk.Button
        """
        self.view['edit_definition'].response(gtk.RESPONSE_CANCEL)
        self.view['edit_definition'].destroy()
        return


    def on_entry_edit_definition_changed(self, entry):
        """Changes the new definition of the PhasesOpModesCombinations selected in the list.
           @param   entry:  The new entry definition
	   @type    entry:  string
        """
        if len(entry.get_text()) == 0:
            self.definition_correct = False
            self.view.change_color_and_balloon(entry, None)
            self.view['button_edit_definition_edit'].set_sensitive(False)
            return

        if len(entry.get_text()) > entry.get_width_chars():
            entry.set_width_chars(len(entry.get_text()))

        error_lst, impl_name = self._check_entry_syntax(entry)

        if not error_lst:
            self.definition_correct = True
            self.view.change_color_and_balloon(entry, None)
            self.view['button_edit_definition_edit'].set_sensitive(True)
	    self.effect_valid = True
        else:
            self.delay.change_color_and_balloon(entry, error_lst)
            self.definition_correct = False
            self.view['button_edit_definition_edit'].set_sensitive(False)

        return
        

    def _check_entry_syntax(self, entry):
        """Checks the errors of the new entries of Definitions.
           @param   entry:  The new entry name
	   @type    entry:  string
        """
        contents = entry.get_text()
        error_lst, impl_name = functions.get_errors_in_atomic_proposition_with_templates_observable(
            contents, self.model.mission_model.slim_model)
        return [l.__str__() for l in error_lst], impl_name
        
pass # end of PhaseOpModeCombinationEditDefinitionDialogCtrl class
# ----------------------------------------------------------------------
