# =============================================================================
#  This file is part of the ``HASDEL'' Toolset.
#  Copyright (C) 2014 by Fondazione Bruno Kessler
#  Copyright (C) 2014 by RWTH Aachen University
#
#
#  The HASDEL Toolset can be copied, modified and/or redistributed
#  under the terms of the HASDEL General Public License, either
#  version 1 of the License, or (at your option) any later version.
#
#  You should have received a copy of the HASDEL General Public
#  License along with this software. If not, please contact the
#  copyright owners.
#
#  The HASDEL Toolset is distributed in the hope that it will be
#  useful, but WITHOUT ANY WARRANTY; without even the implied warranty
#  of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  HASDEL General Public License for more details.
#
#  For more information on the HASDEL Project see the Project Web
#  site at <http://compassinternal.informatik.rwth-aachen.de/hasdel-site/> 
#  or email to <hasdel-support@lists.rwth-aachen.de>.  
#
#  Please report bugs to <hasdel-support@lists.rwth-aachen.de>.
# =============================================================================


import thread
import gobject
import gtk

from gtkmvc import Controller
import options

# base for controllers of activities
class ToolHandlersCtrl (Controller):
    """Controller for the list of processes with monitors"""

    def __init__(self, model, view):
        Controller.__init__(self, model, view)
        self.__enabled = False
        return

    def register_view(self, view):
        Controller.register_view(self, view)

        # sets up limits:
        if options.MEMOUT >= 0: 
            self.view['cb_mem_lim'].set_active(True)
            self.view['sb_mem_lim'].set_value(options.MEMOUT)
            pass

        if options.TIMEOUT >= 0: 
            self.view['cb_time_lim'].set_active(True)
            self.view['sb_time_lim'].set_value(options.MEMOUT)
            pass
        
        # sets up the treeview for processes:
        tv = self.view['tv_processes']
        tv.set_model(self.model.tools)
        
        rend = gtk.CellRendererText()
        col = gtk.TreeViewColumn('Name', rend)
        col.set_attributes(rend, text=0)
        tv.append_column(col)

        def __get_info(col, rend, model, iter, info_num):
            tool = model.get_value(iter, 1)
            if tool is None: 
                if info_num == 0: rend.set_property('text', "")
                else:
                    tot = 0
                    for row in model[iter].iterchildren():
                        tool = row[1]
                        tot += tool.get_status()[info_num]
                        pass
                    rend.set_property('text', tot)
                    pass
            else:
                info = tool.get_status()
                rend.set_property('text', info[info_num])
                pass
            return

        rend = gtk.CellRendererText()
        col = gtk.TreeViewColumn('Status', rend)
        col.set_cell_data_func(rend, __get_info, 0)
        tv.append_column(col)        

        rend = gtk.CellRendererText()
        col = gtk.TreeViewColumn('T. Time', rend)
        col.set_cell_data_func(rend, __get_info, 1)
        tv.append_column(col)        

        rend = gtk.CellRendererText()
        col = gtk.TreeViewColumn('P. Time', rend)
        col.set_cell_data_func(rend, __get_info, 2)
        tv.append_column(col)        

        rend = gtk.CellRendererText()
        col = gtk.TreeViewColumn('Mem (KB)', rend)
        col.set_cell_data_func(rend, __get_info, 3)
        tv.append_column(col)        

        rend = gtk.CellRendererText()
        col = gtk.TreeViewColumn('Peak (KB)', rend)
        col.set_cell_data_func(rend, __get_info, 4)
        tv.append_column(col)        
        return        

    def disable(self):
        """Disable the view"""
        self.__enabled = False
        self.view.show(False)
        return

    def enable(self):
        if self.__enabled: return

        self.__enabled = True
        self.view.show(True)

        def refresh_tv(self):
            self.model.tools.foreach(gtk.TreeModel.row_changed)
            return self.__enabled

        gobject.timeout_add(1000, refresh_tv, self)
        return

    # signals

    def on_bu_apply_limits_clicked(self, button):
        memout = self.view['sb_mem_lim'].get_value() if self.view['cb_mem_lim'].get_active() else -1
        timeout = self.view['sb_time_lim'].get_value() if self.view['cb_time_lim'].get_active() else -1
        self.model.set_tool_limits(memout, timeout)
        return

    def on_cb_mem_lim_toggled(self, cb): self.view['sb_mem_lim'].set_sensitive(cb.get_active())
    def on_cb_time_lim_toggled(self, cb): self.view['sb_time_lim'].set_sensitive(cb.get_active())

    pass #end of class
# ----------------------------------------------------------------------
