# =============================================================================
#  This file is part of the ``HASDEL'' Toolset.
#  Copyright (C) 2014 by Fondazione Bruno Kessler
#  Copyright (C) 2014 by RWTH Aachen University
#
#
#  The HASDEL Toolset can be copied, modified and/or redistributed
#  under the terms of the HASDEL General Public License, either
#  version 1 of the License, or (at your option) any later version.
#
#  You should have received a copy of the HASDEL General Public
#  License along with this software. If not, please contact the
#  copyright owners.
#
#  The HASDEL Toolset is distributed in the hope that it will be
#  useful, but WITHOUT ANY WARRANTY; without even the implied warranty
#  of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  HASDEL General Public License for more details.
#
#  For more information on the HASDEL Project see the Project Web
#  site at <http://compassinternal.informatik.rwth-aachen.de/hasdel-site/> 
#  or email to <hasdel-support@lists.rwth-aachen.de>.  
#
#  Please report bugs to <hasdel-support@lists.rwth-aachen.de>.
# =============================================================================


from gtkmvc import ModelMT


# Base activity model
class ActivityModel(ModelMT):
    """This is the base class for all models that need to access
    the slim and the properties models. These are all activities like
    Correctness, Safety, etc."""
    
    busy = 0 # busy level 
    __observables__ = ("busy", )

    def __init__(self, appl_model, slim_model, props_model):
        """Base constructor
        @param appl_model is the top-level model which is used to communicate status. 
        @slim slim_model is the SlimModel used to get all info needed about slim level
        @slim props_model is the PropertiesModel used to get all info needed about the properties
        """
        ModelMT.__init__(self)
        
        self.appl_model = appl_model
        self.slim_model = slim_model
        self.props_model = props_model
        return
    pass # end of class
# ----------------------------------------------------------------------
