# =============================================================================
#  This file is part of the ``HASDEL'' Toolset.
#  Copyright (C) 2014 by Fondazione Bruno Kessler
#  Copyright (C) 2014 by RWTH Aachen University
#
#
#  The HASDEL Toolset can be copied, modified and/or redistributed
#  under the terms of the HASDEL General Public License, either
#  version 1 of the License, or (at your option) any later version.
#
#  You should have received a copy of the HASDEL General Public
#  License along with this software. If not, please contact the
#  copyright owners.
#
#  The HASDEL Toolset is distributed in the hope that it will be
#  useful, but WITHOUT ANY WARRANTY; without even the implied warranty
#  of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  HASDEL General Public License for more details.
#
#  For more information on the HASDEL Project see the Project Web
#  site at <http://compassinternal.informatik.rwth-aachen.de/hasdel-site/> 
#  or email to <hasdel-support@lists.rwth-aachen.de>.  
#
#  Please report bugs to <hasdel-support@lists.rwth-aachen.de>.
# =============================================================================


import gtk

class Balloon(gtk.Window):
    """This class represents the tooltip balloon used in the COMPASS GUI.
    It provides methods to actively show and hide the the balloon.
    """
    
    def __init__(self, parent):
        gtk.Window.__init__(self)
        
        self.set_transient_for(parent)
        self.set_destroy_with_parent(True)
        self.set_decorated(False)
        self.set_accept_focus(False)
        self.set_property("skip-taskbar-hint", True)
        
        # TextView to display the messages.
        self.txtview = gtk.TextView()
        self.txtview.set_property("wrap-mode",gtk.WRAP_WORD)
        self.txtbuf = gtk.TextBuffer()
        self.txtview.set_buffer(self.txtbuf)
        self.add(self.txtview)
        color = gtk.gdk.color_parse('#f6f6b8')
        self.txtview.modify_base(gtk.STATE_NORMAL, color)
        
    def set_text(self, txt):
        """Sets the error message to be displayed via the balloon to txt"""
        self.txtbuf.set_text(txt)
        
    def change_color_and_balloon(self, entry, error_lst):
        """
        This method manages the entries' colors and displays a balloon with 
        error messages adjacent to the currently edited entry.
        
        @param   entry:  The reference to an entry that contains the input.
        @type    entry:  gtk.Entry
        @param   error_lst: List of errors, empty when there are none.
        @param   error_lst: list
        """
        
        if error_lst: # there are errors
            # calculate the balloon's position
            try:
                window_x, window_y = entry.get_parent_window().get_origin()
            except Exception:
                # in case the calling window was closed before the balloon
                # could be displayed, abort
                return
            bottom_left_x = entry.allocation.x + window_x
            bottom_left_y = entry.allocation.y + entry.allocation.height + \
                            window_y
            
            self.set_text("\n".join(error_lst))
            self.resize(400,30)
            self.show_all()
            #move after show_all, otherwise the move is ignored
            #for initial placement
            self.move(bottom_left_x, bottom_left_y)
            entry.modify_base(gtk.STATE_NORMAL, gtk.gdk.color_parse("indian red"))
        else: # there are no errors
            self.hide_all()
            entry.modify_base(gtk.STATE_NORMAL, gtk.gdk.color_parse("pale green"))
        pass
    
    pass #end of class Balloon
