# =============================================================================
#  This file is part of the ``HASDEL'' Toolset.
#  Copyright (C) 2014 by Fondazione Bruno Kessler
#  Copyright (C) 2014 by GMV Aerospace and Defence  
#  Copyright (C) 2014 by RWTH Aachen University
#
#
#  The HASDEL Toolset can be copied, modified and/or redistributed
#  under the terms of the HASDEL General Public License, either
#  version 1 of the License, or (at your option) any later version.
#
#  You should have received a copy of the HASDEL General Public
#  License along with this software. If not, please contact the
#  copyright owners.
#
#  The HASDEL Toolset is distributed in the hope that it will be
#  useful, but WITHOUT ANY WARRANTY; without even the implied warranty
#  of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  HASDEL General Public License for more details.
#
#  For more information on the HASDEL Project see the Project Web
#  site at <http://compassinternal.informatik.rwth-aachen.de/hasdel-site/> 
#  or email to <hasdel-support@lists.rwth-aachen.de>.  
#
#  Please report bugs to <hasdel-support@lists.rwth-aachen.de>.
# =============================================================================

import os.path
import options


def generate_unique_mission_basename():
    """Generates an unique basename for the mission_specification xml documents (.mxml).

    @return  base + "-" + str(num): The unique basename generated.
    @rtype   base + "-" + str(num): string
    """

    base = options.BASE_FILE_NAME        
    if base == "": base = "mission"
    if not os.path.exists(base + ".mxml"):
        return base
            
    num = 2    
    while os.path.exists(base + "-" + str(num) + ".mxml"):
        num += 1            

    return base + "-" + str(num)

pass # end of generate_unique_mission_basename
# -----------------------------------------------------------------------


def generate_unique_tfpg_xml_basename():
    """Generates an unique basename for the tfpg xml documents (.txml).

    @return  base + "-" + str(num): The unique basename generated.
    @rtype   base + "-" + str(num): string
    """

    base = options.BASE_FILE_NAME        
    if base == "": base = "tfpg"
    if not os.path.exists(base + ".txml"):
        return base
        
    num = 2    
    while os.path.exists(base + "-" + str(num) + ".txml"):
        num += 1         
    return base + "-" + str(num)

pass # end of generate_unique_tfpg_xml_basename
# -----------------------------------------------------------------------


def generate_unique_tfpg_human_readable_basename():
    """Generates an unique basename for the tfpg human_readable documents (.tfpg).

    @return  base + "-" + str(num): The unique basename generated.
    @rtype   base + "-" + str(num): string
    """

    base = options.BASE_FILE_NAME        
    if base == "": base = "tfpg"
    if not os.path.exists(base + ".tfpg"):
        return base
        
    num = 2    
    while os.path.exists(base + "-" + str(num) + ".tfpg"):
        num += 1         
    return base + "-" + str(num)

pass # end of generate_unique_tfpg_human_readable_basename
# -----------------------------------------------------------------------
