# =============================================================================
#  This file is part of the ``HASDEL'' Toolset.
#  Copyright (C) 2014 by Fondazione Bruno Kessler
#  Copyright (C) 2014 by RWTH Aachen University
#
#
#  The HASDEL Toolset can be copied, modified and/or redistributed
#  under the terms of the HASDEL General Public License, either
#  version 1 of the License, or (at your option) any later version.
#
#  You should have received a copy of the HASDEL General Public
#  License along with this software. If not, please contact the
#  copyright owners.
#
#  The HASDEL Toolset is distributed in the hope that it will be
#  useful, but WITHOUT ANY WARRANTY; without even the implied warranty
#  of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  HASDEL General Public License for more details.
#
#  For more information on the HASDEL Project see the Project Web
#  site at <http://compassinternal.informatik.rwth-aachen.de/hasdel-site/> 
#  or email to <hasdel-support@lists.rwth-aachen.de>.  
#
#  Please report bugs to <hasdel-support@lists.rwth-aachen.de>.
# =============================================================================


# set of functions for dumping files

# In Python 2.5, the "with" statement needs to be imported.
# (In Python 2.6, it is enabled by default.)
from __future__ import with_statement

import slim_compiler.src.smv.api as slim2smv_api
import property_management.src.api as properties_api

#from slim_compiler.src.errorManagement.errors import *
from invokers import slim
import options
from utils.misc import get_out_fname

from slim_compiler.src.dataStructure.visitors.printVisitor import toSLIMVisitor


def dump_ap_map_and_replace_aps(root_impl, formulae_doc, dinfo, override_out_dir=None):
    if formulae_doc is not None:
        proposition_to_label_dict = properties_api.create_proposition_to_label_dict(formulae_doc)
        
        for formula in formulae_doc:
            properties_api.replace_atomic_propositions(formula, proposition_to_label_dict)
        
        label_to_smv_proposition_dict = {}
        for (proposition, label) in proposition_to_label_dict.iteritems():
            smv_proposition = properties_api.get_atomic_proposition_in_smv_format(proposition, root_impl)
            label_to_smv_proposition_dict[label] = smv_proposition
            pass

        fname = get_out_fname(options.BASE_FILE_NAME + ".map", override_out_dir)
        dump_label_to_proposition_map(fname, label_to_smv_proposition_dict)
        
        dinfo.label_ap_map = open(fname).read()
        pass    
    return 

def dump_ap_map(root_impl, formulae_doc, dinfo, override_out_dir=None):
    if formulae_doc is not None:
        proposition_to_label_dict = properties_api.create_proposition_to_label_dict(formulae_doc)
        
        label_to_smv_proposition_dict = {}
        for (proposition, label) in proposition_to_label_dict.iteritems():
            smv_proposition = properties_api.get_atomic_proposition_in_smv_format(proposition, root_impl)
            label_to_smv_proposition_dict[label] = smv_proposition

        fname = get_out_fname(options.BASE_FILE_NAME + ".map", override_out_dir)
        dump_label_to_proposition_map(fname, label_to_smv_proposition_dict)
        
        dinfo.label_ap_map = open(fname).read()
    return

def dump_label_to_proposition_map(filename, label_to_proposition_dict):
    """Writes a file that specifies the mapping of MRMC-compatible labels to
    atomic propositions.
    
    Such a file can be used as input for the NuSMV-to-Sigref translator.
    Note: If the specified file does not exist, it will be created. Otherwise
    it will be overwritten.
    
    The content of the file will look like this:
    label1: atomic_proposition1;
    label2: atomic_proposition2;
    
    @param   filename:                  The name of the file to which the
                                        mapping shall be written.
    @type    filename:                  string
    @param   label_to_proposition_dict: A dictionary that maps MRMC-compatible
                                        labels to atomic propositions.
    @type    label_to_proposition_dict: dict (string -> string)
    """
    with open(filename, "w") as output_file:
        for (label, proposition) in label_to_proposition_dict.iteritems():
            output_file.write("%s: %s;\n" % (label, proposition))


def dump_event_name_to_label_map(filename, event_name_to_label_dict):
    """Writes a file that specifies the mapping of event names of a fault
    tree to MRMC-compatible labels.
    
    Such a file can be used as input for the DFT-to-MRMC translator.
    Note: If the specified file does not exist, it will be created. Otherwise
    it will be overwritten.
    
    The content of the file will look like this:
    event_name1: label1
    event_name2: label2
    
    @param   filename:                 The name of the file to which the
                                       mapping shall be written.
    @type    filename:                 string
    @param   event_name_to_label_dict: A dictionary that maps event names of a
                                       fault tree to MRMC-compatible labels.
    @type    event_name_to_label_dict: dict (string -> string)
    """
    output_file = open(filename, "w")
    try:
        for (event_name, label) in event_name_to_label_dict.iteritems():
            output_file.write("%s: %s\n" % (event_name, label))
    finally:
        output_file.close()


def dump_event_name_to_rate_map(filename, event_name_to_rate_dict):
    """Writes a file that specifies the mapping of event names of a fault
    tree to rates.
    
    Such a file can be used as input for the DFT-to-MRMC translator.
    Note: If the specified file does not exist, it will be created. Otherwise
    it will be overwritten.
    
    The content of the file will look like this:
    event_name1: 3.2
    event_name2: 0.7
    
    @param   filename:                The name of the file to which the
                                      mapping shall be written.
    @type    filename:                string
    @param   event_name_to_rate_dict: A dictionary that maps event names of a
                                      fault tree to rates.
    @type    event_name_to_rate_dict: dict (string -> string)
    """
    with open(filename, "w") as output_file:
        for (event_name, rate) in event_name_to_rate_dict.iteritems():
            output_file.write("%s: %s\n" % (event_name, rate))


def dump_error_event_probability_map(root_impl, dinfo, override_out_dir=None):
    fname = get_out_fname(options.BASE_FILE_NAME + ".epm", override_out_dir)
    ev_file = open(fname, "w")
    slim2smv_api.dump_error_event_probability_map(root_impl, ev_file)
    ev_file.close()
    
    ev_file = open(fname, "r")
    res = ""
    for line in ev_file:
        res += line + "\n"
    ev_file.close()
    dinfo.error_event_probability_map = res
    return


def dump_smv_language(root_impl, dinfo):
    """dumps smv language within a file"""
    fname = get_out_fname(options.BASE_FILE_NAME + ".lang")
    lang_file = open(fname, "w")
    res = ""
    for n,t in slim2smv_api.get_smv_language(root_impl).iteritems():
        res += n + " : " + t + "\n"
    lang_file.write(res)
    lang_file.close()
    dinfo.smv_language = res
    return


def dump_failures_observations_alarms(root_impl, dinfo):
    fname = get_out_fname(options.BASE_FILE_NAME + ".fail_obs_alm")
    fo_file = open(fname, "w")
    slim2smv_api.dump_failures_observations_alarms(root_impl, fo_file)
    fo_file.close()
    
    fo_file = open(fname, "r")
    res = ""
    for line in fo_file:
        res += line + "\n"
    fo_file.close()
    dinfo.failures_observations_alarms = res      
    return

def dump_fault_extended_slim_model(ext_compile_info, out_file_name):
    """Given the compile information extended after faults
    injections this function outputs the obtained SLIM model into a file.
    Note: generation of such files is purely for debugging purposes"""
    of = open(out_file_name, "w")
    of.write(toSLIMVisitor().visit(ext_compile_info))
    of.write("\n")
    of.close()
    pass
