#!/usr/bin/python

# =============================================================================
#  This file is part of the ``HASDEL'' Toolset.
#  Copyright (C) 2014 by Fondazione Bruno Kessler
#  Copyright (C) 2014 by RWTH Aachen University
#
#
#  The HASDEL Toolset can be copied, modified and/or redistributed
#  under the terms of the HASDEL General Public License, either
#  version 1 of the License, or (at your option) any later version.
#
#  You should have received a copy of the HASDEL General Public
#  License along with this software. If not, please contact the
#  copyright owners.
#
#  The HASDEL Toolset is distributed in the hope that it will be
#  useful, but WITHOUT ANY WARRANTY; without even the implied warranty
#  of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  HASDEL General Public License for more details.
#
#  For more information on the HASDEL Project see the Project Web
#  site at <http://compassinternal.informatik.rwth-aachen.de/hasdel-site/> 
#  or email to <hasdel-support@lists.rwth-aachen.de>.  
#
#  Please report bugs to <hasdel-support@lists.rwth-aachen.de>.
# =============================================================================


import sys
import os.path

top_dir = os.path.abspath(os.path.dirname(__file__))
antlr_dir = "../resources/antlr/antlr-python-runtime-3.1.1/"
sys.path.append(os.path.join(top_dir, antlr_dir))

import slim_compiler.src.smv.api as slim2smv_api
import property_management.src.api as properties_api

from invokers import dump, mrmc, nusmv, slim
from utils import debug_info as debug
from utils import log
from utils.misc import get_out_fname
import options

from runscripts.basic import ToolScript


class MainScript (ToolScript):

    __actions = ["smv-dumping", "to-sigref", "to-mrmc"]

    def __init__(self, derived_actions=[]):
        super(MainScript, self).__init__(derived_actions=self.__actions + derived_actions)
        return

    def action(self):
        super(MainScript, self).action()

        # dumps ap map
        dump.dump_ap_map_and_replace_aps(root_impl, formulae_doc, debug_info)

        # generate smv
        smv_file_name, infinite, timed = slim.convert_to_smv(root_impl)

        # store smv file in debug object
        debug_info.slim2smv_output = open(smv_file_name, "r").read()

        # error event-prob map
        dump.dump_error_event_probability_map(root_impl, debug_info)

        # smv language
        dump.dump_smv_language(root_impl, debug_info)

        # failure observation alarm
        dump.dump_failures_observations_alarms(root_impl, debug_info)

        # BDD ordering
        if not infinite: nusmv.launch_smv_to_bdd_ordering(get_out_fname(options.BASE_FILE_NAME + ".smv"))
        self.check_stop_request("smv-dumping")

        # sigref
        if formulae_doc is not None and len(formulae_doc):
            debug_info.smv2sigref_output = nusmv.launch_smv_to_sigref(
                get_out_fname(options.BASE_FILE_NAME + ".smv"),
                get_out_fname(options.BASE_FILE_NAME + ".epm"),
                ord_fname=get_out_fname(options.BASE_FILE_NAME + ".ord") if not infinite else None,
                label_fname=get_out_fname(options.BASE_FILE_NAME + ".map"),
                out_fname=get_out_fname(options.BASE_FILE_NAME + ".xml"),
                tau_name=slim2smv_api.get_top_tau_event())
        else:
            debug_info.smv2sigref_output = nusmv.launch_smv_to_sigref(
                get_out_fname(options.BASE_FILE_NAME + ".smv"),
                get_out_fname(options.BASE_FILE_NAME + ".epm"),
                ord_fname=get_out_fname(options.BASE_FILE_NAME + ".ord") if not infinite else None,
                out_fname=get_out_fname(options.BASE_FILE_NAME + ".xml"),
                tau_name=slim2smv_api.get_top_tau_event())
            pass

        self.check_stop_request("to-sigref")

        # sigref2mrmx
        debug_info.sigref2mrmc_output = mrmc.launch_sigref_to_mrmc(get_out_fname(options.BASE_FILE_NAME + ".xml"))
        self.check_stop_request("to-mrmc")

        # mrmc
        if formulae_doc:
            formulae_strings = [properties_api.get_formula_as_mrmc_string(formula)
                                for formula in formulae_doc]
            debug_info.formulae_strings = formulae_strings

            mrmc_output = mrmc.launch_mrmc(get_out_fname(options.BASE_FILE_NAME + ".tra"),
                                           get_out_fname(options.BASE_FILE_NAME + ".lab"),
                                           formulae_strings)
            debug_info.mrmc_output = mrmc_output
            pass

        log.system.info("Successfully ran main script.")
        return

    pass  # end of class MainScript


if __name__ == "__main__":
    main = MainScript()

    try:
        main.run()
    except Exception as e:
        log.system.error("Unexpected error.")
        debug.emit_error_and_quit(e, main.get_debug_info())
