# =============================================================================
#  This file is part of the ``HASDEL'' Toolset.
#  Copyright (C) 2014 by Fondazione Bruno Kessler
#  Copyright (C) 2014 by RWTH Aachen University
#
#
#  The HASDEL Toolset can be copied, modified and/or redistributed
#  under the terms of the HASDEL General Public License, either
#  version 1 of the License, or (at your option) any later version.
#
#  You should have received a copy of the HASDEL General Public
#  License along with this software. If not, please contact the
#  copyright owners.
#
#  The HASDEL Toolset is distributed in the hope that it will be
#  useful, but WITHOUT ANY WARRANTY; without even the implied warranty
#  of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  HASDEL General Public License for more details.
#
#  For more information on the HASDEL Project see the Project Web
#  site at <http://compassinternal.informatik.rwth-aachen.de/hasdel-site/> 
#  or email to <hasdel-support@lists.rwth-aachen.de>.  
#
#  Please report bugs to <hasdel-support@lists.rwth-aachen.de>.
# =============================================================================


# This module contains global options for slim2mrmc processes This
# needs to be restructured in a more global vision at the moment
# this module is a simple placeholder keeping all options in one
# place

import sys
import os.path

# ----------------------------------------------------------------------
# SLIM2MRMC
# ----------------------------------------------------------------------

PROG_NAME = ""
PROG_CMDLINE = ""
PROG_VER = (2,3,0)

# Non-flag program arguments
POSITIONAL_ARGS = []

# Names of input slim files
SLIM_FILE_NAMES = []

# Name of output smv file
BASE_FILE_NAME = ""

# Name of failure injection file
FAULT_INJECTION_FILE_NAME = ""

# A flag to output a fault extended model. Used for debugging only.
OUTPUT_FAULT_EXTENDED = False

# A flag to steer the parser in accepting extended models. Used for debugging only.
PARSER_ACCEPT_EXTENDED_MODEL = False

# Name of properties file
PROPERTIES_FILE_NAME = ""

# Name of the root implementation
ROOT_NAME = ""

# default logging level
LOGGING_SYS_LEVEL = ""
LOGGING_TEST_LEVEL = ""
LOGGING_USER_LEVEL = ""
LOGGING_FNAME = ""

# To stop after parsing, checking, fault injection, smv dumping,
# NuSMV to sigref translation, sigref to mrmc translation, etc.
# This is used mainly for debugging
STOP_AFTER = ""

# Perform step-wise analysis of the properties
STEPWISE = False
STEPWISE_GRANULARITY = 10

# To enable the output of subprocess calls
DISPLAY_CALL_OUTPUT = False

# ----------------------------------------------------------------------
# Directories
# ----------------------------------------------------------------------
top_dir = os.path.dirname(__file__)

# top level directory, the root for all sources
TOP_LEVEL_DIR = top_dir

# place for resources
RESOURCES_DIR = os.path.join(TOP_LEVEL_DIR, "../resources")

# place for binary external executable files
BIN_DIR = os.path.join(TOP_LEVEL_DIR, "bin")

# Name of a directory for output and logging files. If the empty
# string is given, the basename of the slim file will be used.  If
# empty and if a directory with the basename already exists, a
# numeric suffix will be added.
OUT_DIR_NAME = ""

# ----------------------------------------------------------------------
# Model-checkers to be used
# ----------------------------------------------------------------------

FORCED_MC = ""
MODEL_CHECKERS = ["mrmc","imca"]
AVAILABLE_MODEL_CHECKERS = ["mrmc","imca"]

# ----------------------------------------------------------------------
# External tools
# ----------------------------------------------------------------------
SMV2SIGREF = "NuSMV"
DEGRADED_NUSMV = "NuSMV"
NUSMV = "NuSMV4Compass"
ORDER_BITS = "order_bits.pl"
MRMC = "mrmc"
FTD = "ftd"
IMCA = "imca"
MODEL2IMC = "model2imc"
SLIMSIM = "slimsim"

ERROR_BOUND = 0.001
MEMOUT = -1  # expressed in MBytes (-1 means no limit)
TIMEOUT = -1 # expressed in seconds (-1 means no limit)


# ----------------------------------------------------------------------
# Runscripts
# ----------------------------------------------------------------------
CLOCK_BOUND_FNAME = None  # filename to read bounds from
GEN_CLOCK_BOUND = False  # if generate clock bounds file


# ----------------------------------------------------------------------
# SMV Specific
# ----------------------------------------------------------------------

# Size of words generated when encoding slim integers:
SMV_WORD_SIZE = 32

# use signed or unsigned words
SMV_WORD_SIGNED = True

# Enable smv file to contain debugging specifications
SMV_ENABLE_DEBUG = False

# Enable smv->slim name conversions of 'internal' names
# Note: this is needed by transition-based guided simulation
SMV_ENABLE_INTERNAL_SLIM_NAMES = True

# If time-transitions are forced to occur in a fair loop
SMV_FAIR_TIMED = True

# generate time variable
SMV_GENERATE_TIME_VAR = False


# ----------------------------------------------------------------------
# NuSMV Specific
# ----------------------------------------------------------------------
# By default, use SAT-based Bounded Model Checking for LTL formulae
NUSMV_USE_BMC = False
NUSMV_BMC_USE_SBMC = True
NUSMV_SBMC_IS_COMPLETE = False
NUSMV_BMC_LENGTH = 10  # length (k) of BMC
NUSMV_SIM_LENGTH = 10  # length of simulation
NUSMV_KLIVE_BOUND = 10  # bound for k-liveness search
NUSMV_CLOCK_BOUND = "10.0 hour"  # bound for clock-divergence search


# ----------------------------------------------------------------------
# FSAP Specific
# ----------------------------------------------------------------------
FSAP_FAULT_CONFIGURATION_CARDINALITY = 1
FSAP_KEEP_SMV_IDS = False
FSAP_PROPERTY_NUMBER = 0
FSAP_USE_DYNAMIC = False

# change to True to have compact FMEA tables when possible (i.e. no
# dynamic table is requested)
FSAP_COMPACT_FMEA_TABLE = False

# Only for fault tree generation runscript:
FSAP_OUTPUT_FAULT_TREE_XML_AND_RATES = False


# ----------------------------------------------------------------------
# FDIR Specific
# ----------------------------------------------------------------------
PFDIR_ERROR_STATE = None
PFDIR_OBSERVABLE = None
PFDIR_UPPER_TIME_BOUND = 0

DIAG_DELAY = -1.0

# ----------------------------------------------------------------------
# DFT Specific
# ----------------------------------------------------------------------
SEVERITY = None


# ----------------------------------------------------------------------
# Hasdel Specific
# ----------------------------------------------------------------------
MISSION_FILE_NAME = ""
TFPG_FILE_NAME = ""
TFPG_ASSOC_FILE_NAME = ""
MC_CONFIDENCE=0.99
MC_ERRORBOUND=0.05
MC_STEPBOUND=10000
MC_STRATEGY='asap'
MC_IGNOREDEADLOCK=True
MC_IGNORESTEPBOUND=False
MC_THREADS=8
