# =============================================================================
#  This file is part of the ``HASDEL'' Toolset.
#  Copyright (C) 2014 by Fondazione Bruno Kessler
#  Copyright (C) 2014 by RWTH Aachen University
#
#
#  The HASDEL Toolset can be copied, modified and/or redistributed
#  under the terms of the HASDEL General Public License, either
#  version 1 of the License, or (at your option) any later version.
#
#  You should have received a copy of the HASDEL General Public
#  License along with this software. If not, please contact the
#  copyright owners.
#
#  The HASDEL Toolset is distributed in the hope that it will be
#  useful, but WITHOUT ANY WARRANTY; without even the implied warranty
#  of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  HASDEL General Public License for more details.
#
#  For more information on the HASDEL Project see the Project Web
#  site at <http://compassinternal.informatik.rwth-aachen.de/hasdel-site/> 
#  or email to <hasdel-support@lists.rwth-aachen.de>.  
#
#  Please report bugs to <hasdel-support@lists.rwth-aachen.de>.
# =============================================================================


"""This module provides the classes to represent errors in formulae documents,
properties documents and patterns documents.

Note: These errors are errors that are detected when documents are validated.
They are not intended to be raised as exceptions.
"""

class DocumentSyntaxError(object):
    """An object representing a syntax error in a formulae document, a
    properties document or a patterns document.
    
    Syntax errors are detected by validating the document against an XML
    Schema Definition.
    """
    
    def __init__(self, error_description):
        """Initializes the DocumentSyntaxError object.
        
        @param   error_description: The description of the syntax error.
        @type    error_description: string
        """
        self.description = error_description
        
    def __str__(self):
        """Returns a string representation of this syntax error.
        
        @return: The description of this syntax error.
        @rtype:  string
        """
        return self.description


class InvalidAtomicPropositionError(object):
    """An object representing an error in an atomic proposition."""
    
    def __init__(self, error_description):
        """Initializes the InvalidAtomicPropositionError object.
        
        @param   error_description: The description of the error.
        @type    error_description: string
        """
        self.description = error_description
        
    def __str__(self):
        """Returns a string representation of this error.
        
        @return: The description of this error.
        @rtype:  string
        """
        return self.description
