##  ===========================================================================
##  Author: Roberto Cavada <cavada@fbk.eu>
##
##  Copyright (c) 2005 by FBK 
##
##  This is free software; you can redistribute it and/or
##  modify it under the terms of the GNU Lesser General Public
##  License as published by the Free Software Foundation; either
##  version 2 of the License, or (at your option) any later version.
##
##  This software is distributed in the hope that it will be useful,
##  but WITHOUT ANY WARRANTY; without even the implied warranty of
##  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
##  Lesser General Public License for more details.
##
##  You should have received a copy of the GNU Lesser General Public
##  License along with this library; if not, write to the Free Software
##  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA.
##
##  For more information about this software see <http://rat.fbk.eu>
##  or email to the RAT Team <rat@fbk.eu>.
##  Please report bugs to <rat@fbk.eu>.
##
##  ===========================================================================

# to test available requirments
import pygtk
pygtk.require("2.0")

# to link documentation
import src

def setup_path():
    """Sets up the python include paths"""
    import os.path; import sys

    if sys.argv[0]:
        top_dir = os.path.dirname(os.path.abspath(sys.argv[0]))
        sys.path = [os.path.join(top_dir, "src")] + sys.path
        pass
    return

def setup_resources():
    """Import application's icons and other resources"""
    import src.utils.application
    src.utils.application.load_resources()
    return

def main(project=None):
    """Top level launcher. Instantiates and connects the application
    model, controller and view, and starts the gtk events loop. If a
    project was requested to be loaded from the command line,
    instantiates a new project and loaqds it into the application."""

    from models.application import ApplicationModel as Model
    from controllers.application import ApplicationCtrl as Ctrl
    from views.application import ApplicationView as View
    
    m = Model()
    c = Ctrl(m)
    v = View(c)

    if project:
        c.create_new_project()
        m.project.open(project)
        pass

    import gtk
    gtk.main()
    return

def print_banner():
    import os.path
    import sys
    
    print_version()
    print "Usage:"
    print "$> python", os.path.basename(sys.argv[0]), \
          "[-h|--help] [-v|--version] [-f <FILE.rat>|--project=<FILE.rat>]"
    return

def print_version():
    from src.utils.application import get_version_string
    print "RAT - %s" % get_version_string()
    return

def print_usage():
    print_banner()
    print "OPTIONS:"
    print "-h\t\t Prints this message"
    print "-v\t\t Prints the program version"
    print "-f <FILE.rat>\t Loads the given project file"
    print
    return

def process_options():
    """For command line options processing"""
    import sys
    import getopt
    import os.path
    
    try: opts, args = getopt.getopt(sys.argv[1:], "hvf:",
                                    ["help", "version", "project="])
    except getopt.GetoptError:
        print "Error: invalid arguments. See --help for valid options."
        sys.exit(2)
        pass

    # options processing:
    res = None
    for o,a in opts:
        if o in ("-h", "--help"): print_usage(); sys.exit()
        if o in ("-v", "--version"): print_version(); sys.exit()
        if o in ("-f", "--project") and res is None: res = a
        pass

    if res and not os.path.isfile(res):
        print "Error: RAT project file '%s' not found"
        sys.exit(1)
        pass

    return res

if __name__ == "__main__":
    setup_path()
    project = process_options()
    setup_resources()
    main(project)
    pass
