##  ===========================================================================
##  Authors: Roberto Cavada <cavada@fbk.eu>
##           Ingo Pill <ipill@ist.tugraz.at>
##
##  Copyright (c) 2005 by FBK
##  Copyright (c) 2005 by Graz University of Technology
##
##  This is free software; you can redistribute it and/or
##  modify it under the terms of the GNU Lesser General Public
##  License as published by the Free Software Foundation; either
##  version 2 of the License, or (at your option) any later version.
##
##  This software is distributed in the hope that it will be useful,
##  but WITHOUT ANY WARRANTY; without even the implied warranty of
##  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
##  Lesser General Public License for more details.
##
##  You should have received a copy of the GNU Lesser General Public
##  License along with this library; if not, write to the Free Software
##  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA.
##
##  For more information about this software see <http://rat.fbk.eu>
##  or email to the RAT Team <rat@fbk.eu>.
##  Please report bugs to <rat@fbk.eu>.
##
##  ===========================================================================

from gtkmvc.controller import Controller
import views.trace

import gtk

# ----------------------------------------------------------------------
class TraceCtrl (Controller):
    """This is the controller for a single trace."""

    def __init__(self, model):
        Controller.__init__(self, model)

        # colours for cell background
        self.bg = "#F0F0F0"
        self.bg_loop = "#D0D0D0"
        self.bg_sel = "#E0E0E0"
        self.bg_loop_sel = "#2020F0"
        self.tips = gtk.Tooltips()
        self.tips.enable()
        return

    def cell_name_func(self, col, cell, model, iter):
        stripe = model.get_value(iter, 0)
        cell.set_property("text", stripe.get_name())
        return

    def cell_data_func(self, col, cell, model, iter, args):
        """@param args is (treeview, step_num)"""
        
        tv, step = args
        stripe = model.get_value(iter, 0)

        cell.set_property("is-boolean", stripe.is_boolean())
        cell.set_property("value", stripe.get_value_at_step(step))
        
        if step > 0: cell.set_property("prev-value", stripe.get_value_at_step(step-1))
        else: cell.set_property("prev-value", None)
        
        if step < stripe.get_steps()-1:
            cell.set_property("next-value", stripe.get_value_at_step(step+1))
        else: cell.set_property("next-value", None)

        # background colours:
        if tv.get_selection().iter_is_selected(iter):
            if model.is_step_in_loop(step+1): cell.set_property("cell-background", self.bg_loop_sel)
            else: cell.set_property("cell-background", self.bg_sel)
        elif model.is_step_in_loop(step+1): cell.set_property("cell-background", self.bg_loop)
        else: cell.set_property("cell-background", self.bg)
        return
                      

    def registerView(self, view):
        Controller.registerView(self, view)

        self.setup_view(view)
        return

    def setup_view(self, view):        
        tv = view['treeview']

        tv.set_model(self.model)

        rend = gtk.CellRendererText()
        col = gtk.TreeViewColumn("Name", rend)
        col.set_cell_data_func(rend, self.cell_name_func)
        col.set_resizable(True)
        tv.append_column(col)

        for i in range(self.model.get_max_steps()):
            rend = views.trace.CellRendererTrace()

            rend.set_property("cell-background", self.bg)
            rend.set_property("waveform-fg-color", "dark green")
            rend.set_property("waveform-sel-fg-color", "light green")
            rend.set_property("bool-waveform-width", 3)
            
            lbl = gtk.Label("Step%d" % (i+1))
            if self.model.is_step_a_loop(i+1):
                # shows an arrow to explicitate that loop begins at that step:
                wid = gtk.HBox()
                ar = gtk.Arrow(gtk.ARROW_DOWN, gtk.SHADOW_IN); ar.show()
                self.tips.set_tip(ar, "A loop starts at this step")
                self.tips.set_tip(wid, "A loop starts at this step")
                wid.add(ar)
                lbl.show()
                wid.add(lbl)                
            else: wid = lbl

            wid.show()

            col = gtk.TreeViewColumn()
            col.set_widget(wid)
            col.pack_start(rend)
            col.set_cell_data_func(rend, self.cell_data_func, (tv, i))            
            tv.append_column(col)
            pass

        return
 
    pass # end of class

class TraceCtrlPS (Controller):
    """This is the controller for a single trace."""

    def __init__(self, model, title="Signal", editable=False, parentCtrl=None):
        Controller.__init__(self, model)
	self.title=title
	self.editable=editable
	self.parentCtrl=parentCtrl
	self.DNC=model
        return

    def set_title(self, title):
        self.title=title
	return	
	
    def get_title(self):
        return self.title
	
    def set_editable(self, editable):
        self.editable=editable
	return

    def is_editable(self):
        return self.editable
	
    def set_parent_Ctrl(self, parent_Ctrl):
        self.parentCtrl=parent_Ctrl
	return

    def get_parent_Ctrl(self):
        return self.parentCtrl
    
    def cell_name_func(self, col, cell, model, iter):
        stripe = model.get_value(iter, 0)
        cell.set_property("text", stripe.get_name())
	return

    def cell_data_func(self, col, cell, model, iter, step):
        stripe = model.get_value(iter, 0)

        cell.set_property("is-boolean", stripe.is_boolean())
        cell.set_property("value", stripe.get_value_at_step(step)[0])
        
        if step > 0: cell.set_property("prev-value", stripe.get_value_at_step(step-1)[0])
        else: cell.set_property("prev-value", None)
        
        if step < stripe.get_steps()-1:
            cell.set_property("next-value", stripe.get_value_at_step(step+1)[0])
        else: cell.set_property("next-value", None)
		
	if ((stripe.get_value_at_step(step)[1])==0):
	  cell.set_property("waveform-fg-color", "dark red")
	  cell.set_property("waveform-sel-fg-color", "red")
	elif ((stripe.get_value_at_step(step)[1])==2):
	  cell.set_property("waveform-fg-color", "black")
	  cell.set_property("waveform-sel-fg-color", "white")  
	else:
	  cell.set_property("waveform-fg-color", "dark green")
	  cell.set_property("waveform-sel-fg-color", "green")
	
        return
                     

    def registerView(self, view):
        Controller.registerView(self, view)

        self.setup_view(view)
        return


    def setup_view(self, view):        
        tv = view['treeview']

        tv.set_model(self.model)

        rend = gtk.CellRendererText()
        col = gtk.TreeViewColumn(self.title, rend)
        col.set_cell_data_func(rend, self.cell_name_func)
	col.set_property("expand", True)
	# for PS this column is not resizable, but the one which expands
        tv.append_column(col)

        for i in range(self.model.get_max_steps()):
            rend = views.trace.CellRendererTrace()

            # for PS we have only one loop and distinct with the bg color
	    # between the finite and infinite part
	    if i<self.model.loops[0]:
	      rend.set_property("cell-background", "#C0C0C0")
	    else:
              rend.set_property("cell-background", "#909090")
            #rend.set_property("frame-fg-color", "dark red")
            rend.set_property("waveform-fg-color", "dark green")
            rend.set_property("waveform-sel-fg-color", "green")
        
            col = gtk.TreeViewColumn("Step%d" % (i+1), rend)
            col.set_cell_data_func(rend, self.cell_data_func, i)            
            tv.append_column(col)
            pass

        return
	

    def update_view(self):  # added by Ipill      
        tv = self.view['treeview']

        # remove waveform
        for i in tv.get_columns():
	  tv.remove_column(i)
	  
	rend = gtk.CellRendererText()
        col = gtk.TreeViewColumn(self.title, rend)
        col.set_cell_data_func(rend, self.cell_name_func)
	col.set_property("expand", True)
	# for PS this column is not resizable, but the one which expands
        tv.append_column(col)
		
        for i in range(self.model.max_loop):
            rend = views.trace.CellRendererTrace()
            # for PS we have only one loop and distinct with the bg color
	    # between the finite and infinite part
	    if i<self.model.loops[0]:
	      rend.set_property("cell-background", "#C0C0C0")
	    else:
              rend.set_property("cell-background", "#909090")
            #rend.set_property("frame-fg-color", "dark red")
            rend.set_property("waveform-fg-color", "dark green")
            rend.set_property("waveform-sel-fg-color", "green")
        
            col = gtk.TreeViewColumn("Step%d" % (i+1), rend)
            col.set_cell_data_func(rend, self.cell_data_func, i)            
            tv.append_column(col)
            pass

        return
   
    def edit_menu(self, pathinfo):
        menu = gtk.Menu()

        item = gtk.MenuItem("Insert timestep")
        item.connect("activate", self.parentCtrl.insertTimeStep, pathinfo )
        item.show()
        menu.append(item)

        item = gtk.MenuItem("Remove timestep")
        item.connect("activate", self.parentCtrl.removeTimeStep, pathinfo )
        item.show()
        menu.append(item)

        item = gtk.MenuItem("Fix value to False")
        item.connect("activate", self.parentCtrl.setSignalValueToFalse, pathinfo)
        item.show()
        menu.append(item)

        item = gtk.MenuItem("Fix value to True")
        item.connect("activate", self.parentCtrl.setSignalValueToTrue, pathinfo)        
        item.show()
        menu.append(item)

        item = gtk.MenuItem("Set to 'Do not care'")
        item.connect("activate", self.parentCtrl.setSignalValueToDNC, pathinfo)        
        item.show()
        menu.append(item)

        return menu

    def signal_menu(self, pathinfo):
        menu = gtk.Menu()
	
        item = gtk.MenuItem("Fix current values of this signal")
        item.connect("activate", self.parentCtrl.fixSignalValues, pathinfo )
        item.show()
        menu.append(item)

        item = gtk.MenuItem("Fix all values of this signal to False")
        item.connect("activate", self.parentCtrl.setSignalToFalse, pathinfo )
        item.show()
        menu.append(item)

        item = gtk.MenuItem("Fix all values of this signal to True")
        item.connect("activate", self.parentCtrl.setSignalToTrue, pathinfo )
        item.show()
        menu.append(item)

        item = gtk.MenuItem("Set all values of this signal to 'Do not care'")
        item.connect("activate", self.parentCtrl.setSignalToDNC, pathinfo )
        item.show()
        menu.append(item)
    
        return menu

    def on_treeview_button_press_event(self, treeview, event):
        
        #self.edit_entry_at(self.model.get_iter(pthinfo[0]))
        #return True
        
	if ((event.button == 3) & (self.editable==True)):
	    pathinfo = treeview.get_path_at_pos(int(event.x), int(event.y))
	    	    
	    if (pathinfo is None): return False
	    if (pathinfo[1]==self.view['treeview'].get_column(0)): 
	      menu = self.signal_menu(pathinfo)
	      menu.popup(None, None, None, event.button, event.time)
	    else:
	      menu = self.edit_menu(pathinfo)
              menu.popup(None, None, None, event.button, event.time)

            return True

        return False        

 
    pass # end of class

