##  ===========================================================================
##  Author: Roberto Cavada <cavada@fbk.eu>
##
##  Copyright (c) 2005 by FBK 
##
##  This is free software; you can redistribute it and/or
##  modify it under the terms of the GNU Lesser General Public
##  License as published by the Free Software Foundation; either
##  version 2 of the License, or (at your option) any later version.
##
##  This software is distributed in the hope that it will be useful,
##  but WITHOUT ANY WARRANTY; without even the implied warranty of
##  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
##  Lesser General Public License for more details.
##
##  You should have received a copy of the GNU Lesser General Public
##  License along with this library; if not, write to the Free Software
##  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA.
##
##  For more information about this software see <http://rat.fbk.eu>
##  or email to the RAT Team <rat@fbk.eu>.
##  Please report bugs to <rat@fbk.eu>.
##
##  ===========================================================================

from gtkmvc.model import Model

from models.project import ProjectModel
import models.options

import utils.globals

import os.path


# ----------------------------------------------------------------------
class ApplicationModel (Model):
    """The top-level model, representing the application's main model"""

    # ------------------------- #
    #   Observable properties   #
    # ------------------------- #

    __properties__ = {
        "is_busy" : False, # a stubbed tool is running in the background
        "traces_window_visible" : False, # Traces windows is currently shown
        }
    
    def __init__(self):
        Model.__init__(self)

        # initially, an empty, new model exists
        self.project = ProjectModel(self)
        self.project.registerObserver(self) # self observers the project model

        # the project options model
        self.options = models.options.ApplicationOptionsModel(utils.globals.APPL_OPTIONS_FILE)
        return

    def new_project(self):
        if self.project: self.project.quit()

        self.project = ProjectModel(self)
        self.project.registerObserver(self) # self observers the project model
        return

    def quit(self):
        """Called when it is time to quit the application"""
        if self.project: self.project.quit()
        return

    # ---------------------------------------- #
    #   Notifications for observed properties  #
    # ---------------------------------------- #

    def property_is_project_busy_change_notification(self, model, old, new):        
        if old != new:
            if new and not self.is_busy: self.is_busy = True
            elif not new and self.is_busy: self.is_busy = False
            pass
        return

    def property_traces_window_shown_change_notification(self, model, old, new):
        """The project's property that decides whether the project has
        a traces window visible"""
        if new != old and self.traces_window_visible != new:
            self.traces_window_visible = new
            pass
        return
        


    pass # end of class


class ApplicationWizardModel (Model):
    """The new project wizard model"""
    
    def __init__(self):
        Model.__init__(self)

        self.types = {
            'pa' : "<i>Property Assurance</i> allows for analyzing requirements as a whole (e.g. satisfibility, entailment, ...).",
            'ps' : "<i>Property Simulation</i> allows for inspecting admissible and not admissible behaviors of a set of requirements.",
            're' : "<i>Realizability</i> allows for checking whether a set of requirements admits an implementation.",
            }           
        
        return

    def get_description(self, key):
        if not self.types.has_key(key): return ""
        return self.types[key]

    pass # end of class

