##  ===========================================================================
##  Author: Roberto Cavada <cavada@fbk.eu>
##
##  Copyright (c) 2005 by FBK 
##
##  This is free software; you can redistribute it and/or
##  modify it under the terms of the GNU Lesser General Public
##  License as published by the Free Software Foundation; either
##  version 2 of the License, or (at your option) any later version.
##
##  This software is distributed in the hope that it will be useful,
##  but WITHOUT ANY WARRANTY; without even the implied warranty of
##  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
##  Lesser General Public License for more details.
##
##  You should have received a copy of the GNU Lesser General Public
##  License along with this library; if not, write to the Free Software
##  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA.
##
##  For more information about this software see <http://rat.fbk.eu>
##  or email to the RAT Team <rat@fbk.eu>.
##  Please report bugs to <rat@fbk.eu>.
##
##  ===========================================================================


from gtkmvc.model import Model

import utils.globals

import gtk
import gobject

class CheckingPanelModel (Model):

    """This is the model of the checking panel. One instance at the
    moment is embedded within the property assurance window."""


    # -------------------------------------- #
    #      Set of observable properties      #
    # -------------------------------------- #    
    __properties__ = {
        'busy_level' : 0  # how many tools are running
        }

    def __init__(self, parent):
        Model.__init__(self)
        self.parent = parent

        # Available solvers: sat name, has incremental
        self.solvers = gtk.ListStore(gobject.TYPE_STRING, gobject.TYPE_BOOLEAN)
        self.solvers.set(self.solvers.append(None), 0, "MiniSAT", 1, True)
        self.solvers.set(self.solvers.append(None), 0, "Sim", 1, False)       

        self.use_tech = None      
        self.solver = None
        self.length = None
        self.loop = None
        self.use_inc = True
        self.use_sbmc = True
        self.use_comp = False
        self.use_vu = True
        
        # BDD
        self.part_meth = None
        self.reord_meth = None
        self.use_coi = False
        self.use_dyn_reord = True
        return

    def get_command_and_options(self):        
        """Returns a string and a list of options. The string is the
        command for checking"""

        if self.use_tech == "sat":
            l = [
                "set sat_solver %s" % self.solver,
                ]
            if not self.use_sbmc:
                cmd = "check_pslspec -b -k %s -l %s" % (self.length, self.loop)
            else:
                cmd = "check_pslspec -s -k %s -l %s" % (self.length, self.loop)
                if self.use_inc:
                    if self.use_comp: cmd += " -c"
                    if not self.use_vu: cmd += " -N"
                    pass
                pass            
                
            if self.use_inc: cmd += " -i"

            return (cmd, l)
            pass

        # BDD
        l = [
            "set partition_method %s" % self.part_meth,
            ]
        if self.use_dyn_reord:
            l.append("set reorder_method %s" % self.reord_meth)
            l.append("set dynamic_reorder")
        else: l.append("unset dynamic_reorder")

        if self.use_coi: l.append("set cone_of_influence")
        else: l.append("unset cone_of_influence")
        return ("check_pslspec", l)


    def get_use_sat(self): return self.use_tech == "sat"

    pass # end of class



class ReCheckingPanelModel (CheckingPanelModel):
    """Specific panel for Realizability"""
    def __init__(self, parent):
        CheckingPanelModel.__init__(self, parent)
        return

    def get_command_and_options(self):
        res = CheckingPanelModel.get_command_and_options(self)        
        return ("check_property", res[1])
    
    pass # end of class
    
