##  ===========================================================================
##
##  This is free software; you can redistribute it and/or
##  modify it under the terms of the GNU Lesser General Public
##  License as published by the Free Software Foundation; either
##  version 2 of the License, or (at your option) any later version.
##
##  This software is distributed in the hope that it will be useful,
##  but WITHOUT ANY WARRANTY; without even the implied warranty of
##  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
##  Lesser General Public License for more details.
##
##  You should have received a copy of the GNU Lesser General Public
##  License along with this library; if not, write to the Free Software
##  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA.
##
##  For more information about this software see <http://rat.fbk.eu>
##  or email to the RAT Team <rat@fbk.eu>.
##  Please report bugs to <rat@fbk.eu>.
##
##  ===========================================================================


import os
import os.path
import time

from base import BatchModeStub


import utils.utilities


# ----------------------------------------------------------------------
class MardukStub(BatchModeStub):
    """Stubbing of Marduk in batch mode"""

    def __init__(self, marduk_path, text_buffer=None, workdirectory=None, spec_file="_input.xml"):
        BatchModeStub.__init__(self, "Marduk", marduk_path, text_buffer, workdirectory)
        
        if ~os.access(marduk_path, os.X_OK):
            try:
                os.chmod(marduk_path, 0775)
            except:
                print "\n\nERROR: Stub Marduk is not executable, and changing permissions was denied."
                print "Make file <rat-dir>/resources/marduk/src/marduk.py executable!\n\n"
                raise
            
        
        self.spec_file = spec_file

        self.signals = None
        self.requirements = None
        return
        
    def set_signals_requirements(self, signals, requirements):
        """@param signals is a SignalsModel instance
        @param requirements is a RequirementsModel instance"""
        self.signals = signals
        self.requirements = requirements
        return

    def __dump_synthesis_problem(self, filename):
        from xml.dom import getDOMImplementation
        impl = getDOMImplementation()
        root = impl.createDocument(None, "project", None)

        self.signals.to_xml(root, root.documentElement)
        self.requirements.to_xml(root, root.documentElement)
        
        f = open(filename, "w")
        root.documentElement.writexml(f, addindent="  ", newl=os.linesep)
        f.close()

        root.unlink()
        return

    def begin_session(self):
        if self.text_buffer:
            self.text_buffer.insert(self.text_buffer.get_end_iter(),
                                    """
# Synthesis options are: '%s'
""" % " ".join(self.opts))
            pass
       

        # generates the source smv file:
        self.__dump_synthesis_problem(self.spec_file)
        return

    def end_session(self):
        if os.path.exists(self.spec_file): os.remove(self.spec_file)
        
        self.cleanup()
        return

    def process_session_result(self):
        return None

    pass # end of class
    
                         
