##  ===========================================================================
##  Author: Ingo Pill <ipill@ist.tugraz.at>
##
##  Copyright (c) 2005 by Graz University of Technology
##
##  This is free software; you can redistribute it and/or
##  modify it under the terms of the GNU Lesser General Public
##  License as published by the Free Software Foundation; either
##  version 2 of the License, or (at your option) any later version.
##
##  This software is distributed in the hope that it will be useful,
##  but WITHOUT ANY WARRANTY; without even the implied warranty of
##  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
##  Lesser General Public License for more details.
##
##  You should have received a copy of the GNU Lesser General Public
##  License along with this library; if not, write to the Free Software
##  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA.
##
##  For more information about this software see <http://rat.fbk.eu>
##  or email to the RAT Team <rat@fbk.eu>.
##  Please report bugs to <rat@fbk.eu>.
##
##  ===========================================================================


from base import ScriptBasedInteractiveModeStub
from tools.utils.vis import VisResults
import utils.utilities
import os
import gobject
# ----------------------------------------------------------------------
class VisInteractiveModeStub(ScriptBasedInteractiveModeStub):
    """Stub for using VIS in interactive Mode"""

    def __init__(self, vis_path, text_buffer=None):
        ScriptBasedInteractiveModeStub.__init__(self, "vis", vis_path, text_buffer)

        self.getResults = VisResults()
        return

    def begin_session(self):
        
        ScriptBasedInteractiveModeStub.begin_session(self)
        
        assert self.session_script_name
        return
    
    pass # end of class



# ----------------------------------------------------------------------
class PSVisStub(VisInteractiveModeStub):
    """This is the stub for VIS used by Property Simulation"""

    def __init__(self, vis_path, text_buffer=None):
        VisInteractiveModeStub.__init__(self, vis_path, text_buffer)
	self.errorMessage=''
        return

    def setProperty(self, property):
        self.property = property
        return
        
    def setAssumptions(self, assumptions):
        combinedAssString=''	 
        if len(assumptions)==0: self.assumptions = None
	else:
	  for i in assumptions:
	    combinedAssString=combinedAssString+i+'&&'
	  combinedAssString=combinedAssString[:-2]
	  self.assumptions=combinedAssString
	#print self.assumptions
	return
    
    def setProjectName(self, projectName):
        self.projectName=projectName
	return
   
   
    def begin_session(self):
        assert self.property
        assert self.projectName
	
        #print self.property
        
	if self.assumptions==None:
	  commandString=('sugar_sp -p %s -f "%s" ' % (self.projectName, self.property))
	else:
	  commandString=('sugar_sp -p %s -f "%s" -a "%s" ' % (self.projectName, self.property, self.assumptions))

	# generate command file for VIS batch mode
	batchFileName=self.projectName+".bat"
	batchFile = open(batchFileName, "w")
        batchFile.write(commandString)
        batchFile.close()
        self.add_opt('-x')
	self.add_opt('-f %s' % (batchFileName))
	#self.add_command('quit') # dummy command
	
        VisInteractiveModeStub.begin_session(self)

        return

    def end_session(self):
        #if self.resultStatus!='Vis Error':
        if os.path.exists(self.projectName+".bat"): os.remove(self.projectName+".bat")
        if os.path.exists(self.projectName+".fml"): os.remove(self.projectName+".fml")
        if os.path.exists(self.projectName+".str"): os.remove(self.projectName+".str")
        if os.path.exists(self.projectName+".evl"): os.remove(self.projectName+".evl")
        if os.path.exists(self.projectName+".dbg"): os.remove(self.projectName+".dbg")
        if os.path.exists(self.projectName+".err"): os.remove(self.projectName+".err")
        #print 'END_SESSION'
        VisInteractiveModeStub.end_session(self)
        self.cleanup()
        return

    def process_session_result(self):
        """@return None if no trace was found, or a filled TraceModel otherwise"""
	#print 'processing results'
	self.errorMessage=''
	
        if not os.path.exists(self.projectName+".dbg"):
	  self.resultStatus='Vis Error'
	  return None
	if not os.path.exists(self.projectName+'.err'):
	  self.resultStatus='Vis Error'
	  return None
        
	#1 detecting parse error for formula
	errorFile = open(self.projectName+'.err','r')
	errorLines = errorFile.readlines()
	errorLines.reverse()
        for i in errorLines:
	  if i[:48]=='sugar_sp formula error: error in parsing formula':
	    self.resultStatus='Vis Error'
	    self.errorMessage='Error while parsing formula'
	    return None
	    
	    
        visFile = open(self.projectName+'.dbg','r')
        visLines = visFile.readlines()
	visLines.reverse()
	for i in visLines:
	  if i[:50]=='# SUGAR_SP: Language is not empty, suggesting path':
	    self.resultStatus='Trace'
	    visFile.close()
            if not ((os.path.exists(self.projectName+".str"))&(os.path.exists(self.projectName+".evl"))): 
	      self.resultStatus='Vis Error'
	      return None
	    return self.getResults.read(self.projectName)
	  if i[:5]=='#SPNT':
	    self.resultStatus='No Trace'
	    visFile.close()
	    return None
	visFile.close()
	
	return None

    def session_is_over(self, val, user_arg):
        """This is a callback called by the session execution thread
         when the work is done. There is a special workaround for vis."""

        # result processing in background, before returning control to main thread
        self.ret_value = val
        self.result = None        
	self.resultStatus = 'Vis Error'
	# Sometimes the ret_val is -1 without reason - workaround TODO resolve bug
	if ((os.path.exists(self.projectName+".str"))&(os.path.exists(self.projectName+".evl"))): 
	  self.result = self.process_session_result()

        # session cleanup
        self.end_session()

        # Returns to the main (gtk) thread
        gobject.idle_add(self.end_callback_wrapper)                         
        return

    pass # end of class
