##  ===========================================================================
##  Author: Roberto Cavada <cavada@fbk.eu>
##
##  Copyright (c) 2005 by FBK 
##
##  This is free software; you can redistribute it and/or
##  modify it under the terms of the GNU Lesser General Public
##  License as published by the Free Software Foundation; either
##  version 2 of the License, or (at your option) any later version.
##
##  This software is distributed in the hope that it will be useful,
##  but WITHOUT ANY WARRANTY; without even the implied warranty of
##  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
##  Lesser General Public License for more details.
##
##  You should have received a copy of the GNU Lesser General Public
##  License along with this library; if not, write to the Free Software
##  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA.
##
##  For more information about this software see <http://rat.fbk.eu>
##  or email to the RAT Team <rat@fbk.eu>.
##  Please report bugs to <rat@fbk.eu>.
##
##  ===========================================================================

import xml.dom.minidom

import models.trace

# This module contains classes and functions to be used as helpers
# in the interaction with nusmv

class NuSMVTraceReader :

    def __init__(self):
        return

    def parse(self, filename):
        """Parses the trace in xml format contained in the given file.
        @return a new filled TraceModel instance, or None if no trace is
        found"""

        root = xml.dom.minidom.parse(filename)
        node = root.documentElement
        if node.tagName != "counter-example": return None

        trace = models.trace.TraceModel()
        
        steps = node.getElementsByTagName("node")
        loops = node.getElementsByTagName("loops")

        # collects steps info
        names = {}
        names_list = []
        step_fields = ("state", "combinatorial", "input")
        for step in steps:
            for field in step_fields: 
                values = step.getElementsByTagName(field)
                for val in values:
                    vars = val.getElementsByTagName("value")
                    for var in vars:
                        name = var.getAttribute("variable")
                        if name not in names:
                            names[name] = []
                            names_list.append(name)
                            pass
                        
                        val = ""
                        for text_child in var.childNodes:
                            if text_child.nodeType == xml.dom.Node.TEXT_NODE:
                                val += text_child.data
                                pass
                            pass
                        
                        names[name].append(val)
                        pass
                    pass
                pass
            pass

        for name in names_list:
            stripe = models.trace.TraceStripeModel(name)
            for step in names[name]: stripe.add_value(step)
            iter = trace.append(None)
            trace.set_value(iter, 0, stripe)
            pass


        loops_strs = ""
        for loop in loops:
            val = ""
            for text_child in loop.childNodes:
                if text_child.nodeType == xml.dom.Node.TEXT_NODE:
                    val += text_child.data
                    pass
                pass            

            val = val.strip()
            if val: loops_strs += val + " "
            pass

        loops_strs = loops_strs.replace(",", " ").strip()
        for loop in loops_strs.strip().split(" "):
            if loop != "": trace.add_loop_at_step(int(loop))
            pass
        
        pass
                
        return trace

    pass # end of class
