##  ===========================================================================
##  Author: Roberto Cavada <cavada@fbk.eu>
##
##  Copyright (c) 2007 by FBK 
##
##  This is free software; you can redistribute it and/or
##  modify it under the terms of the GNU Lesser General Public
##  License as published by the Free Software Foundation; either
##  version 2 of the License, or (at your option) any later version.
##
##  This software is distributed in the hope that it will be useful,
##  but WITHOUT ANY WARRANTY; without even the implied warranty of
##  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
##  Lesser General Public License for more details.
##
##  You should have received a copy of the GNU Lesser General Public
##  License along with this library; if not, write to the Free Software
##  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA.
##
##  For more information about this software see <http://rat.fbk.eu>
##  or email to the RAT Team <rat@fbk.eu>.
##  Please report bugs to <rat@fbk.eu>.
##
##  ===========================================================================



import subprocess
import os
import time
import signal

def create_pipe(cmd, workdirectory=None):
    try:
        if workdirectory == None:
            return subprocess.Popen(cmd.split(), stdin=subprocess.PIPE,
                                    stdout=subprocess.PIPE, stderr=subprocess.STDOUT)
        else:
            return subprocess.Popen(cmd.split(), stdin=subprocess.PIPE,
                                    stdout=subprocess.PIPE, stderr=subprocess.STDOUT,
                                    cwd=workdirectory)
    except:
        import traceback
        traceback.print_exc()
        return None

def kill_pipe(pipe):
    """Kills the Popen object passed. pipe has to be built with
    subprocess module"""
    
    if pipe.poll() is not None: return

    # tries terminating the process first kindly
    pipe.stdin.close()
    for i in range (0, 5):
        time.sleep(0.1)
        if pipe.poll() is not None: return
        pass
    
    if subprocess.mswindows: _terminate_process_win32(pipe)
    else: os.kill(pipe.pid, signal.SIGKILL)
    return
        

def _terminate_process_win32(pipe):
    assert subprocess.mswindows
    try: from subprocess import TerminateProcess
    except:
        try: from win32api import TerminateProcess 
        except:
            try: from ctypes import TerminateProcess
            except:
                # last choice via a windows command
                if os.popen('TASKKILL /PID %d /F' % pipe.pid):
                    import warnings
                    if sys.version_info < (2,5,0): msg = " or install Python version >= 2.5.0"
                    else: msg = ""
                    warnings.warn("It was not possible to kill the process. Install module 'win32api'%s." % msg,
                                  RuntimeWarning)
                    pass
                return
            pass
        pass

    TerminateProcess(pipe._handle, -1)
    return
