##  ===========================================================================
##  Author: Roberto Cavada <cavada@fbk.eu>
##
##  Copyright (c) 2005 by FBK 
##
##  This is free software; you can redistribute it and/or
##  modify it under the terms of the GNU Lesser General Public
##  License as published by the Free Software Foundation; either
##  version 2 of the License, or (at your option) any later version.
##
##  This software is distributed in the hope that it will be useful,
##  but WITHOUT ANY WARRANTY; without even the implied warranty of
##  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
##  Lesser General Public License for more details.
##
##  You should have received a copy of the GNU Lesser General Public
##  License along with this library; if not, write to the Free Software
##  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA.
##
##  For more information about this software see <http://rat.fbk.eu>
##  or email to the RAT Team <rat@fbk.eu>.
##  Please report bugs to <rat@fbk.eu>.
##
##  ===========================================================================

import os.path
import platform
import sys

if sys.argv[0]: top_dir = os.path.dirname(os.path.abspath(sys.argv[0]))
else: top_dir = "."
    

# A set of global vars
TOPDIR = top_dir
RESOURCES_DIR = os.path.join(TOPDIR, "resources")
GLADE_DIR = os.path.join(RESOURCES_DIR, "glade")
IMAGES_DIR = os.path.join(RESOURCES_DIR, "images")
#TOOLS_DIR = os.path.join(RESOURCES_DIR, "bin")
TOOLS_DIR = os.path.join(TOPDIR, "../bin")
STYLES_DIR = os.path.join(RESOURCES_DIR, "styles")
MARDUK_DIR = os.path.join(RESOURCES_DIR, "marduk")
MARDUK_SRC_DIR = os.path.join(MARDUK_DIR, "src")

bits = int(platform.architecture()[0][0:-3])
assert bits in (32, 64), "32 and 64 bit arcitectures only (found %d)" % bits

NUSMV_NAME = None
for name in ("NuSMV4Compass", "NuSMV"):

    if platform.system() == "Windows":
        NUSMV_PATH = os.path.join(TOOLS_DIR, name + "-win%d.exe" % bits)
        VIS_PATH = os.path.join(TOOLS_DIR, "vis.exe")
    else:
        NUSMV_PATH = os.path.join(TOOLS_DIR, name + "-linux%d" % bits)
        VIS_PATH = os.path.join(TOOLS_DIR, "vis")
        pass
    if os.path.exists(NUSMV_PATH):
        NUSMV_NAME = name
        break
    pass

if NUSMV_NAME is None:
    raise IOError("NuSMV executable not found")

MARDUK_PATH = os.path.join(MARDUK_SRC_DIR, "marduk.py")


SHORT_APPL_NAME = "RAT"
PROJECT_FILE_EXTENSION = ".rat"
APPL_VERSION = (1, 2, 0)

APPL_OPTIONS_FILE = None


# ----------------------------------------------------------------------
# Default names for standard categories
CAT_DEF_NAME = "Default"
CAT_NEW_NAME = "New"
CAT_OOD_NAME = "Out of Date"
CAT_TRA_NAME = "Trash"

# ----------------------------------------------------------------------
CATEGORIES = ( CAT_NEW_NAME, CAT_DEF_NAME, CAT_OOD_NAME, CAT_TRA_NAME )

CAT_DESCRIPTIONS = {
    CAT_DEF_NAME : "This is the default category for traces",
    CAT_NEW_NAME : "This is the category of those traces that have been just created",
    CAT_OOD_NAME : "Contains the traces whose dependencies might be no longer consistent",
    CAT_TRA_NAME : "Contains the traces that have been deleted",
    }


