##  ===========================================================================
##  Author: Roberto Cavada <cavada@fbk.eu>
##
##  Copyright (c) 2005 by FBK 
##
##  This is free software; you can redistribute it and/or
##  modify it under the terms of the GNU Lesser General Public
##  License as published by the Free Software Foundation; either
##  version 2 of the License, or (at your option) any later version.
##
##  This software is distributed in the hope that it will be useful,
##  but WITHOUT ANY WARRANTY; without even the implied warranty of
##  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
##  Lesser General Public License for more details.
##
##  You should have received a copy of the GNU Lesser General Public
##  License along with this library; if not, write to the Free Software
##  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA.
##
##  For more information about this software see <http://rat.fbk.eu>
##  or email to the RAT Team <rat@fbk.eu>.
##  Please report bugs to <rat@fbk.eu>.
##
##  ===========================================================================

from gtkmvc.view import View 

from project import ProjectView
import utils.globals
import utils.application

import os.path
import gtk

## ----------------------------------------------------------------------
class ApplicationView (View):
    """This is the main application's view."""

    GLADE_FILE = os.path.join(utils.globals.GLADE_DIR, "application.glade") 

    def __init__(self, ctrl):
        View.__init__(self, ctrl, self.GLADE_FILE, ("window_main", "vbox_banner"),
                      register=False)

        self.setup_view()
        
        self.project = None

        ctrl.registerView(self)
        return

    def setup_view(self):

        self['image_banner'].set_from_file(os.path.join(utils.globals.IMAGES_DIR,
                                                 "dialog/banner.png"))
        self['label_appl_name'].modify_bg(gtk.STATE_NORMAL, gtk.gdk.color_parse("yellow"))

        self['label_appl_name'].set_markup(self['label_appl_name'].get_label() \
                                           % utils.application.get_short_version_string())
        
        self['vbox_main'].add(self['vbox_banner'])
        self['tb_show_traces'].set_stock_id("rat-stock-trace")
        return

    def new_project(self, project_ctrl):

        # detaches previous project view, if there is any
        if self.project: self['vbox_main'].remove(self.project.get_top_widget())
        else: self['vbox_main'].remove(self['vbox_banner']) # detaches the banner

        # creates and connects a new project view to the application
        self.project = ProjectView(project_ctrl, self)
        self['vbox_main'].add(self.project.get_top_widget())

        # enables controls for project interaction
        wids = ('item_show_traces', 'tb_show_traces', 'rb_prop_assurance',
#HIDE                'rb_prop_simulation', 'rb_realizability'
                )
        for wid in wids: self[wid].set_sensitive(True)
        
        return

    def set_title(self, title):
        self['window_main'].set_title(title)
        return

    def show_progress_bar(self, show=True):
        pb = self['progress_bar']
        if pb is None:
            pb = gtk.ProgressBar()
            self['progress_bar'] = pb
            self['statusbar'].pack_end(pb, expand=False)
            pass

        if show: pb.show()
        else: pb.hide()
        return

    def tick_progress_bar(self):
        self['progress_bar'].pulse()
        return

    pass # end of class
    
    
    
## ----------------------------------------------------------------------
class ApplicationAboutView (View):
    
    GLADE_FILE = os.path.join(utils.globals.GLADE_DIR, "application.glade")

    def __init__(self, ctrl):
        View.__init__(self, ctrl, self.GLADE_FILE, "dialog_about",
                      register=False)

        self.setup_view()
        ctrl.registerView(self)
        return

    def setup_view(self):
        self['image_about'].set_from_file(os.path.join(utils.globals.IMAGES_DIR,
                                                       "dialog/logo.png"))

        f = open(os.path.join(utils.globals.TOPDIR, "License"), "r")
        buf = self['textview_license'].get_buffer()
        buf.set_text(f.read())

        self['label_about_name'].set_markup(self['label_about_name'].get_label() \
                                            % utils.application.get_version_string())
        
        return

    def run(self):
        w = self.get_top_widget()
        res = w.run()
        w.destroy()
        return res
        

    pass # end of class


## ----------------------------------------------------------------------
class ApplicationWizardView (View):
    
    GLADE_FILE = os.path.join(utils.globals.GLADE_DIR, "application.glade")

    def __init__(self, ctrl):
        View.__init__(self, ctrl, self.GLADE_FILE, "window_wizard", register=False)

        self.setup_view()
        ctrl.registerView(self)
        return

    def setup_view(self):
        self['image_wizard'].set_from_file(os.path.join(utils.globals.IMAGES_DIR,
                                                        "dialog/wizard.png"))
        self['notebook'].set_show_tabs(False)
        return

    def set_description(self, text): self['label_desc'].set_markup(text)

    def destroy(self): self.get_top_widget().destroy()

    pass # end of class

