##  ===========================================================================
##  Author: Roberto Cavada <cavada@fbk.eu>
##
##  Copyright (c) 2005 by FBK 
##
##  This is free software; you can redistribute it and/or
##  modify it under the terms of the GNU Lesser General Public
##  License as published by the Free Software Foundation; either
##  version 2 of the License, or (at your option) any later version.
##
##  This software is distributed in the hope that it will be useful,
##  but WITHOUT ANY WARRANTY; without even the implied warranty of
##  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
##  Lesser General Public License for more details.
##
##  You should have received a copy of the GNU Lesser General Public
##  License along with this library; if not, write to the Free Software
##  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA.
##
##  For more information about this software see <http://rat.fbk.eu>
##  or email to the RAT Team <rat@fbk.eu>.
##  Please report bugs to <rat@fbk.eu>.
##
##  ===========================================================================


from gtkmvc.model import Model
from gtkmvc.controller import Controller
from gtkmvc.view import View

from utils import globals
from gtk import RESPONSE_OK
import os.path

# ----------------------------------------------------------------------
class DialogView (View):
    GLADE_FILE = os.path.join(globals.GLADE_DIR, "dialogs.glade")
    def __init__(self, ctrl, top_widget):        
        View.__init__(self, ctrl, self.GLADE_FILE, top_widget)
        return

    def run(self):
        res = self.get_top_widget().run()
        self.get_top_widget().destroy()
        return res
    
    pass # end of class

# ----------------------------------------------------------------------
class InvalidName (DialogView):
    def __init__(self, ctrl):
        DialogView.__init__(self, ctrl, "dialog_invalid_name")
        return
    pass # end of class


# ----------------------------------------------------------------------
class DialogMessage (DialogView):
    def __init__(self, ctrl, message, details):
        DialogView.__init__(self, ctrl, "dialog_message")
        self['label_message'].set_markup("<big><b>%s</b></big>" % message)
        self['label_details'].set_text(details)        
        return
    pass # end of class


# ----------------------------------------------------------------------
class WarningRequestOkCancel (DialogView):
    def __init__(self, ctrl, message, details):
        DialogView.__init__(self, ctrl, "dialog_confirm_cancel_ok")
        self['label_message'].set_markup("<big><b>%s</b></big>" % message)
        self['label_details'].set_text(details)
        return
    pass # end of class


# ----------------------------------------------------------------------
class QuestionConfirmAction (DialogView):
    def __init__(self, ctrl, action_name, message, details):
        DialogView.__init__(self, ctrl, "dialog_confirm_action")
        self['label_message'].set_markup("<big><b>%s</b></big>" % message)
        self['label_details'].set_text(details)
        self['button_action'].set_label(action_name)
        return
    pass # end of class


# ----------------------------------------------------------------------
class QuestionConfirmCategoryRemoval (DialogView):
    def __init__(self, ctrl, message, details):
        DialogView.__init__(self, ctrl, "dialog_del_category")
        self['label_message'].set_markup("<big><b>%s</b></big>" % message)
        self['label_details'].set_text(details)
        self['button_action'].set_label("Proceed")
        return

    def run(self):
        r = self.get_top_widget().run()
        res = (r, self['checkbutton_move_to_trash'].get_active())
        self.get_top_widget().destroy()
        return res

# ----------------------------------------------------------------------
class SaveAsCustomFileChooserDialog (DialogView):
    def __init__(self, ctrl, message, *args):
        DialogView.__init__(self, ctrl, "dialog_custom_saveas")

        (extra_info,) = args
        self.set_extra_info(extra_info)
        self.set_default_response(RESPONSE_OK)
        return

    def get_filename(self):
        return self['dialog_custom_saveas'].get_filename()

    def set_extra_info(self, flag):
        self['cb_extra_info'].set_active(flag)
        return

    def get_extra_info(self):
        return self['cb_extra_info'].get_active()

    def set_default_response(self, flag):
        self['dialog_custom_saveas'].set_default_response(flag)
        return

    def run(self):
        return self.get_top_widget().run()

    def destroy(self):
        self['dialog_custom_saveas'].destroy()

    pass # end of class
