#!/usr/bin/env python

# =============================================================================
#  This file is part of the ``HASDEL'' Toolset.
#  Copyright (C) 2014 by Fondazione Bruno Kessler
#  Copyright (C) 2014 by RWTH Aachen University
#
#
#  The HASDEL Toolset can be copied, modified and/or redistributed
#  under the terms of the HASDEL General Public License, either
#  version 1 of the License, or (at your option) any later version.
#
#  You should have received a copy of the HASDEL General Public
#  License along with this software. If not, please contact the
#  copyright owners.
#
#  The HASDEL Toolset is distributed in the hope that it will be
#  useful, but WITHOUT ANY WARRANTY; without even the implied warranty
#  of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  HASDEL General Public License for more details.
#
#  For more information on the HASDEL Project see the Project Web
#  site at <http://compassinternal.informatik.rwth-aachen.de/hasdel-site/> 
#  or email to <hasdel-support@lists.rwth-aachen.de>.  
#
#  Please report bugs to <hasdel-support@lists.rwth-aachen.de>.
# =============================================================================



"""Fault Recovery Analysis

Check the capability of a system to recover from a fault.

Inputs      M : an operational model.
            FM : a fault model.
            FDM : an FDIR model.
            P : a set of properties.

Outputs     Ok shall be returned if the recoverability property holds,
            a counterexample trace Tr otherwise.

Processing Using model checking techniques, the COMPASS toolset
            shall check if a system is able to recover from a
            fault.  given an operational model, extended with a
            fault model and an FDIR model, and a property
            representing the recoverability. A counterexample in
            XML format shall be generated if the property does not
            hold.
"""

import booter # import this as first module in scripts


from runscripts.model_checking__srs_015 import NuSMVModelChecking
class NuSMVFaultRecoveryAnalysis (NuSMVModelChecking):
    """This script is used to perform fault recovery analysis"""

    def check_options(self, opt_parser, options):
        # wrt model checking, fault model and FDIR are mandatory
        super(NuSMVFaultRecoveryAnalysis, self).check_options(opt_parser, options)

        if not options.FAULT_INJECTION_FILE_NAME:
            opt_parser.error("Fault injection file is required.")
            pass
        return        

    pass # end of class
# ----------------------------------------------------------------------
        
        
if __name__ == "__main__":
    
    main = NuSMVFaultRecoveryAnalysis()
    main.run()
    
