/**
# =============================================================================
#  This file is part of the ``COMPASS'' Toolset.
#  Copyright (C) 2010 by Fondazione Bruno Kessler
#  Copyright (C) 2010 by RWTH Aachen University
#
#
#  The COMPASS Toolset can be copied, modified and/or redistributed
#  under the terms of the COMPASS General Public License, either
#  version 1 of the License, or (at your option) any later version.
#
#  You should have received a copy of the COMPASS General Public
#  License along with this software. If not, please contact the
#  copyright owners.
#
#  The COMPASS Toolset is distributed in the hope that it will be
#  useful, but WITHOUT ANY WARRANTY; without even the implied warranty
#  of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  COMPASS General Public License for more details.
#
#  For more information on the COMPASS Project see the Project Web
#  site at <http://compass.informatik.rwth-aachen.de/> or email to
#  <compass-support@lists.rwth-aachen.de>.  
#
#  Please report bugs to <compass-support@lists.rwth-aachen.de>.
# =============================================================================

**/

grammar Slim;

options
{
	language=Python;
	output=AST;
	backtrack=true;
}

tokens
{
	PACKAGE = 'package';
	PUBLIC = 'public';
	PRIVATE = 'private';
	END = 'end';
	SEMICOLON = ';';
	FEATURES = 'features';
	MODEL = 'model';
	THREAD = 'thread';
	GROUP = 'group';
	PROCESS = 'process';
	DATA = 'data';
	MEMORY = 'memory';
	PROCESSOR = 'processor';
	BUS = 'bus';
	DEVICE = 'device';
	SYSTEM = 'system';
	COLON = ':';
	IN = 'in';
	OUT = 'out';
	ON = 'on';
	EVENT = 'event';
	DATA = 'data';
	PORT = 'port';
	NONBLOCKING = 'nonblocking';
	DOT = '.';
	FDIR = 'fdir';
	OBSERVABLE = 'observable';
	ALARM = 'alarm';
	IMPLEMENTATION = 'implementation';
	SUBCOMPONENTS = 'subcomponents';
	CONNECTIONS = 'connections';
	MODES = 'modes';
	TRANSITIONS = 'transitions';
	STORED = 'stored';
	RUNNING = 'running';
	ACCESSES = 'accesses';
	ARROW = '->';
	INITIAL = 'initial';
	ACTIVATION = 'activation';
	ATACTIVATION = '@activation';
	MODE = 'mode';
	URGENT = 'urgent';
	WHILE = 'while';
	AND = 'and';
	LT = '<';
	LE = '<=';
	GT = '>';
	GE = '>=';
	CASE = 'case';
	OTHERWISE = 'otherwise';
	THEN = 'then';
	LBRACKET = '[';
	RBRACKET = ']';
	RESET = 'reset';
	ASSIGN = ':=';
	LPAREN = '(';
	RPAREN = ')';
	COMMA = ',';
	STATE = 'state';
	ERROR = 'error';
	NOMINAL = 'nominal';
	PROPAGATION = 'propagation';
	OCCURRENCE = 'occurrence';
	FIXED = 'fixed';
	POISSON = 'poisson';
	STATES = 'states';
	EVENTS = 'events';
	CLOCKS = 'clocks';
	RESET = 'reset';
	WHEN = 'when';
	WITHIN = 'within';
	TO = 'to';
	NOT = 'not';
	OR = 'or';
	XOR = 'xor';
	XNOR = 'xnor';
	IFF = 'iff';
	IMP = 'imp';
	DERIVATIVE = '\'';
	EQ = '=';
	NEQ = '!=';
	BOOL = 'bool';
	CLOCK = 'clock';
	CONTINUOUS = 'continuous';
	ENUM = 'enum';
	INT = 'int';
	REAL = 'real';
	DEFAULT = 'default';
	MSEC = 'msec';
	SEC = 'sec';
	MIN = 'min';
	HOUR = 'hour';
	DAY = 'day';
	TIMES = '*';
	PLUS = '+';
	MINUS = '-';
	DIVIDE = '/';
	MOD = 'mod';
	OCCURRENCE = 'occurrence';
	TRUE = 'true';
	FALSE = 'false';
	PROB = 'prob';
	PER = 'per';
	FLOWS = 'flows';
	PASSIVE = 'passive';
		
	COMP_TYPE;
	COMP_IMPL;
	ERR_TYPE;
	ERR_IMPL;
	IEPORT;
	OEPORT;
	IDPORT;
	ODPORT;
	ERRORPORT;
	SUBCOMPONENT;
	THREAD_GROUP;
	MODE_TRANS;
	MODE_TRANS_PROB;
	IE_PROP;
	OE_PROP;
	ERR_EVENT;
	ERR_CLOCK;
	ERR_TRANS;
	NAME;
	FQ_NAME_OR_IDENT; // fully qualified identifier or name
	DATA_CONNECTION;
	EVENT_CONNECTION;
	NORMAL;
	TRAJ_EXPR; // deprecated? is actually never used
	UNARY_MINUS;
	FLOW;
}

@header {
    from slim_compiler.src.errorManagement.exceptions import *
    from slim_compiler.src import globals
}

@lexer::header {
    from slim_compiler.src.errorManagement.exceptions import *
    from slim_compiler.src import globals
}

@members{
	"""Stack of current rules being parsed. Mainly used by the error handler."""
	rules = []
	
	def parseError(self, message):
		"""
		Insert exception in error list, but do not interrupt parser.
		"""
		self.reportError(SLIMParserException(message, self.input))
}

/* ****** start parser ************************************************* */
/* BEGIN package contents */
specification
@init { self.rules.append("SLIM specification") }
@after { self.rules.pop() }
	:	(
			(packageSpecification | declaration)+ 
		) | { self.parseError("Must declare at least one package or declaration")}
		EOF
	;
	
packageSpecification
@init { self.rules.append("package specification") }
@after { self.rules.pop() }
	:	PACKAGE^ IDENT 
			PUBLIC (declaration+ | { self.parseError("Expected at least one public declaration")})
			(PRIVATE (declaration+ | { self.parseError("Expected private declaration")}))?
		END! IDENT SEMICOLON!
	;
	
declaration
	:	{globals.PARSER_ACCEPT_FDIR or globals.PARSER_ACCEPT_NOMINAL}?=> componentType
	|	{globals.PARSER_ACCEPT_FDIR or globals.PARSER_ACCEPT_NOMINAL}?=> componentImplementation
	|	{globals.PARSER_ACCEPT_ERROR}?=> errorType
	|	{globals.PARSER_ACCEPT_ERROR}?=> errorImplementation
	;
	
componentType
@init { self.rules.append("component specification") }
@after { self.rules.pop() }
	:	({globals.PARSER_ACCEPT_NOMINAL}?=> (FDIR)?) componentCategory IDENT
			(FEATURES (portSpec+ | {self.parseError("Expecting feature declaration")}))?
		END IDENT SEMICOLON
		-> ^(COMP_TYPE (FDIR)? componentCategory IDENT portSpec* IDENT)
	;

componentImplementation
@init { self.rules.append("component implementation") }
@after { self.rules.pop() }
	:   	({globals.PARSER_ACCEPT_FDIR}?=> (FDIR)?) componentCategory IMPLEMENTATION name
			(SUBCOMPONENTS (subcomponent+ | {self.parseError("Expecting subcomponent specification")}))?
			(CONNECTIONS (portConnection+ | {self.parseError("Expecting port connection")}))?
			(FLOWS (flow+ | {self.parseError("Expecting flow specification")}))?
			(MODES (mode+ | {self.parseError("Expecting mode specification")}) (TRANSITIONS (modeTransition+| {self.parseError("Expecting mode transition")}))?)?
		END name SEMICOLON
		-> ^(COMP_IMPL (FDIR)? componentCategory name subcomponent* portConnection* flow* mode* modeTransition* name) 
	;

errorType
@init { self.rules.append("error specification") }
@after { self.rules.pop() }
	:	ERROR MODEL IDENT 
			(FEATURES (errorFeature+ | {self.parseError("Expecting error feature")}))?
		END IDENT SEMICOLON
		-> ^(ERR_TYPE IDENT errorFeature* IDENT)
	;

errorImplementation
@init { self.rules.append("error implementation") }
@after { self.rules.pop() }
	:	ERROR MODEL IMPLEMENTATION name
			(EVENTS (errorEvent+ | {self.parseError("Expecting error event")}))?
			(CLOCKS (errorClock+ | {self.parseError("Expecting error clock")}))?
			(STATES (errorState+ | {self.parseError("Expecting error state")}))?
			(TRANSITIONS (errorTransition+ | {self.parseError("Expecting error transition")}))?
		END name SEMICOLON
		-> ^(ERR_IMPL name errorClock* errorState* errorEvent* errorTransition* name)
	;
/* END package contents*/
/****************************************/
/* BEGIN categories and datatypes */	
componentCategory
	:	PROCESS
	|	THREAD GROUP -> THREAD_GROUP
	|	THREAD
	|	PROCESSOR
	|	MEMORY
	|	DEVICE
	|	BUS
	|	SYSTEM
	;
		
dataType
@init { self.rules.append("data type") }
@after { self.rules.pop() }
	:	BOOL
	|	CLOCK timeScale?
	|	CONTINUOUS
	|	ENUM^ LPAREN! identifierList RPAREN!
	|	INT
	|   REAL
	|   intRange
	;
	
timeScale
@init { self.rules.append("time scale") }
@after { self.rules.pop() }
	:	MSEC
	|	SEC
	|	MIN
	|	HOUR
	|	DAY
	;
	
intRange
@init { self.rules.append("integer range") }
@after { self.rules.pop() }
    :   LBRACKET! l=number DOT! DOT! u=number RBRACKET!
    {
        if $l.text.find('.') != -1 or $u.text.find('.') != -1:
            self.parseError("Range cannot specify real numbers")
    }
    ;
	
identifierList
@init { self.rules.append("identifier list") }
@after { self.rules.pop() }
	:	IDENT (COMMA! IDENT)*
	;
/* END categories and datatypes */	
/* BEGIN port definitions */	
portSpec
@init { self.rules.append("port specification") }
@after { self.rules.pop() }
	:	IDENT COLON IN EVENT PORT SEMICOLON
		-> ^(IEPORT IDENT)
	|	IDENT COLON OUT EVENT PORT (NONBLOCKING)? SEMICOLON
		-> ^(OEPORT IDENT (NONBLOCKING)?)
	|	IDENT COLON IN DATA PORT dataType (initialization)? SEMICOLON
		-> ^(IDPORT IDENT dataType initialization?)		
	|	IDENT COLON OUT DATA PORT dataType (initialization)? (OBSERVABLE | ALARM)? SEMICOLON
		-> ^(ODPORT IDENT dataType initialization? OBSERVABLE? (ALARM)?)
	|	{globals.PARSER_ACCEPT_EXTENDED_MODEL}?=>IDENT COLON ERROR EVENT PORT (POISSON NUMBER)? SEMICOLON
		-> ^(ERRORPORT IDENT (POISSON NUMBER)?)
	;
	
initialization
	:	DEFAULT! value
	;

value
@init { self.rules.append("value") }
@after { self.rules.pop() }
	:	TRUE
	|	FALSE
	|	number
	|	IDENT
	;

timeScaledNumber
	: number timeScale^
	| number
	;

number
@init { self.rules.append("number") }
@after { self.rules.pop() }
    :   MINUS! NUMBER
		{ $NUMBER.setText("-" + $NUMBER.getText()) }
	|	NUMBER
	;
			
/* END port definitions */	
/* BEGIN qualified identifiers */
name
@init { self.rules.append("name") }
@after { self.rules.pop() }
	:	IDENT DOT IDENT
		-> ^(NAME IDENT IDENT)
	;
		
uniqueNameOrIdentifier
	:	IDENT COLON COLON  nameOrIdentifier
		-> ^(FQ_NAME_OR_IDENT IDENT nameOrIdentifier)
	|	nameOrIdentifier
	;
	
nameOrIdentifier
	:	name
	|	IDENT
	;
/* END qualified identifiers */	
/* BEGIN subcomponent declaration */
subcomponent
@init { self.rules.append("subcomponent") }
@after { self.rules.pop() }
	:	IDENT COLON componentCategory uniqueNameOrIdentifier binding* inModes? SEMICOLON
		-> ^(componentCategory IDENT uniqueNameOrIdentifier inModes? binding*)
	|	IDENT COLON DATA dataType initialization? inModes? OBSERVABLE? SEMICOLON
		-> ^(DATA IDENT inModes? dataType initialization? OBSERVABLE?)
	;

binding
	:	(STORED^ IN! | RUNNING^ ON! | ACCESSES^) IDENT
	;
/* END subcomponent declaration */
/* BEGIN port connections */	
portConnection
@init { self.rules.append("port connection") }
@after { self.rules.pop() }
	:	EVENT PORT nameOrIdentifier ARROW nameOrIdentifier inModes? PASSIVE? SEMICOLON
		-> ^(EVENT_CONNECTION nameOrIdentifier nameOrIdentifier inModes? PASSIVE?)
	|	DATA PORT nameOrIdentifier ARROW nameOrIdentifier inModes? SEMICOLON
		-> ^(DATA_CONNECTION nameOrIdentifier nameOrIdentifier inModes?)
	;
/* END port connections */
/* BEGIN flows */
flow
@init { self.rules.append("flow") }
@after { self.rules.pop() }
    :   {self.allow_names = True} nameOrIdentifier ASSIGN expression {self.allow_names = False} (inModes)? SEMICOLON
        -> ^(FLOW nameOrIdentifier expression (inModes)?)
    ;
   
/* END flows */
/* BEGIN modes */		
mode
@init { self.rules.append("mode specification") }
@after { self.rules.pop() }
	:	IDENT COLON modeType MODE 
        urgent?
        (WHILE invariant)? 
        SEMICOLON
		-> ^(modeType IDENT urgent? invariant?)
	;

urgent
@init { self.rules.append("urgent") }
@after { self.rules.pop() }
	:	URGENT IN timeScaledNumber -> ^(URGENT timeScaledNumber)
	|	URGENT IN NUMBER -> ^(URGENT NUMBER)
	;
	
modeType
	:	INITIAL
	|	ACTIVATION
	|	{globals.PARSER_ACCEPT_NOMINAL and globals.PARSER_ACCEPT_ERROR}?=> ERROR
	|	
		-> NORMAL	// normal mode
	;
	
invariant
@init { self.rules.append("invariant") }
@after { self.rules.pop() }
	:	atomicInvariant (AND^ atomicInvariant)*
	;
	
atomicInvariant
	:	expression
	|	trajectoryEquation
	;
	
/* END modes */
/* BEGIN expressions */
expression
@init { self.rules.append("expression") }
@after { self.rules.pop() }
	: 	impExpression
	;

expressionsOnly
	:	{globals.PARSER_EXPRESSIONS_ONLY}?=> expression EOF!
	;

impExpression
	:	iffExpression (IMP^ iffExpression)*
	;

iffExpression
	:	orExpression (IFF^ orExpression)*
	;

orExpression
	:	andExpression ((OR|XOR|XNOR)^ andExpression)*
	;
	
andExpression
	:	compareExpression (AND^ compareExpression)*
	;
		
compareExpression
	:	relativeExpression ({not globals.PARSER_FT_BOOL_EXPRESSIONS_ONLY}?=> (EQ|NEQ)^ relativeExpression)*
	;
	
relativeExpression
	:	addExpression ({not globals.PARSER_FT_BOOL_EXPRESSIONS_ONLY}?=> (LT|LE|GT|GE)^ addExpression)*
	;

addExpression
	:	mulExpression ({not globals.PARSER_FT_BOOL_EXPRESSIONS_ONLY}?=> (PLUS|MINUS)^ mulExpression)*
	;

mulExpression
	:	timeExpression ({not globals.PARSER_FT_BOOL_EXPRESSIONS_ONLY}?=> (TIMES|DIVIDE|MOD)^ timeExpression)*
	;

timeExpression
	: unaryExpression (timeScale^)*
	;

unaryExpression
	:	(NOT unaryExpression) -> ^(NOT unaryExpression)
	|	(MINUS unaryExpression) -> ^(UNARY_MINUS unaryExpression)
	|	atomicExpression
	;

atomicExpression
	:	{not globals.PARSER_FT_BOOL_EXPRESSIONS_ONLY}?=> number
	|	{globals.PARSER_EXPRESSIONS_ONLY and not globals.PARSER_FT_BOOL_EXPRESSIONS_ONLY}?=> atomicExpressionForParserExpressionsOnly
	|	{self.allow_names}?=> name
	|	IDENT
	|	{not globals.PARSER_FT_BOOL_EXPRESSIONS_ONLY}?=> (TRUE | FALSE)
	|	LPAREN! expression RPAREN!
	|	{not globals.PARSER_FT_BOOL_EXPRESSIONS_ONLY}?=> CASE^ (expression COLON! expression SEMICOLON!)* (expression COLON! expression) OTHERWISE expression END!
	;
	
atomicExpressionForParserExpressionsOnly
    :   ENUM! COLON! i1=IDENT
		{
			$i1.setText("enum:" + $i1.text)
		}
	|	MODE! COLON! i1=IDENT
		{
			$i1.setText("mode:" + $i1.text)		
		}
	|	ERROR! COLON! i1=IDENT
		{
			$i1.setText("error:" + $i1.text)		
		}
  	|   MODE
 	       -> {CommonTree(CommonToken(type=IDENT,text=$MODE.text)) } 
 	|   ERROR
  	      -> {CommonTree(CommonToken(type=IDENT,text=$ERROR.text)) } 
        	|	i1=IDENT (DOT! i2=IDENT! {$i1.setText($i1.text + "." + $i2.text) } )* 
        		(DOT! (s=MODE!|s=ERROR!) {$i1.setText($i1.text + "." + $s.text) } )? 
	;
		
trajectoryEquation
	:	IDENT DERIVATIVE^ EQ! trajectoryExpression
	;
	
trajectoryExpression
	:	number (PER timeScale)?
	;
	
/* END expressions */
/* BEGIN mode transitions */
modeTransition
@init { self.rules.append("mode transition") }
@after { self.rules.pop() }
	:	sourceMode MINUS LBRACKET modeTrigger? (WHEN modeGuard)? (WITHIN timeDelay)? (THEN modeEffect)? RBRACKET ARROW IDENT SEMICOLON
		-> ^(MODE_TRANS sourceMode IDENT modeTrigger? (WHEN modeGuard)? (WITHIN timeDelay)? (THEN modeEffect)?)
	|	IDENT MINUS LBRACKET PROB NUMBER RBRACKET ARROW IDENT SEMICOLON
		-> ^(MODE_TRANS_PROB IDENT IDENT NUMBER)
	;

timeDelay
@init { self.rules.append("time delay")}
@after { self.rules.pop();}
	:	timeScaledNumber TO! timeScaledNumber
	;

sourceMode
    :   IDENT
    |   TIMES
    ;
	
modeTrigger
	:	nameOrIdentifierDisjunction
	|	RESET
	|	ATACTIVATION
	;
	
nameOrIdentifierDisjunction
    :   nameOrIdentifier (OR^ nameOrIdentifier)*
    ;
	
modeGuard
	:	expression
	;
	
modeEffect
	:	assignment (SEMICOLON! assignment)*
	;

effectsOnly
	:	{globals.PARSER_EFFECTS_ONLY}?=> modeEffect EOF!
	; 

assignmentsOnly
	:	{globals.PARSER_ASSIGNMENTS_ONLY}?=> assignment EOF!
	;
		
assignment
	:	IDENT ASSIGN^ expression
	|	{self.allow_names}?=> name ASSIGN^ expression
	;
	
inModes
	:	IN! MODES! LPAREN! identifierList RPAREN!
	;
/* END mode transitions */
/* BEGIN error features */	
errorFeature
	:	errorTypeState
	|	errorPropagation
	;
	
errorTypeState
@init { self.rules.append("error state") }
@after { self.rules.pop() }
	:	IDENT COLON stateType STATE SEMICOLON
		-> ^(stateType IDENT)
	;
	
stateType
	:	INITIAL
	|	ACTIVATION
	|	ERROR
	;
	
errorPropagation
@init { self.rules.append("error propagation") }
@after { self.rules.pop() }
	:	inPropagation
	|	outPropagation
	;
	
inPropagation
	:	IDENT COLON IN ERROR PROPAGATION SEMICOLON
		-> ^(IE_PROP IDENT)
	;
	
outPropagation
	:	IDENT COLON OUT ERROR PROPAGATION SEMICOLON
		-> ^(OE_PROP IDENT)
	;
	
occurrence
	:	OCCURRENCE! distribution number
	;
	
distribution
	:	POISSON
	;
/* END error features */
/* BEGIN error implementation contents */
errorEvent
@init { self.rules.append("error event") }
@after { self.rules.pop() }
	:	IDENT COLON ERROR EVENT occurrence? (PER timeScale)? SEMICOLON
		-> ^(ERR_EVENT IDENT occurrence? timeScale?)
	;

errorClock
@init { self.rules.append("error clock") }
@after { self.rules.pop() }
	:	IDENT COLON DATA CLOCK timeScale? SEMICOLON
		-> ^(ERR_CLOCK IDENT CLOCK timeScale?)
	;

errorState
@init { self.rules.append("error state") }
@after { self.rules.pop() }
	:	IDENT COLON stateType STATE urgent? (WHILE invariant)? SEMICOLON
		-> ^(stateType IDENT urgent? invariant?)
	;

errorTransition
@init { self.rules.append("error transition") }
@after { self.rules.pop() }
	:	sourceState MINUS LBRACKET errorTrigger (WHEN modeGuard)? (WITHIN timeDelay)? (THEN modeEffect)? RBRACKET ARROW IDENT SEMICOLON
		-> ^(ERR_TRANS sourceState errorTrigger IDENT (WHEN modeGuard)? (WITHIN timeDelay)? (THEN modeEffect)?)
	;

sourceState
    :   IDENT
    |   TIMES
    ;
	
errorTrigger
	:	IDENT
	|	RESET
	|	ATACTIVATION
	;
/* END error implementation contents */				
/* ****** end parser *************************************************** */


/* ****** start lexer ************************************************** */

IDENT
	: 	{globals.PARSER_ACCEPT_EXTENDED_MODEL}?=>('_'|'#')ALPHANUM
	|	ALPHANUM
	;

fragment ALPHANUM
	:	('A'..'Z'|'a'..'z')('A'..'Z'|'a'..'z'|'0'..'9'|'_')*
	;
	
NUMBER
	:	(('0'..'9')+ ('..'))=> ('0'..'9')+
	|   ('0'..'9')+ (('.') ('0'..'9')+)?
	;
			
/* ------ ignored ------------------------------------------------------ */
COMMENT
	:	'/*' ( options {greedy=false;} : . )* '*/'
		{ $channel=HIDDEN; }
	|	'--' ~('\r'|'\n')* ('\r'|'\n')?
		{ $channel=HIDDEN; }
	;
WS	
	:	(' '|'\r'|'\t'|'\u000C'|'u'|'\n')
		{ $channel=HIDDEN; }
	;

/* ****** end lexer **************************************************** */
