/**
# =============================================================================
#  This file is part of the ``COMPASS'' Toolset.
#  Copyright (C) 2010 by Fondazione Bruno Kessler
#  Copyright (C) 2010 by RWTH Aachen University
#
#
#  The COMPASS Toolset can be copied, modified and/or redistributed
#  under the terms of the COMPASS General Public License, either
#  version 1 of the License, or (at your option) any later version.
#
#  You should have received a copy of the COMPASS General Public
#  License along with this software. If not, please contact the
#  copyright owners.
#
#  The COMPASS Toolset is distributed in the hope that it will be
#  useful, but WITHOUT ANY WARRANTY; without even the implied warranty
#  of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  COMPASS General Public License for more details.
#
#  For more information on the COMPASS Project see the Project Web
#  site at <http://compass.informatik.rwth-aachen.de/> or email to
#  <compass-support@lists.rwth-aachen.de>.  
#
#  Please report bugs to <compass-support@lists.rwth-aachen.de>.
# =============================================================================

**/

tree grammar SlimWalkerIdentCheck;

options
{
	language=Python;
	tokenVocab=Slim;
	ASTLabelType=CommonTree;
	backtrack=true;
}

@header {

from slim_compiler.src.dataStructure.compile import *
from slim_compiler.src.dataStructure.nominalbehaviour import *
from slim_compiler.src.dataStructure.errorbehaviour import *
from slim_compiler.src.dataStructure.expressions import *
from slim_compiler.src.errorManagement.errors import *
}

specification [compileInfo]
	scope {
		the_package
	}
	:	(packageSpecification[compileInfo] 
	|	{
			package = $compileInfo.add_package("__default__", 0, 0)
			$specification::the_package = package.get_public_subpackage()
		}
		declaration[compileInfo])+
	;
	
packageSpecification [compileInfo]
	:	^(PACKAGE 
				// Package identifier
				i=ident
				
		{
			#add the package if it was not already declared
			package = $compileInfo.add_package($i.name,
								$i.start.line, $i.start.charPositionInLine + 1)
			$specification::the_package = package
		}
			
			PUBLIC
		{
			$specification::the_package = package.get_public_subpackage()
		} 
				// Public declarations 
				declaration[compileInfo]+ 
			(PRIVATE 
			{
				$specification::the_package = package.get_private_subpackage()
			}
				// Private declarations 
				declaration[compileInfo]+
			)?
			// end identifier
			j=ident
			
		{
			if $i.name != $j.name:
            	$compileInfo.add_error(
            			EndIdentifierOfPackageDoesNotMatchError($compileInfo.current_file,
                	                                            $j.start.line,
                    	                                        $j.start.charPositionInLine + 1,
                    	                                        "A-4", $specification::the_package.parent,
                        	                                    $j.name))
		}
	
		)
	;	

declaration [compileInfo]
	scope {
		the_component
	}
		// component type declaration
	:	^(COMP_TYPE
		{
			fdir = False
		}
				(FDIR { fdir = True })? 
				cc=componentCategory 
				// identifier 
				i=ident

		{	
			if $specification::the_package:
				$declaration::the_component = \
					$specification::the_package.add_component_type($i.name,
					Category.str_to_category($cc.name), fdir, $i.start.line,
					$i.start.charPositionInLine + 1)
		}

				// defined ports 
				portSpec[compileInfo]*
				// end identifier	
				j=ident
				
		{
			if $declaration::the_component:
				if $i.name != $j.name:
            		$compileInfo.add_error(
                    	EndIdentifierOfComponentTypeDoesNotMatchError($compileInfo.current_file,
                                                                  	  $j.start.line,
                                                                  	  $j.start.charPositionInLine + 1,
                                                                  	  "B-3", $declaration::the_component,
                                                                  	  $j.name))
		}				
				
		)
	|	^(node=COMP_IMPL
		{
			fdir = False
		}
				(FDIR { fdir = True })?
				cc=componentCategory 
				// identifier 
				i=ident
				
		{
			if $specification::the_package:
				$declaration::the_component = \
					$specification::the_package.add_component_implementation(
					$i.name, Category.str_to_category($cc.name), fdir, $i.start.line,
					$i.start.charPositionInLine + 1)
		}
		
				// used subcomponents 
				subcomponent[compileInfo]* 
				// connections between ports 
				portConnection*
				flow*
				
		{
			has_modes = False
		}
				  
				// declared modes 
				(mode {has_modes = True})*
				
		{
			if not has_modes:
				if $declaration::the_component:
						$declaration::the_component.add_initial_mode(
								DEFAULTINITMODE, None, None, -1, -1)
		} 
				// transitions between modes 
				modeTransition*
				// end identifier
				j=ident
				
		{
			if $declaration::the_component:
				if $i.name != $j.name:
            		$compileInfo.add_error(
                    	EndIdentifierOfComponentImplementationDoesNotMatchError(
                                                                    $compileInfo.current_file,
                                                                    $j.start.line,
                                                                    $j.start.charPositionInLine + 1,
                                                                    "E-6", $declaration::the_component,
                                                        			$j.name))        
				
		}
								
		) 
	|	^(ERR_TYPE 
				// identifier 
				i=ident
				
		{
			if $specification::the_package:
				$declaration::the_component = \
					$specification::the_package.add_error_type($i.name,
					$i.start.line, $i.start.charPositionInLine + 1)
		}
								 
				// defined error states and propagations 
				errorFeature*
				// end identifier
				j=ident
				
		{
			if $declaration::the_component:
				if $i.name != $j.name:
					$compileInfo.add_error(
						EndIdentifierOfErrorTypeDoesNotMatchError(
								$compileInfo.current_file, $j.start.line,
								$j.start.charPositionInLine + 1, "J-4", $declaration::the_component,
								$j.name))
																				
		}				
				
		)
	|	^(ERR_IMPL 
				// identifier
				i=ident
				
		{
			if $specification::the_package:
				$declaration::the_component = \
					$specification::the_package.add_error_implementation($i.name,
					$i.start.line, $i.start.charPositionInLine + 1)
		}		
				// error clocks
				errorClock[compileInfo]*				
				// error states 
				errorState* 
				// error events
				errorEvent* 
				// error transitions
				errorTransition*
				// end identifier
				j=ident
				
		{
			if $declaration::the_component:
				if $i.name != $j.name:
					$compileInfo.add_error(
						EndIdentifierOfErrorImplementationDoesNotMatchError(
								$compileInfo.current_file, $j.start.line,
								$j.start.charPositionInLine + 1, "K-11", $declaration::the_component,
								$j.name))
		}
		)
	;

// component categories	
componentCategory returns [name]
	:
	|	i=PROCESS
	{
		$name = $i.text
	}
	|	i=THREAD_GROUP
	{
		$name = "thread group"
	}
	|	i=THREAD
	{
		$name = $i.text
	}
	|	i=PROCESSOR
	{
		$name = $i.text
	}
	|	i=MEMORY
	{
		$name = $i.text
	}
	|	i=DEVICE
	{
		$name = $i.text
	}
	|	i=BUS
	{
		$name = $i.text
	}
	|	i=SYSTEM
	{
		$name = $i.text
	}
	;
	
// datatypes	
dataType [compileInfo] returns [dataType]
	:	i=BOOL
	{
		$dataType = Bool($compileInfo.current_file, $i.line,
						 $i.charPositionInLine + 1)
	}
	|	
	{
		time_unit = None
	}
		i=CLOCK (ts=timeScaleUnit{time_unit=$ts.unit})?
	{
		$dataType = Clock(time_unit, $compileInfo.current_file, $i.line,
						 $i.charPositionInLine + 1)
	}
	|	i=CONTINUOUS
	{
		$dataType = Continuous($compileInfo.current_file, $i.line,
						 $i.charPositionInLine + 1)
	}
	|		
	{
		values = []
	}
		^(node=ENUM 
				// identifiers representing values
				(elem=ident { values.append($elem.name) })+
		)
		{
			$dataType = Enum(values, $compileInfo.current_file, $node.line,
							 $node.charPositionInLine + 1)
		}
	|	i=INT
	{
		$dataType = Int($compileInfo.current_file, $i.line,
						 $i.charPositionInLine + 1)
	}
	|	i=REAL
	{
		$dataType = Real($compileInfo.current_file, $i.line,
						 $i.charPositionInLine + 1)
	}
	|   r=intRange 
	{
	    dataType = IntRange($r.lower, $r.upper, $compileInfo.current_file,
	    					$r.start.line, $r.start.charPositionInLine + 1)
	}
	;

// Values conforming to expressions.TimeUnitType
timeScaleUnit returns [unit]
	:	i=MSEC
	{
		$unit = TimeUnitType.MSEC
	}
	|	i=SEC
	{
		$unit = TimeUnitType.SEC
	}
	|	i=MIN
	{
		$unit = TimeUnitType.MIN
	}
	|	i=HOUR
	{
		$unit = TimeUnitType.HOUR
	}
	|	i=DAY
	{
		$unit = TimeUnitType.DAY
	}
	;

intRange returns [lower, upper]
    :   low=NUMBER up=NUMBER
    {
	    $lower = int($low.text)
	    $upper = int($up.text)
	}
    ;
	
portSpec [compileInfo]
	// incoming event port
	:	^(IEPORT 
				i=ident
		)
		
	{
		if $declaration::the_component:
			$declaration::the_component.add_port($compileInfo.current_file, $i.name, True, True, False, None, None,
				False, False, $i.start.line, $i.start.charPositionInLine + 1)
	}
	
	// outgoing event port
	|	^(OEPORT 
				i=ident
	{
		nonblocking = False
	}
				(NONBLOCKING {nonblocking = True})?
	{
		if $declaration::the_component:
			$declaration::the_component.add_port($compileInfo.current_file, $i.name, False, True, nonblocking, None, None,
				False, False, $i.start.line, $i.start.charPositionInLine + 1)
	}
		)
	// incoming data port
	|	^(IDPORT 
				i=ident 
				// type of data port
				d=dataType[compileInfo]
				// initialization value 
				initval=expr[True]?
	{
		if $declaration::the_component:
			$declaration::the_component.add_port($compileInfo.current_file, $i.name, True, False, False, $d.dataType,
				$initval.value, False, False, $i.start.line,
				$i.start.charPositionInLine + 1)	
	}
				
		)
	// outgoing data port
	|	^(ODPORT 
				i=ident 
				// type of data port
				d=dataType[compileInfo]
				// initialization value 
				initval=expr[True]?
	{
		observable = False
		alarm = False
	}
				(OBSERVABLE {observable = True})?
				(ALARM {alarm = True})?

	{
		if $declaration::the_component:
			$declaration::the_component.add_port($compileInfo.current_file, $i.name, False, False, False, $d.dataType,
				$initval.value, observable, alarm, $i.start.line,
				$i.start.charPositionInLine + 1)	
	}
		)
		// error event port
	|	^(ERRORPORT i=ident
	{
		distribution=None
		param=None
	}
		(POISSON {distribution=$POISSON.text} NUMBER {param=$NUMBER.text})?
	{
		if $declaration::the_component:
			$declaration::the_component.add_error_event_port($i.name, distribution, param, None, 
				$i.start.line, $i.start.charPositionInLine + 1)
	}
		)
	;
	
ident returns [name]
	// implementation identifier
	:	^(NAME 
				// error or component type
				type=ident 
				// identifier 
				i=ident
		)
		{
			$name = $type.text + "." + $i.text
		}
	// package name/identifier
	|	^(FQ_NAME_OR_IDENT 
				// package name
				package=ident 
				// identifier/name
				n=ident
		)
		{
			if $package.text != "": 
				$name = $package.text + "::" + $n.text
			else:
				$name = $n.text
		}
	|	IDENT
		{
			$name = $IDENT.text
		}
	;

subcomponent [compileInfo]
	// data subcomponent
	:	^(DATA 		
				// local identifier
				i=ident 
		{
			in_modes = []
		}
				// modes for which this data component is active
				(m=ident { in_modes.append($m.name) })*
				
				// type
			 	type=dataType[compileInfo] 
				// initialization value
				initval=expr[True]?
		{
			if $declaration::the_component:
				$declaration::the_component.add_data_subcomponent($i.name,
					$type.dataType, $initval.value, in_modes,
					$i.start.line, $i.start.charPositionInLine + 1)
		}
				
		)
	// all remaining other component categories
	|	^(cc=componentCategory	
				// local identifier
				i=ident 
				// component implementation to which this component refers
				impl=ident
		{
			in_modes = []
		}
				// modes for which this component is active
				(m=ident { in_modes.append($m.name) })*
		{
			stored_in = []
			running_on = []
			accesses = []
		} 
				// bindings
				(bind=binding
				{
					if $bind.type == "storedIn":
						stored_in.append($bind.name)
					elif $bind.type == "runningOn":
						running_on.append($bind.name)
					elif $bind.type == "accesses":
						accesses.append($bind.name)
				}
				)*
		{
			if $declaration::the_component:
				$declaration::the_component.add_non_data_subcomponent($i.name,
					Category.str_to_category($cc.name), $impl.name, accesses,
					stored_in, running_on, in_modes, $i.start.line,
					$i.start.charPositionInLine + 1)
		}
		)
 	;
 	

binding returns [type, name]
	// stored in memory binding
	:	^(STORED 
				// identifier of memory implementation
				i=ident
				
		{
			$type = "storedIn"
			$name = $i.name
		}
		)
	// running on processor binding
	|	^(RUNNING 
				// identifier of processor implementation
				i=ident
		{
			$type = "runningOn"
			$name = $i.name
		}
		)
	// accesses bus binding
	|	^(ACCESSES 
				// identifier of bus implementation
				i=ident
		{
			$type = "accesses"
			$name = $i.name
		}
		)
	;

portConnection
// event connection
	:	^(EVENT_CONNECTION
	
		{
			in_modes = []
			passive = False;
		}
	
				// from port 
				fromPort=ident 
				// to port 
				toPort=ident 
				// modes for which this connection holds
				(i=ident { in_modes.append($i.name) })*
				// is the connection passive?
				(PASSIVE { passive = True } )?
				
				{
					if $declaration::the_component:
						$declaration::the_component.add_event_port_connection(
						$fromPort.name, $toPort.name, in_modes, passive,
						$fromPort.start.line,
						$fromPort.start.charPositionInLine + 1)
				}
		)
	|	^(DATA_CONNECTION
	
		{
			in_modes = []
		}
				// from port 
				fromPort=ident 
				// to port 
				toPort=ident 
				// modes for which this connection holds
				(i=ident  { in_modes.append($i.name) })*
				
				{
					if $declaration::the_component:
						$declaration::the_component.add_data_port_connection(
						$fromPort.name, $toPort.name, in_modes,
						$fromPort.start.line,
						$fromPort.start.charPositionInLine + 1)			
				}
		)
	;
	
flow
	:	^(FLOW
		{
			in_modes = []
		}
			i=ident
			e=expr[True]
			(m=ident { in_modes.append($m.name) })*
		{
			if $declaration::the_component:
				$declaration::the_component.add_flow(BinaryOperatorTreeNode(
														IdentifierTreeNode($i.name, $i.start.line, $i.start.charPositionInLine + 1),
														$e.value,
														OperatorType.ASSIGN,
													 	$i.start.line, $i.start.charPositionInLine + 1),
												in_modes, $i.start.line, $i.start.charPositionInLine + 1)
		}
		)
	;
	
mode
// initial mode
	:	^(INITIAL
				// identifier 
				i=ident
				// urgent
				u=urgent?
				// invariant 	
				invariant=expr[True]?
				
				{
					if $declaration::the_component:
						$declaration::the_component.add_initial_mode(
								$i.name, $invariant.value, $u.value, $i.start.line,
								$i.start.charPositionInLine + 1)
				}
				
		)
// activation mode
	|	^(ACTIVATION 
				// identifier 
				i=ident 
				// urgent
				u=urgent?
				// invariant 	
				invariant=expr[True]?
				
				{
					if $declaration::the_component:
						$declaration::the_component.add_activation_mode(
								$i.name, $invariant.value, $u.value, $i.start.line,
								$i.start.charPositionInLine + 1)
				}
		)
	|	^(ERROR
				// identifier
				i=ident
				// urgent
				u=urgent?
				// invariant
				invariant=expr[True]?

				{
					if $declaration::the_component:
						$declaration::the_component.add_error_mode(
								$i.name, $invariant.value, $u.value, $i.start.line,
								$i.start.charPositionInLine + 1)
						
						$declaration::the_component.is_error_component_implementation = True
				}
				
		)
// a non-initial, non-activation mode, and non-error mode i.e., normal mode
	|	^(NORMAL 
				// identifier
				i=ident
				// urgent
				u=urgent?				
				// invariant
				invariant=expr[True]?
				
				{
					if $declaration::the_component:
						$declaration::the_component.add_mode(
								$i.name, $invariant.value, $u.value, $i.start.line,
								$i.start.charPositionInLine + 1)
				}
)
	;

urgent returns [value]
	:	^(URGENT bound=expr[True]
		{
			$value = $bound.value
		})
	;

// expressions	
expr [evaluate] returns [value]
	:	^(NOT exp=expr[evaluate])
		{
			if not $evaluate:
				$value = "not " + $exp.value
			else:
				$value = UnaryOperatorTreeNode($exp.value, OperatorType.NOT,
											   $exp.start.line,
									 		   $exp.start.charPositionInLine + 1) 
		}
	|	^(IMP lexp=expr[evaluate] rexp=expr[evaluate])
		{
			if not $evaluate:
				$value = $lexp.value + " imp " + $rexp.value
			else:
				$value = BinaryOperatorTreeNode($lexp.value, $rexp.value,
										  OperatorType.IMP, $lexp.start.line,
										  $lexp.start.charPositionInLine + 1)
		}
	|	^(IFF lexp=expr[evaluate] rexp=expr[evaluate])
		{
			if not $evaluate:
				$value = $lexp.value + " iff " + $rexp.value
			else:
				$value = BinaryOperatorTreeNode($lexp.value, $rexp.value,
										  OperatorType.IFF, $lexp.start.line,
										  $lexp.start.charPositionInLine + 1)
		}
	|	^(XNOR lexp=expr[evaluate] rexp=expr[evaluate])
		{
			if not $evaluate:
				$value = $lexp.value + " xnor " + $rexp.value
			else:
				$value = BinaryOperatorTreeNode($lexp.value, $rexp.value,
										  OperatorType.XNOR, $lexp.start.line,
										  $lexp.start.charPositionInLine + 1)
		}
	|	^(OR lexp=expr[evaluate] rexp=expr[evaluate])
		{
			if not $evaluate:
				$value = $lexp.value + " or " + $rexp.value
			else:
				$value = BinaryOperatorTreeNode($lexp.value, $rexp.value,
										  OperatorType.OR, $lexp.start.line,
										  $lexp.start.charPositionInLine + 1)		
		}
	|	^(AND lexp=expr[evaluate] rexp=expr[evaluate])
		{
			if not $evaluate:
				$value = $lexp.value + " and " + $rexp.value
			else:
				$value = BinaryOperatorTreeNode($lexp.value, $rexp.value,
										  OperatorType.AND, $lexp.start.line,
										  $lexp.start.charPositionInLine + 1)
		}
	|	^(XOR lexp=expr[evaluate] rexp=expr[evaluate])
		{
			if not $evaluate:
				$value = $lexp.value + " xor " + $rexp.value
			else:
				$value = BinaryOperatorTreeNode($lexp.value, $rexp.value,
										  OperatorType.XOR, $lexp.start.line,
										  $lexp.start.charPositionInLine + 1)
		}
	|	^(EQ lexp=expr[evaluate] rexp=expr[evaluate])
		{
			if not $evaluate:
				$value = $lexp.value + " == " + $rexp.value
			else:
				$value = BinaryOperatorTreeNode($lexp.value, $rexp.value,
										  OperatorType.EQUAL,
										  $lexp.start.line,
										  $lexp.start.charPositionInLine + 1)
		}
	|	^(NEQ lexp=expr[evaluate] rexp=expr[evaluate])
		{
			if not $evaluate:
				$value = $lexp.value + " != " + $rexp.value
			else:
				$value = BinaryOperatorTreeNode($lexp.value, $rexp.value,
										  OperatorType.NOTEQUAL,
										  $lexp.start.line,
										  $lexp.start.charPositionInLine + 1)
		}
	|	^(LT lexp=expr[evaluate] rexp=expr[evaluate])
		{
			if not $evaluate:
				$value = $lexp.value + " < " + $rexp.value
			else:
				$value = BinaryOperatorTreeNode($lexp.value, $rexp.value,
										  OperatorType.LESS, $lexp.start.line,
										  $lexp.start.charPositionInLine + 1)
		}
	|	^(LE lexp=expr[evaluate] rexp=expr[evaluate])
		{
			if not $evaluate:
				$value = $lexp.value + " <= " + $rexp.value
			else:
				$value = BinaryOperatorTreeNode($lexp.value, $rexp.value,
										  OperatorType.LESSOREQUAL,
										  $lexp.start.line,
										  $lexp.start.charPositionInLine + 1)
		}
	|	^(GT lexp=expr[evaluate] rexp=expr[evaluate])
		{
			if not $evaluate:
				$value = $lexp.value + " > " + $rexp.value
			else:
				$value = BinaryOperatorTreeNode($lexp.value, $rexp.value,
										  OperatorType.GREATER,
										  $lexp.start.line,
										  $lexp.start.charPositionInLine + 1)
		}
	|	^(GE lexp=expr[evaluate] rexp=expr[evaluate])
		{
			if not $evaluate:
				$value = $lexp.value + " >= " + $rexp.value
			else:
				$value = BinaryOperatorTreeNode($lexp.value, $rexp.value,
										  OperatorType.GREATEROREQUAL,
										  $lexp.start.line,
										  $lexp.start.charPositionInLine + 1)
		}
	|	^(PLUS lexp=expr[evaluate] rexp=expr[evaluate])
		{
			if not $evaluate:
				$value = $lexp.value + " + " + $rexp.value
			else:
				$value = BinaryOperatorTreeNode($lexp.value, $rexp.value,
										  OperatorType.PLUS,
										  $lexp.start.line,
										  $lexp.start.charPositionInLine + 1)
		}
	|	^(MINUS lexp=expr[evaluate] rexp=expr[evaluate])
		{
			if not $evaluate:
				$value = $lexp.value + " - " + $rexp.value
			else:
				$value = BinaryOperatorTreeNode($lexp.value, $rexp.value,
										  OperatorType.MINUS,
										  $lexp.start.line,
										  $lexp.start.charPositionInLine + 1)
		}
	|	^(TIMES lexp=expr[evaluate] rexp=expr[evaluate])
		{
			if not $evaluate:
				$value = $lexp.value + " * " + $rexp.value
			else:
				$value = BinaryOperatorTreeNode($lexp.value, $rexp.value,
										  OperatorType.TIMES,
										  $lexp.start.line,
										  $lexp.start.charPositionInLine + 1)
		}
	|	^(DIVIDE lexp=expr[evaluate] rexp=expr[evaluate])
		{
			if not $evaluate:
				$value = $lexp.value + " / " + $rexp.value
			else:
				$value = BinaryOperatorTreeNode($lexp.value, $rexp.value,
										  OperatorType.DIVIDE,
										  $lexp.start.line,
										  $lexp.start.charPositionInLine + 1)
		}
	| 	^(MOD lexp=expr[evaluate] rexp=expr[evaluate])
		{
			if not $evaluate:
		        	$value = $lexp.value + " mod " + $rexp.value
			else:
		        	$value = BinaryOperatorTreeNode($lexp.value, $rexp.value,
					OperatorType.MOD,
					$lexp.start.line,
					$lexp.start.charPositionInLine + 1)
		}		
	|	^(c=CASE
		{
			cases = []
		}
			(lexp=expr[evaluate] rexp=expr[evaluate]
			{
				cases.append(($lexp.value, $rexp.value))
			}
			)+ 
			OTHERWISE 
			default=expr[evaluate]
		{
			if not $evaluate:
				$value = "case "
	for (cond, res) in cases:
		$value += cond + " : " + res + "; "
		$value += "otherwise : " + $default.value
		$value += " end"
			else:
				$value = CaseExpressionTreeNode(cases, $default.value,
												$c.line,
												$c.charPositionInLine + 1)
					
		}
		)
// trajectory equation
	|	^(DERIVATIVE 
				// variable
				i=ident 
				// flow expression
				trajExpr=expr[evaluate]
				{time_unit=None}
				(PER ts=timeScaleUnit{time_unit=$ts.unit})?
		)
		{
			if not $evaluate:
				$value = $i.name + "\' = " + $trajExpr.value
			else:
				$value = TrajectoryEquationTreeNode(
							IdentifierTreeNode($i.name, $i.start.line,
											   $i.start.charPositionInLine + 1),
							$trajExpr.value, $i.start.line,
							$i.start.charPositionInLine + 1,
							time_unit)
		}
// trajectory expression
// Note: this one seems to be deprecated as it is never used
	|	^(TRAJ_EXPR
				// variable
				n=NUMBER
		)
		{	
			$value = TrajectoryExpressionTreeNode(
							None,
							None, int($n.text), $n.start.line,
							$n.start.charPositionInLine + 1)
		}
	|	n=TRUE
		{
			if not $evaluate:
				$value = "true"
			else:
				$value = BooleanTreeNode(True, $n.line,
										 $n.charPositionInLine + 1)
		}
	|	n=FALSE
		{
			if not $evaluate:
				$value = "false"
			else:
				$value = BooleanTreeNode(False, $n.line,
										 $n.charPositionInLine + 1)
		}
	|	{
			t = None
		}
		n=NUMBER
		{
			if not $evaluate:
				$value = $n.text
			else:
				$value = NumberTreeNode($n.text, None,
										$n.line,
										$n.charPositionInLine + 1) 
		}
	|	i=ident
		{
			if not $evaluate:
				$value = $i.name
			else:
				$value = IdentifierTreeNode($i.name, $i.start.line,
											$i.start.charPositionInLine + 1)
		}
	|	^(NAME i1=ident i2=ident)
		{
			if not $evaluate:
				$value = $i1.name + "." + $i2.name
			else:
				$value = IdentifierTreeNode($i1.name + "." + $i2.name,
											$i1.start.line,
											$i.start.charPositionInLine + 1)
		}
	|	^(UNARY_MINUS e=expr[evaluate])
		{
			if not $evaluate:
				$value = "-" + $i1.name
			else:
				$value = UnaryOperatorTreeNode(
								$e.value,
								OperatorType.UNARY_MINUS,
								$e.start.line,
								$e.start.charPositionInLine + 1)
		}
	|	e=timeScaleOperator[evaluate]
		{
			$value = $e.value
		}
	;

timeScaleOperator [evaluate] returns [value]
	:	^(i=MSEC exp=expr[evaluate])
	{
		if not $evaluate:
			$value = $exp.value + " msec"
		else:
			$value = UnaryOperatorTreeNode($exp.value, OperatorType.MSEC,
			                               $i.token.line,
			                               $i.token.charPositionInLine + 1)
	}
	|	^(i=SEC exp=expr[evaluate])
	{
		if not $evaluate:
			$value = $exp.value + " sec"
		else:
			$value = UnaryOperatorTreeNode($exp.value, OperatorType.SEC,
			                               $i.token.line,
			                               $i.token.charPositionInLine + 1)
	}
	|	^(i=MIN exp=expr[evaluate])
	{
		if not $evaluate:
			$value = $exp.value + " min"
		else:
			$value = UnaryOperatorTreeNode($exp.value, OperatorType.MIN,
			                               $i.token.line,
			                               $i.token.charPositionInLine + 1)
	}
	|	^(i=HOUR exp=expr[evaluate])
	{
		if not $evaluate:
			$value = $exp.value + " hour"
		else:
			$value = UnaryOperatorTreeNode($exp.value, OperatorType.HOUR,
			                               $i.token.line,
			                               $i.token.charPositionInLine + 1)
	}
	|	^(i=DAY exp=expr[evaluate])
	{
		if not $evaluate:
			$value = $exp.value + " day"
		else:
			$value = UnaryOperatorTreeNode($exp.value, OperatorType.DAY,
			                               $i.token.line,
			                               $i.token.charPositionInLine + 1)
	}
	;

sourceMode returns [value]
	:   TIMES
		{
	    	$value = "*"
		}
    |   i=ident
    	{
			$value = $i.name 
		}
	;

// transition between a two modes	
modeTransition
	:	^(MODE_TRANS 
			{
				effects = []
				guard = None
				triggers = [None]
				time_delay = None
			}
				// from mode
				fMode=sourceMode
				// to mode
				toMode=ident
				// trigger
				(triggers=modeTrigger { triggers = $triggers.value } )?
			(WHEN 
				// guards 
				guard=modeGuard { guard = $guard.value} )?
			(WITHIN
				lower=expr[True] upper=expr[True] {time_delay = ($lower.value, $upper.value)})?
			(THEN 
				// effects 
				(effect=modeEffect { effects.append($effect.value) })+)?
				
			{
				if $declaration::the_component:
				    # for trigger in triggers:
				        if triggers[0] == '@activation':
				            $declaration::the_component.add_reactivation_mode_transition(
				                $fMode.value, $toMode.name,
				                guard, time_delay, effects, $fMode.start.line,
				                $fMode.start.charPositionInLine + 1)
				        else:
				            $declaration::the_component.add_normal_mode_transition(
				                $fMode.value, $toMode.name, triggers,
				                guard, time_delay, effects, $fMode.start.line,
				                $fMode.start.charPositionInLine + 1)
			}	
				
		)
	|	^(MODE_TRANS_PROB
				//from mode
				fromMode=ident
				//to mode
				toMode=ident
				//probabilty
				prob=expr[True]
				
			{
				if $declaration::the_component:
					$declaration::the_component.add_probability_mode_transition(
							$fromMode.name, $toMode.name, $prob.value,
							$fromMode.start.line,
							$fromMode.start.charPositionInLine + 1)
			}
		)
	;
		
modeTrigger returns [value]
// event identifier
	:	i=ident
		{
			$value = [$i.name]
		}
	|   ^(OR trig1=modeTrigger trig2=modeTrigger)
	    {
	        $value = $trig1.value + $trig2.value 
	    }
// reset 
	|	RESET
		{
			$value = ["reset"]
		}
// reactivation
	|	ATACTIVATION
		{
			$value = ["@activation"]
		}
	;
	
modeGuard returns [value]
// guard as a boolean expression 
	:	e=expr[True]
		{
			$value = $e.value 
		}
	;

modeEffect returns [value]
	:	^(ASSIGN 
				// data port/component that is changed
				data=ident 
				expression=expr[True]
		)
		{
			$value = BinaryOperatorTreeNode(IdentifierTreeNode($data.name,
					$data.start.line, $data.start.charPositionInLine + 1),
				  	$expression.value, OperatorType.ASSIGN, $data.start.line,
				  	$data.start.charPositionInLine + 1)
		}
	;

errorFeature
	:	errorState
	|	errorPropagation
	;

errorClock[compileInfo]
	:	^(ERR_CLOCK
				i=ident
			 	clockType=dataType[compileInfo] //always clock
				{
					if $declaration::the_component:
						$declaration::the_component.add_clock($i.name, clockType,
							$i.start.line, $i.start.charPositionInLine + 1)
				}
		)
	;
	
errorState
// initial error state
	:	^(INITIAL 
				i=ident
				// urgent
				u=urgent?
				// invariant
				invariant=expr[True]?
				{
					if $declaration::the_component:
						$declaration::the_component.add_initial_state(
							$i.name, $invariant.value, $u.value, $i.start.line,
							$i.start.charPositionInLine + 1)
				}
		) 
// activation error state
	|	^(ACTIVATION 
				i=ident
				// urgent
				u=urgent?				
				// invariant
				invariant=expr[True]?
				{
					if $declaration::the_component:
						$declaration::the_component.add_activation_state(
							$i.name, $invariant.value, $u.value, $i.start.line,
							$i.start.charPositionInLine + 1)
				}
		)
// error state
	|	^(ERROR 
				i=ident
				// urgent
				u=urgent?				
				// invariant
				invariant=expr[True]?
				{
					if $declaration::the_component:
						$declaration::the_component.add_error_state(
							$i.name, $invariant.value, $u.value, $i.start.line,
							$i.start.charPositionInLine + 1)
				}
		)
	;
	
errorPropagation
// incoming event propagation
	:	^(IE_PROP 
				i=ident
				{
					if $declaration::the_component:
						$declaration::the_component.add_in_propagation($i.name,
							$i.start.line, $i.start.charPositionInLine + 1)
				}
		)
// outgoing event propagation
	|	^(OE_PROP 
				i=ident 
				{
					if $declaration::the_component:
						$declaration::the_component.add_out_propagation($i.name,
							$i.start.line, $i.start.charPositionInLine + 1)
				}
		)
	;
	
occurrence returns [dist, rate, time]
	:	d=distribution n=NUMBER
	
		{
			$dist = $d.dist
			$rate = $n.text
			# TODO: check why time is unused
		}
	;
	
distribution returns[dist]
	:   POISSON
		{
			$dist = DistributionType.POISSON
		}
	;

errorEvent
	:	^(ERR_EVENT
				{
					myRate = None
					myDist = None
					myTime = None # TODO: get rid of this deprecated stuff
					time_unit = None
				} 
				i=ident 
				(occ=occurrence  { myRate = $occ.rate; myDist = $occ.dist; myTime = $occ.time} )?
				(ts=timeScaleUnit{time_unit=$ts.unit})?
				{
					if $declaration::the_component:
						$declaration::the_component.add_event($i.name,
						myDist, myRate, time_unit, $i.start.line, $i.start.charPositionInLine + 1)
				}
		)
	;
	
errorTransition
	:	^(ERR_TRANS 
				fromState=sourceState 
				trigger=errorTrigger 
				toState=ident
				{
					effects = []
					guard = None
					time_delay = None
				}
				(WHEN 
					// guards 
					guard=modeGuard { guard = $guard.value} )? 
				(WITHIN
					lower=expr[True] upper=expr[True] {time_delay = ($lower.value, $upper.value)})?
				(THEN 
					// effects 
					(effect=modeEffect { effects.append($effect.value) })+)?
				{
					if $declaration::the_component:
					    if trigger == '@activation':
					        $declaration::the_component.add_reactivation_transition($fromState.name,
					            $toState.name, guard, time_delay, effects, $fromState.start.line, $fromState.start.charPositionInLine + 1)
					    else:
					        $declaration::the_component.add_transition($fromState.name,
					        $trigger.name, $toState.name, guard, time_delay, effects, $fromState.start.line, $fromState.start.charPositionInLine + 1)
				}
		)
	;
	
sourceState returns [name]
    :   i=ident
    {
	    $name = $i.name 
	}
    |   TIMES
    {
		$name = "*"
	}
    ;
	
errorTrigger returns [name]
	:	IDENT
		{
			$name = $IDENT.text
		}
	|	RESET
		{
			$name = "reset"
		}
	|	ATACTIVATION
		{
			$name = "@activation"
		}
	;
