# =============================================================================
#  This file is part of the ``HASDEL'' Toolset.
#  Copyright (C) 2014 by Fondazione Bruno Kessler
#  Copyright (C) 2014 by RWTH Aachen University
#
#
#  The HASDEL Toolset can be copied, modified and/or redistributed
#  under the terms of the HASDEL General Public License, either
#  version 1 of the License, or (at your option) any later version.
#
#  You should have received a copy of the HASDEL General Public
#  License along with this software. If not, please contact the
#  copyright owners.
#
#  The HASDEL Toolset is distributed in the hope that it will be
#  useful, but WITHOUT ANY WARRANTY; without even the implied warranty
#  of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  HASDEL General Public License for more details.
#
#  For more information on the HASDEL Project see the Project Web
#  site at <http://compassinternal.informatik.rwth-aachen.de/hasdel-site/> 
#  or email to <hasdel-support@lists.rwth-aachen.de>.  
#
#  Please report bugs to <hasdel-support@lists.rwth-aachen.de>.
# =============================================================================


"""This module provides some constants for the model extension package."""
# TODO: In this file, a maximum line length of 80 is used, instead of 79 as
# specified in PEP8.

DEFAULT_STARTING_STATE_NAME = "_startingState"
"""The default name for the starting state of an error model.
If an error model has not error states at all, the corresponding error
subcomponent of the extended model will have an artificial starting state with
this default name.
@type: string
"""

ERROR_STATES_ENUM_PORT_NAME = "_errorState"
"""The name of the enum ports that are used to indicate the current error state
of an extended component or its error subcomponent.
Note: This name is guaranteed to be unique among the port names of a component
because it starts with "_", which is not allowed for usual port names.
@type: string
"""

ERROR_SUBCOMPONENT_IDENTIFIER = "_errorSubcomponent"
"""The identifier that is used to refer to an error subcomponent from within
its supercomponent.
Note: This identifier is guaranteed to be unique among the subcomponent
identifiers of any given supercomponent, because it starts with "_", which is
not allowed for usual subcomponent identifiers.
@type: string
"""

ERROR_COMPONENT_CATEGORY = "system"
"""The name of the category to be used for error subcomponents.
@type: string
"""

RESET_TRIGGER = "reset"
"""The special trigger "reset" which can be used in mode and error transitions.
@type: string
"""

RESET_PORT_NAME = "_resetEvent"
"""The name of the in event ports of error components that are used to trigger
reset transitions.
@type: string
"""

DEFAULT_PACKAGE_NAME = "__default__"
"""The name of the default SLIM package.
@type: string
"""
