


// ----- Message Types ----------------------------------------------------------------------------

mtype = { update, invalidate, trymultiway, activate, deactivate, reset };



// ----- MACROs for ensuring atomicity of transitions ---------------------------------------------

bool gflagGoNextTransition = true;  // initially true for Environment proctype

#define nT(guard) d_step { \
	guard && !gflagGoNextTransition; \
	gflagGoNextTransition = true }
#define fT d_step { \
	gflagGoNextTransition = false }



// ----- Counter Variable for Multiway Event Communication ----------------------------------------

byte multiwayCounter;   // initially 0 (SPIN default), byte suffices for at most 255 components



// ----- DEFINE integer type & constants for enum data types --------------------------------------

#define enumIntType byte



// ----- DEFINE integer type & constants for nominal modes ----------------------------------------

#define modeIntType byte
#define modeID_m 1



// ----- DEFINE placeholders with atomic propositions ---------------------------------------------
// Note: Defined names for enums/modes cannot be used in properties as CPP does not replace text in later #defines.







// ================================================================================================






// ----- Global Variable Declarations for storing Components' Data --------------------------------

// *** root : __default__::Main.Impl ***
chan chan_root = [0] of { mtype, int };
modeIntType nominalMode_root;
int odpStore_root_odp; int odpNew_root_odp;
int dscStore_root_dscU;
int dscStore_root_dscB; int dscNew_root_dscB;






// ================================================================================================






// ----- Environment ------------------------------------------------------------------------------

active proctype Environment()
{
	// *** Activate & Update root component ***
	chan_root!activate, 0;
	chan_root?activate, _;

	// For tests: manipulate root's input data port variables here

	chan_root!update, 0;
	chan_root?update, _;
	odpStore_root_odp = odpNew_root_odp; 

	fT;  // Allow components to take transitions (initially nT reserved for Environment)



	// *** React to Messages from root component ***
	do

	// Invalidate: react with update
	:: chan_root?invalidate, _ ->
		chan_root!update, 0;
		chan_root?update, _;
		odpStore_root_odp = odpNew_root_odp; 
		fT;  // Environment responsible for fT after invalidate-update-cycle!

	// Trymultiway: respond 0 partners
	:: chan_root?trymultiway, _ ->
		chan_root!trymultiway, 0;

	od;
}






// ================================================================================================






// ----- root : __default__::Main.Impl --------------------------------------------------


active proctype process1_root()
{



	// *** Wait for first activation ***
FirstActivationOrReactivationWithoutModeHistory:
	chan_root?activate, _;
	d_step {

		// Initialize Own Data Ports
		odpNew_root_odp = 1; odpStore_root_odp = 1;

		// Initialize Data Subcomponents (only those active in start mode)
		dscStore_root_dscU = 2;
		dscStore_root_dscB = 3;

		// Initialize Mode Info
		nominalMode_root = modeID_m;
	}

	// Activate Non-Data Subcomponents (only those active in start mode)

	// Activation completed
	chan_root!activate, 0;
	goto mode_m;



	// *** Run as automaton & React to messages ***


	// --- Mode: m ---
mode_m:
	do

	// Proactive Transition (w/o event): m -[when EmptyExpression then (odp) := (10); (dscU) := ((odp) + (dscB)); (dscB) := (30); ]-> m
	:: nT(true) -> d_step {
		odpNew_root_odp = 10;
		dscStore_root_dscU = (odpStore_root_odp) + (dscStore_root_dscB);
		dscStore_root_dscB = 30;
		nominalMode_root = modeID_m; }
		chan_root!invalidate, 0;  // propagate changes; fT by root!
		goto mode_m;

	// Proactive Transition (w/o event): m -[when EmptyExpression then (odp) := (11); (dscB) := (31); (dscU) := ((odp) + (dscB)); ]-> m
	:: nT(true) -> d_step {
		odpNew_root_odp = 11;
		dscNew_root_dscB = 31;
		dscStore_root_dscU = (odpStore_root_odp) + (dscStore_root_dscB);
		dscStore_root_dscB = dscNew_root_dscB; dscNew_root_dscB = 0;  // Copy & reset buffer
		nominalMode_root = modeID_m; }
		chan_root!invalidate, 0;  // propagate changes; fT by root!
		goto mode_m;

	// Incomming Message: update (from parent)
	:: chan_root?update, _ ->

		// No active subcomponents to update

		// Update own out data ports

		// Send update acknowledge to parent
		chan_root!update, 0;

	// Incomming Message: invalidate (from subcomponents only)

	// Incomming Message: deactivate (from parent) - keep mode history
	:: chan_root?deactivate, _ ->
		// deactivate active subcomponents
		chan_root!deactivate, 0;
		// reactivate
		chan_root?activate, _;
		chan_root!activate, 0;

	// Incomming Message: reset (from parent)
	:: chan_root?reset, _ ->
		// reset active subcomponents
		// activate already deactivated subcomponents to reset them (only those with mode history)
		// confirm reset
		chan_root!reset, 0;
		goto FirstActivationOrReactivationWithoutModeHistory;

	od;
	// --- /Mode: m ---

}
