#!/usr/bin/python 

# =============================================================================
#  This file is part of the ``HASDEL'' Toolset.
#  Copyright (C) 2014 by Fondazione Bruno Kessler
#  Copyright (C) 2014 by RWTH Aachen University
#
#
#  The HASDEL Toolset can be copied, modified and/or redistributed
#  under the terms of the HASDEL General Public License, either
#  version 1 of the License, or (at your option) any later version.
#
#  You should have received a copy of the HASDEL General Public
#  License along with this software. If not, please contact the
#  copyright owners.
#
#  The HASDEL Toolset is distributed in the hope that it will be
#  useful, but WITHOUT ANY WARRANTY; without even the implied warranty
#  of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  HASDEL General Public License for more details.
#
#  For more information on the HASDEL Project see the Project Web
#  site at <http://compassinternal.informatik.rwth-aachen.de/hasdel-site/> 
#  or email to <hasdel-support@lists.rwth-aachen.de>.  
#
#  Please report bugs to <hasdel-support@lists.rwth-aachen.de>.
# =============================================================================



# This is a temporary file intended to be used to test slim2smv
# converter

import sys
import os
import os.path

sys.path.append('../../../../resources/antlr/antlr-python-runtime-3.1.1/')
pathname = os.path.abspath(os.path.join(os.path.dirname(sys.argv[0]), ".."))
print pathname
for ____d in ('dataStructure', 'errorManagement', 'generated'):
    sys.path.append(os.path.join(pathname, ____d))

import antlr3
import antlr3.tree
from slim_compiler.src.generated.SlimLexer import SlimLexer
from slim_compiler.src.generated.SlimParser import SlimParser
from slim_compiler.src.generated.SlimWalkerIdentCheck import SlimWalkerIdentCheck
from slim_compiler.src.errorManagement.errors import *
from slim_compiler.src.dataStructure.compile import CompileInformation


def parseFile(cinfo):
    char_stream = antlr3.ANTLRFileStream(cinfo.current_file)
    
    lexer = SlimLexer(char_stream)
    tokens = antlr3.CommonTokenStream(lexer)
    parser = SlimParser(tokens)
    r = parser.specification()
        
    # this is the root of the AST
    root = r.tree
    nodes = antlr3.tree.CommonTreeNodeStream(root)
    nodes.setTokenStream(tokens)
    slimWalkerIdentCheck = SlimWalkerIdentCheck(nodes)
    result = slimWalkerIdentCheck.specification(cinfo)    
    return


# ----------------------------------------------------------------------
import support
from slim_compiler.src.dataStructure import nominalbehaviour, expressions
import unittest

global_comp_info = None
class TestCompileInfo(unittest.TestCase):

    def setUp(self):
        self.cinfo = global_comp_info
        return

    def testget_root_components(self):
        roots = support.get_root_components(self.cinfo)
        self.assertEqual(len(roots), 1)
        root = roots.pop()
        self.assertEqual(root.name, "Main.Impl")
        return 

    def testget_component_types_with_impl(self):
        comps = support.get_component_types_with_impl(self.cinfo)
        
        keys = "Main Adder Random Bus Bit".split()
        self.assertEqual(len(comps), len(keys))
        for k in comps: self.assert_(k.name in keys)
        
        for k in comps: self.assertEqual(len(comps[k]), 1)         
        return
    
    pass # end of test for compile info


# ----------------------------------------------------------------------
class TestAdder(unittest.TestCase):

    def setUp(self):
        self.cinfo = global_comp_info
        comps = support.get_component_types_with_impl(self.cinfo)
        self.comp = None
        for k in comps: 
            if k.name == "Adder": self.comp = comps[k][0]
            pass
        
        assert(self.comp)
        return

    def testis_input_event(self):
        self.assert_(not support.is_input_event(self.comp, "input1"))
        self.assert_(not support.is_input_event(self.comp, "input2"))
        self.assert_(not support.is_input_event(self.comp, "output"))
        return

    def testare_modes_complete(self):
        self.assert_(support.are_modes_complete(self.comp, set(["init"])))
        self.assert_(support.are_modes_complete(self.comp, set(["init", "dummy"])))
        self.assert_(not support.are_modes_complete(self.comp, set(["dummy"])))
        self.assert_(not support.are_modes_complete(self.comp, set()))
        return

    def testget_subcomponent_impl_from_name(self):
        self.assertRaises(KeyError, support.get_subcomponent_impl_from_name, 
                          self.comp, "dummy")
        return

    def testget_subcomponent_(self):
        checks = (support.get_subcomponent_from_port, 
                  support.get_subcomponent_impl_from_port, 
                  support.get_non_data_subcomponent_from_name, 
                  support.get_subcomponent_impl_from_name,)

        # exception is raised if port or subcompoennt is not found
        for check in checks: self.assertRaises(KeyError, check, self.comp, "dummy.dummy")

        # None is returned for port when no sc is specified
        self.assertEqual(support.get_subcomponent_from_port(self.comp, "dummyport"), None)
        self.assertEqual(support.get_subcomponent_impl_from_port(self.comp, "dummyport"), None)
        return

    def testget_subcomponent_events_with_event(self):
        events = support.get_subcomponent_events_with_event(self.comp, "dummy")
        self.assertEqual(len(events), 0)
        return

    def testget_component_impl_modes(self):
        modes = support.get_component_impl_modes(self.comp)
        self.assertEqual(len(modes), 1)
        self.assert_("init" in modes)
        self.assert_(support.get_default_mode_name() not in modes)
        return

    def testget_trans_with_event(self):
        t = support.get_trans_with_event(self.comp, "dummy")
        self.assert_(len(t) == 0)

        t = support.get_trans_with_tau(self.comp)
        self.assert_(len(t) == 1)
        self.assert_(isinstance(t.pop(), nominalbehaviour.ModeTransition))
        return

    def testget_trans_id(self):
        trans = self.comp.normal_mode_transitions
        self.assertEqual(len(trans), 1)
        trans_id = support.get_trans_id(self.comp, trans[0])
        self.assertEqual(trans_id, "init_init_tau_0")
        return

    def testget_trans_assigns(self):
        trans = self.comp.normal_mode_transitions
        self.assertEqual(len(trans), 1)
        ass = support.get_trans_assigns(self.comp, trans[0], complete=True)
        self.assertEqual(len(ass), 1)        
        return

    def testget_data_subcomponents_in_expr(self):
        trans = self.comp.normal_mode_transitions
        ass = support.get_trans_assigns(self.comp, trans[0], complete=True)
        self.assertEqual(len(ass), 1)
        d,expr = ass.popitem()
        dscs = support.get_data_subcomponents_in_expr(self.comp, expr)
        self.assertEqual(len(dscs), 0)

        # trans is expected to be "input1 != input2"
        dsop = support.get_data_ports_in_expr(self.comp, expr)
        self.assertEqual(len(dsop), 2)        
        self.assertEqual(set(("input1", "input2")), set([d.name for d in dsop]))
        return
        
    def testsubstitute_ids_in_expr(self):
        trans = self.comp.normal_mode_transitions
        ass = support.get_trans_assigns(self.comp, trans[0], complete=True)
        self.assertEqual(len(ass), 1)
        d,expr = ass.popitem()
        
        dsop = support.get_data_ports_in_expr(self.comp, expr)

        # expr is "input1 != input2"
        # substitutes input2 with output to obtain "input1 != output"
        ctype = self.comp.get_referenced_component_type()
        mapping = { ctype.in_data_ports["input2"] : expressions.IdentifierTreeNode(d.name, -1, -1) }

        expr2 = support.substitute_ids_in_expr(expr, mapping)
        dsop2 = support.get_data_ports_in_expr(self.comp, expr2)
        self.assertEqual(len(dsop2), 2)        
        self.assertEqual(set(("output", "input1")), set([d.name for d in dsop2]))        
        return
        

    pass # end of class


# ----------------------------------------------------------------------
class TestMain(unittest.TestCase):

    def setUp(self):
        self.cinfo = global_comp_info
        comps = support.get_component_types_with_impl(self.cinfo)
        self.comp = None
        for k in comps: 
            if k.name == "Main": self.comp= comps[k][0]
            pass
        
        assert(self.comp)
        return

    def testget_trans_with_event(self):
        t = support.get_trans_with_event(self.comp, "dummy")
        self.assert_(len(t) == 0)

        t = support.get_trans_with_tau(self.comp)
        self.assert_(len(t) == 0)
        return


    def testget_connections_(self):
        self.assert_(not support.get_connections_with_source_event(self.comp, "dummy"))
        self.assert_(not support.get_connections_with_target_event(self.comp, "dummy"))
        self.assert_(not support.get_connections_with_events(self.comp, "dummy1", "dummy2"))
        
        self.assert_(not support.get_connections_with_source_data(self.comp, "output"))
        c1 = support.get_connections_with_target_data(self.comp, "output")
        self.assert_(len(c1) == 1)

        c2 = support.get_connections_with_source_data(self.comp, "adder.output")
        self.assert_(not support.get_connections_with_target_data(self.comp, "adder.output"))
        self.assertEqual(c1, c2)

        c3 = support.get_connections_with_source_data(self.comp, "bit2.output")
        self.assert_(len(c3) == 1)

        m = support.get_connections_modes(self.comp, c2 | c3)
        self.assert_(len(m) == 1)
        self.assertEqual(m.pop(), support.get_default_mode_name())
        return
        
    def testget_subcomponent_modes(self):
        m = support.get_subcomponent_modes(self.comp, 
                      support.get_non_data_subcomponent_from_name(self.comp, "adder"))
        self.assertEqual(len(m), 1)
        self.assertEqual(m.pop(), support.get_default_mode_name())
        return        

    def testis_output_event(self):
        self.assert_(not support.is_output_event(self.comp, "output1"))
        self.assert_(not support.is_output_event(self.comp, "output2"))
        self.assert_(not support.is_output_event(self.comp, "output"))
        return

    def testis_input_event(self):
        self.assert_(not support.is_input_event(self.comp, "input1"))
        self.assert_(not support.is_input_event(self.comp, "input2"))
        self.assert_(not support.is_input_event(self.comp, "output"))
        return

    def testare_modes_complete(self):
        self.assert_(support.are_modes_complete(self.comp, 
                           set([support.get_default_mode_name()])))
        self.assert_(not support.are_modes_complete(self.comp, set()))
        return

    def testget_subcomponent_impl_from_name(self):
        self.assertRaises(KeyError, support.get_subcomponent_impl_from_name, 
                          self.comp, "dummy")
        return

    def testget_subcomponent_(self):
        adder = support.get_non_data_subcomponent_from_name(self.comp, "adder")
        self.assertEqual(adder.name, "adder")
        self.assert_(isinstance(adder, nominalbehaviour.NonDataSubcomponent))

        adderi = support.get_subcomponent_impl_from_name(self.comp, "adder")        
        self.assert_(adderi)
        self.assertEqual(adderi.name, "Adder.Impl")
        self.assert_(isinstance(adderi, nominalbehaviour.ComponentImplementation))
        
        adder2 = support.get_subcomponent_from_port(self.comp, "adder.output")
        self.assertEqual(adder, adder2)
        
        adderi2 = support.get_subcomponent_impl_from_port(self.comp, "adder.output")
        self.assertEqual(adderi, adderi2)
        return

    def testget_subcomponent_events_with_event(self):
        events = support.get_subcomponent_events_with_event(self.comp, "dummy")
        self.assertEqual(len(events), 0)
        return

    def testget_component_impl_modes(self):
        modes = support.get_component_impl_modes(self.comp)
        self.assertEqual(len(modes), 1)
        self.assert_(support.get_default_mode_name() in modes)
        return


# ----------------------------------------------------------------------

    
if __name__ == "__main__":
    cinfo = CompileInformation()
    
    for afile in ("adder.slim",):
        cinfo.current_file = os.path.abspath(afile)
        if not os.path.exists(cinfo.current_file):
            print "Fatal Error: File %s not found" % cinfo.current_file
            quit()
        cinfo.files.append(cinfo.current_file)
        parseFile(cinfo)
        
    cinfo.check()
    if len(cinfo.error_collector.errors) != 0:
        print "Errors: --------------------------------------------------------"
        cinfo.error_collector.output()    
        print "Files processed: -----------------------------------------------"
        cinfo.print_files()
    else:
        print "No errors found. -----------------------------------------------"
        print "Files processed: -----------------------------------------------"
        cinfo.print_files()

        global_comp_info = cinfo
        unittest.main()
