#!/usr/bin/python
# =============================================================================
#  This file is part of the ``HASDEL'' Toolset.
#  Copyright (C) 2014 by Fondazione Bruno Kessler
#  Copyright (C) 2014 by RWTH Aachen University
#
#
#  The HASDEL Toolset can be copied, modified and/or redistributed
#  under the terms of the HASDEL General Public License, either
#  version 1 of the License, or (at your option) any later version.
#
#  You should have received a copy of the HASDEL General Public
#  License along with this software. If not, please contact the
#  copyright owners.
#
#  The HASDEL Toolset is distributed in the hope that it will be
#  useful, but WITHOUT ANY WARRANTY; without even the implied warranty
#  of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  HASDEL General Public License for more details.
#
#  For more information on the HASDEL Project see the Project Web
#  site at <http://compassinternal.informatik.rwth-aachen.de/hasdel-site/> 
#  or email to <hasdel-support@lists.rwth-aachen.de>.  
#
#  Please report bugs to <hasdel-support@lists.rwth-aachen.de>.
# =============================================================================


""" File to test parsing of expressions only 

    NEEDS TO BE DELETED SOON WHEN IT HAS BECOME OBSOLETE
"""


import sys
import os
import os.path

sys.path.append('../../../resources/antlr/antlr-python-runtime-3.1.1/')

import antlr3
import antlr3.tree
import logging

from generated.SlimWalkerIdentCheck import SlimWalkerIdentCheck
from errorManagement.errors import *
from errorManagement.exceptions import *
from dataStructure.compile import CompileInformation
from dataStructure.derivedlexer import DerivedLexer
from dataStructure.derivedparser import DerivedParser
import dataStructure.expressions
from dataStructure.visitors.printVisitor import toSLIMVisitor
from slim_compiler.src import globals
import options
#import smv.slim2smv

from parse_property import *

from optparse import OptionParser


def parseExpression(exprString, extended=False):
    """ Parses a string as an expression and returns the expression tree
    associated with it. If it is an invalid expression, then an 
    InvalidExpressionException is thrown.
    
    If the parameter extended is True, the parser will accept identifiers
    from extended models (e.g. with leading underscores in identifiers)
    """
    globals.PARSER_EXPRESSIONS_ONLY = True
    ext = globals.PARSER_ACCEPT_EXTENDED_MODEL
    if extended:
        globals.PARSER_ACCEPT_EXTENDED_MODEL = True
        
    try:
        cinfo = CompileInformation()
        char_stream = antlr3.ANTLRStringStream(exprString)
        lexer = DerivedLexer(cinfo, char_stream)
        tokens = antlr3.CommonTokenStream(lexer)
        parser = DerivedParser(cinfo, tokens)
        r = parser.expressionsOnly()
    finally:
        if extended:
            globals.PARSER_ACCEPT_EXTENDED_MODEL = ext
    
    if parser.has_parsing_errors() or lexer.has_lexing_errors():
        raise InvalidExpressionException(cinfo.error_collector)
    else:    
        nodes = antlr3.tree.CommonTreeNodeStream(r.tree)
        nodes.setTokenStream(tokens)
        expressionTree = SlimWalkerIdentCheck(nodes).expr(True)
    
    return expressionTree.value

def parseAssignment(assignment_string):
    """ Parses a string as an assignment and returns the expression tree
    associated with it. If it is an invalid assignment, then an 
    InvalidExpressionException is thrown.
    Note: Both sides of the assignment may contain not only simple identifiers
    but also names of the form "subcomp.port_name".
    """
    globals.PARSER_ASSIGNMENTS_ONLY = True
    cinfo = CompileInformation()
    char_stream = antlr3.ANTLRStringStream(assignment_string)
    lexer = DerivedLexer(cinfo, char_stream)
    tokens = antlr3.CommonTokenStream(lexer)
    parser = DerivedParser(cinfo, tokens)
    parser.allow_names = True  # Allow names of the form "subcomp.port_name".
    r = parser.assignmentsOnly()
    
    if parser.has_parsing_errors() or lexer.has_lexing_errors():
        raise InvalidExpressionException(cinfo.error_collector)
    else:    
        nodes = antlr3.tree.CommonTreeNodeStream(r.tree)
        nodes.setTokenStream(tokens)
        effect_tree = SlimWalkerIdentCheck(nodes).modeEffect()
    
    return effect_tree

def parse(filename, cinfo):
    # sanity checks 
    assert cinfo != None
    assert os.path.exists(filename), "File %s not found" % filename
                     
    # let the compile_info structure know we are going to process a new file
    cinfo.current_file = os.path.abspath(filename)
    cinfo.files.append(filename)
    
    char_stream = antlr3.ANTLRFileStream(filename)
    
    lexer = SlimLexer(char_stream)
    tokens = antlr3.CommonTokenStream(lexer)
    parser = DerivedParser(cinfo, tokens)

    r = parser.specification()
    
    # start decorating
    if not parser.has_parsing_errors():
        nodes = antlr3.tree.CommonTreeNodeStream(r.tree)
        nodes.setTokenStream(tokens)
        SlimWalkerIdentCheck(nodes).specification(cinfo)
        
    return
    
# main
if __name__ == "__main__":   
    oparser = OptionParser('''Usage: main.py [OPTION] [DIRS]''')
    (_options, args) = oparser.parse_args()

    print _options
    print args
    
    # set up a new data structure    
    cinfo = CompileInformation()
       
    # parse files and add all information to the data structures   
    if args[0]:
        parse(args[0], cinfo)
    
        # run the checker
        from invokers import slim
        slim.check(cinfo)

    # check errors 
    if len(cinfo.error_collector.errors) != 0:
        logging.debug("Errors: --------------------------------------------------------")
        #cinfo.error_collector.output()    
        logging.info(cinfo.error_collector)
        logging.info(str(len(cinfo.error_collector.errors)) + " error(s)")
        logging.debug("Files processed: -----------------------------------------------")
        map(logging.debug, cinfo.files)
    else: 
        logging.debug(toSLIMVisitor().visit(cinfo))
        
    #root_impl = smv.slim2smv.get_root_impl(cinfo, options.ROOT_NAME)

    logging.basicConfig(level=logging.DEBUG)
    #print root_impl
    
    expression = ""
    for i in range(1, len(args)):
        expression += args[i]
        if i < len(args) - 1:
            expression += " "
            
    a = parseExpression(expression)
    print toSLIMVisitor().visit(a)
    
    print expressions.get_AST(a)
        
    #check_property_tree(cinfo, a, root_impl)
    
    # check validity
    # check(root_impl, cinfo, a)
    
    
