# =============================================================================
#  This file is part of the ``HASDEL'' Toolset.
#  Copyright (C) 2014 by Fondazione Bruno Kessler
#  Copyright (C) 2014 by RWTH Aachen University
#
#
#  The HASDEL Toolset can be copied, modified and/or redistributed
#  under the terms of the HASDEL General Public License, either
#  version 1 of the License, or (at your option) any later version.
#
#  You should have received a copy of the HASDEL General Public
#  License along with this software. If not, please contact the
#  copyright owners.
#
#  The HASDEL Toolset is distributed in the hope that it will be
#  useful, but WITHOUT ANY WARRANTY; without even the implied warranty
#  of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  HASDEL General Public License for more details.
#
#  For more information on the HASDEL Project see the Project Web
#  site at <http://compassinternal.informatik.rwth-aachen.de/hasdel-site/> 
#  or email to <hasdel-support@lists.rwth-aachen.de>.  
#
#  Please report bugs to <hasdel-support@lists.rwth-aachen.de>.
# =============================================================================
import sys
sys.path.append('../../../resources/antlr/antlr-python-runtime-3.1.1/')
sys.path.append('../../')

import unittest

from slim_compiler.src.dataStructure.visitors.expressionVisitor import InvariantToNNFVisitor,\
    SimplifyCaseVisitor, SimplifyExprVisitor, SimplifyLinEq,\
    InvariantIsConvexVisitor
from expression_tests import parse_expression, annotate_expression, check_expression, \
    ExpressionBaseTest

class ExpressionTests(ExpressionBaseTest):
    def setUp(self):
        super(ExpressionTests, self).setUp()
        self.cinfo.uses_time_scaling = False

    def _simplify_case(self, expr_str):
        tree = parse_expression(self.cinfo, expr_str, True)
        if tree == None:
            return False
        if not annotate_expression(self.cinfo, self.scope, tree):
            return False
        if not check_expression(self.cinfo, tree):
            return False

        simplify = SimplifyCaseVisitor()
        tree = simplify.replace_case(tree)
        print tree

        return tree

    def _to_cnf(self, expr_str):
        tree = parse_expression(self.cinfo, expr_str, True)
        if tree == None:
            return False
        if not annotate_expression(self.cinfo, self.scope, tree):
            return False
        if not check_expression(self.cinfo, tree):
            return False

        bool_tree = SimplifyCaseVisitor().replace_case(tree.clone())
        cnf_tree = InvariantToNNFVisitor(False).visit(bool_tree)
        simple_tree = SimplifyExprVisitor().visit(cnf_tree)
        print simple_tree

        return simple_tree

    def _to_lineq(self, expr_str):
        tree = parse_expression(self.cinfo, expr_str, True)
        if tree == None:
            return False
        if not annotate_expression(self.cinfo, self.scope, tree):
            return False
        if not check_expression(self.cinfo, tree):
            return False

        print tree
        bool_tree = SimplifyCaseVisitor().replace_case(tree.clone())
        cnf_tree = InvariantToNNFVisitor().visit(bool_tree)
        simple_tree = SimplifyExprVisitor().visit(cnf_tree)
        lin_tree = SimplifyLinEq().build(simple_tree)
        print lin_tree

        InvariantIsConvexVisitor().visit(lin_tree)

        return simple_tree

    def _test_cnf(self):
        self._to_cnf("bvar and (bvar or bvar)")

        self._to_cnf("bvar or false and true and clkvar >= 12.0")
        self._to_cnf("(bvar or false) and (true and clkvar >= 12.0)")
        self._to_cnf("not (clkvar >= 12.0)")
        self._to_cnf("not (true imp false)")
        self._to_cnf("not (false imp true)")

        self._to_cnf("false xor true")
        self._to_cnf("not bvar iff bvar")

        self._to_cnf("case false: bvar or not bvar; true: clkvar >= 12.0 otherwise clkvar <= 4.0 end")

        self._to_cnf("(bvar and bvar and bvar) or (bvar and bvar and bvar)")

        self._to_cnf("case false: clkvar >= 4; true: clkvar >= 12.0 otherwise clkvar <= 3.0 end")

        self._to_cnf("not(bvar imp not bvar)")

        self._to_cnf("not(bvar = bvar)")

    def _test_case_replace(self):
        self._simplify_case("case false: 1 otherwise 3 end")
        self._simplify_case("ivar < case false: 1 otherwise 3 end")

        self._simplify_case("true and ivar < case false: 1 otherwise 3 end")

        self._simplify_case("not bvar = bvar and ((bvar xor bvar) or (ivar < case false: 1 otherwise 3 end))")

        self._simplify_case("not (bvar = bvar and ((bvar xor bvar) or (ivar < (4 + 3*case false: 1 otherwise 3 end))))")

        self._simplify_case("not (ivar < (4 + 3*case false: 1 otherwise 3 end))")

        self._simplify_case("(ivar < (4 + 3*case false: 1 otherwise 3 end)) and true")

    def _test_case_replace2(self):
        self._simplify_case("2 = case false: 1 otherwise 3 end + case false: 4 otherwise 5 end")

        self._simplify_case("2 = case false: 1 otherwise case true: 8 otherwise 6 end end")

        self._simplify_case("2 = case false: 1 otherwise case true: 8 otherwise 6 end end + case false: 4 otherwise 5 end")

        self._simplify_case("2 = case false: 1 otherwise case true: 8 otherwise 6 end end + case false: 4 otherwise 5 end")

    def test_lineq(self):
        self._to_lineq("clkvar < 6 or clkvar > 6")

        self._to_lineq("clkvar*1 >= 6 or clkvar*1 <= 4")

        self._to_lineq("clkvar*1 >= 6 and clkvar*1 <= 4")

        self._to_lineq("clkvar * 2 + 5 >= 6 or clkvar * -6 <= (500)")

        self._to_lineq("clkvar * 2 + 5 >= clkvar / 2 - 8 + 3 mod 6")

        self._to_lineq("clkvar * 2 + clkvar2 / (8 mod 5) >= clkvar2 / 2 - 8 + clkvar * (3 mod 6)")

if __name__ == "__main__":
    unittest.main()
