# =============================================================================
#  This file is part of the ``HASDEL'' Toolset.
#  Copyright (C) 2014 by Fondazione Bruno Kessler
#  Copyright (C) 2014 by RWTH Aachen University
#
#
#  The HASDEL Toolset can be copied, modified and/or redistributed
#  under the terms of the HASDEL General Public License, either
#  version 1 of the License, or (at your option) any later version.
#
#  You should have received a copy of the HASDEL General Public
#  License along with this software. If not, please contact the
#  copyright owners.
#
#  The HASDEL Toolset is distributed in the hope that it will be
#  useful, but WITHOUT ANY WARRANTY; without even the implied warranty
#  of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  HASDEL General Public License for more details.
#
#  For more information on the HASDEL Project see the Project Web
#  site at <http://compassinternal.informatik.rwth-aachen.de/hasdel-site/> 
#  or email to <hasdel-support@lists.rwth-aachen.de>.  
#
#  Please report bugs to <hasdel-support@lists.rwth-aachen.de>.
# =============================================================================
import sys
sys.path.append('../../../resources/antlr/antlr-python-runtime-3.1.1/')
sys.path.append('../../')

import unittest
import os
import tempfile
from slim_compiler.src import globals
from invokers import slim
from slim_compiler.src.dataStructure.compile import CompileInformation
from slim_compiler.src.dataStructure.visitors.printVisitor import toSLIMVisitor
from slim_compiler.src.dataStructure.visitors.compileVisitor import compareVisitor

def compare(self, slimFile):
    self.assertTrue(os.path.exists(slimFile),"\"%s\" does not exist"%slimFile)
    
    # Parse slim model  
    globals.PARSER_ACCEPT_EXTENDED_MODEL = False
    origCinfo = CompileInformation()
    slim.parse(origCinfo, [slimFile])
    slim.check(origCinfo)
    
    visitorSLIM = toSLIMVisitor().visit(origCinfo)
    
    # Re-parse  model and compare it to the previous one
    f = tempfile.NamedTemporaryFile()
    f.write(visitorSLIM)
    f.flush()
    
    cinfoFromVisitor = CompileInformation()
    slim.parse(cinfoFromVisitor, [f.name])
    slim.check(cinfoFromVisitor)
    
    self.assertTrue(compareVisitor().visit(origCinfo, cinfoFromVisitor))
    f.close()

class Test(unittest.TestCase):
    def test_1(self):
        compare(self, '../../examples/Adder.slim')
        
    def test_2(self):
        compare(self, '../../examples/Adder_err.slim')
        
    def test_3(self):
        compare(self, '../../examples/Adder-2.slim')
        
    def test_4(self):
        compare(self, '../../examples/Adder_err-2.slim')
        
    def test_5(self):
        compare(self, '../../examples/sensorfilter.slim')
        
    def test_6(self):
        compare(self, '../../examples/sensorfilter-deadlockfree.slim')
        
    def test_7(self):
        compare(self, '../../examples/sensorfilterErr.slim')
        
    def test_8(self):
        compare(self, '../../examples/sensorfilterErr-hybrid.slim')
        
    def test_9(self):
        compare(self, '../../examples/sensorfilter-hybrid.slim')
    
if __name__ == "__main__":
    unittest.main()
