#!/bin/env python

# =============================================================================
#  This file is part of the ``HASDEL'' Toolset.
#  Copyright (C) 2014 by Fondazione Bruno Kessler
#
#
#  The HASDEL Toolset can be copied, modified and/or redistributed
#  under the terms of the HASDEL General Public License, either
#  version 1 of the License, or (at your option) any later version.
#
#  You should have received a copy of the HASDEL General Public
#  License along with this software. If not, please contact the
#  copyright owners.
#
#  The HASDEL Toolset is distributed in the hope that it will be
#  useful, but WITHOUT ANY WARRANTY; without even the implied warranty
#  of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  HASDEL General Public License for more details.
#
#  For more information on the HASDEL Project see the Project Web
#  site at <http://compassinternal.informatik.rwth-aachen.de/hasdel-site/> 
#  or email to <hasdel-support@lists.rwth-aachen.de>.  
#
#  Please report bugs to <hasdel-support@lists.rwth-aachen.de>.
# =============================================================================


def process_options():
    """Checks whether the executable has been called properly"""

    global structural_file
    structural_file = ""
    global xml_trace_file
    xml_trace_file = ""
    global dotty_viewer
    dotty_viewer = False
    global output_file
    output_file = ""

    from optparse import OptionParser, OptionValueError
    import src.utils.globals
    
    usage = "usage: %prog [-s <structural_file>] [-t <xml_trace>]"
    parser = OptionParser(usage)
    parser.add_option("-s", dest="structural_file",
                      help="the file representing structural tfpg")
    parser.add_option("-t", dest="xml_trace",
                      help="the XML representing a trace")

    (opts, args) = parser.parse_args()
    
    if opts.structural_file:
        structural_file = opts.structural_file
        src.utils.log.system.debug("-s option specified with file '%s'" % structural_file) 

    if opts.xml_trace:
        xml_trace_file = opts.xml_trace
        src.utils.log.system.debug("-t option specified with file '%s'" % xml_trace_file)
        if not(opts.structural_file):
            print parser.error("You must specify also a structural file!")
            src.utils.log.system.error("Options parsing error")

    return



def setup_path():
    """Sets up the python include paths to include required directories"""

    import os.path;
    import sys

    from src.utils.globals import TOPDIR, RESOURCES_DIR

    sys.path.insert(0, reduce(os.path.join, (TOPDIR, "resources", "external")))
    sys.path.insert(0, os.path.join(TOPDIR, "src"))
    sys.path.insert(0, os.path.join(TOPDIR, "../"))
    sys.path.insert(0, os.path.join(RESOURCES_DIR, "external/gtkmvc.zip"))
    sys.path.insert(0, os.path.join(TOPDIR, "../../resources/antlr/antlr-python-runtime-3.1.1/"))

    return


def create_logging_structure():
    """Creates directory and files for the logging"""

    import src.utils.log
    from src.utils.misc import create_out_dir
    from src.utils import options

    options.OUT_DIR_NAME = "log"
    options.LOGGING_FNAME = "%s/tfpg_log.log" % options.OUT_DIR_NAME
    options.LOGGING_SYS_LEVEL = "debug"

    log_dir = create_out_dir()
    src.utils.log.init()

    return


def main(*args, **kargs):
    """Main module"""

    from src.models.tfpg_model import TFPGModel
    from src.views.tfpg_view import TFPGView
    from src.ctrls.tfpg_ctrl import TfpgCtrl
    import gtk

    model = TFPGModel()
    view = TFPGView(standalone=True)
    controller = TfpgCtrl(model, view, structural_file, xml_trace_file)

    gtk.main()
    return


if __name__ == "__main__":
    setup_path()
    create_logging_structure()
    process_options()
    main()
