
grammar Tfpg;

options
{
    output=AST;
    language=Python;
    backtrack=true;
}

tokens
{
    NAME = 'NAME';
    FM = 'FM';
    AND = 'AND';
    OR = 'OR';
    EDGE = 'EDGE';
    MONITORED = 'MONITORED';
    INFINITY = 'INFINITY';
    ALL = 'ALL';
    MODES = 'MODES';
    COMMA = ',';
    LP = '(';
    RP = ')';
    NODE = 'NODE';
    NODES = 'NODES';
    EDGES = 'EDGES';
    EDGE_MODES = 'EDGE_MODES';
    TFPG = 'TFPG';
}


@members{
    def emitErrorMessage(self, m):
      raise Exception(m)
}


/* ----------------------------------------------------------------------------------------
 * PARSER RULES
 * ---------------------------------------------------------------------------------------- */

tfpg
    : tfpg_name (node)* (modes_list)* (edge)* EOF  -> ^(TFPG tfpg_name (node)* (modes_list)* (edge)*)
    ;

tfpg_name
    : NAME IDENT  -> ^(NAME IDENT)
    ;

node
    : FM IDENT             -> ^(FM IDENT)
    | AND IDENT            -> ^(AND IDENT)
    | AND IDENT MONITORED  -> ^(AND IDENT MONITORED)
    | OR IDENT             -> ^(OR IDENT)
    | OR IDENT MONITORED   -> ^(OR IDENT MONITORED)
    ;

modes_definition
    : MODES modes_list  -> ^(MODES modes_list)
    ;

modes_list
    : MODES IDENT (COMMA IDENT)* -> ^(MODES IDENT (IDENT)*)
    ;

edge
    : EDGE edge_name edge_src edge_dst min_time max_time edge_modes_list
        -> ^(EDGE edge_name edge_src edge_dst min_time max_time edge_modes_list)
    ;

edge_name : IDENT -> IDENT;

edge_src : IDENT -> IDENT;

edge_dst : IDENT -> IDENT;

edge_modes_list 
    : LP IDENT (COMMA IDENT)* RP -> ^(EDGE_MODES IDENT (IDENT)*)
    | ALL                        -> ^(ALL)
    ;

min_time
    : FLOAT -> FLOAT
    ;

max_time
    : FLOAT    -> FLOAT
    | INFINITY -> INFINITY
    ;


/* ----------------------------------------------------------------------------------------
 * LEXER RULES
 * ---------------------------------------------------------------------------------------- */

IDENT
	: ('A'..'Z'|'a'..'z')('A'..'Z'|'a'..'z'|'0'..'9'|'_'|'-')*
	;

FLOAT
    : ('0'..'9')+ (('.') ('0'..'9')+)?
    ;

COMMENT
	: '--' ~('\r'|'\n')* ('\r'|'\n')?
	  { $channel=HIDDEN }
	;

WS	
	: (' '|'\r'|'\t'|'\u000C'|'u'|'\n')
	  { $channel=HIDDEN; }
	;
