# =============================================================================
#  This file is part of the ``HASDEL'' Toolset.
#  Copyright (C) 2014 by Fondazione Bruno Kessler
#
#
#  The HASDEL Toolset can be copied, modified and/or redistributed
#  under the terms of the HASDEL General Public License, either
#  version 1 of the License, or (at your option) any later version.
#
#  You should have received a copy of the HASDEL General Public
#  License along with this software. If not, please contact the
#  copyright owners.
#
#  The HASDEL Toolset is distributed in the hope that it will be
#  useful, but WITHOUT ANY WARRANTY; without even the implied warranty
#  of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  HASDEL General Public License for more details.
#
#  For more information on the HASDEL Project see the Project Web
#  site at <http://compassinternal.informatik.rwth-aachen.de/hasdel-site/> 
#  or email to <hasdel-support@lists.rwth-aachen.de>.  
#
#  Please report bugs to <hasdel-support@lists.rwth-aachen.de>.
# =============================================================================

# This module contains global options for slim2mrmc processes This
# needs to be restructured in a more global vision at the moment
# this module is a simple placeholder keeping all options in one
# place

import os.path

top_dir = os.path.join(os.path.dirname(__file__), "../..")


def svnver():
    """Finds svn revision if found, or returns latest available"""

    # default if things go wrong
    revsvn = int(filter(str.isdigit, "$Revision: 70 $"))

    # first checks if svn is available
    if os.path.isdir(os.path.join(top_dir, ".svn")):
        try:
            from subprocess import Popen, PIPE
            _p = Popen(["svn", "info", top_dir], stdout=PIPE)
            _out = _p.communicate()[0]
            _search = "Revision: "
            for line in _out.split("\n"):
                pos = line.find(_search)
                if pos != -1:
                    revsvn = int(line[pos + len(_search):])
                    break
                pass
            
        except: pass
        pass

    # if failed, use distribution file if available
    if 0 == revsvn:
        fname = os.path.join(top_dir, ".revsvn")
        if os.path.isfile(fname):
            with open(fname) as _file:
                try: revsvn = int(_file.read())
                except: pass
                pass
            pass
        pass
    
    return revsvn

# ----------------------------------------------------------------------
#  Application info
# ----------------------------------------------------------------------

PROG_NAME = "GNuSMV3"
PROG_CMDLINE = ""
PROG_VER = (0, 99, 1, svnver())


# the user home, where configuration is searched/saved
USER_CONFIG_DIR = os.path.join(os.path.expanduser("~"),
                               "." + PROG_NAME.lower())

# Names of input smv file
SMV_FILE_NAME = ""

# Basename of output files
BASE_FILE_NAME = ""

# Name of fault extension file
FAULT_EXTENSION_INFO_FILE_NAME = ""

# A flag to output a fault extended model. Used for debugging only.
OUTPUT_FAULT_EXTENDED = False

# Name of properties file
PROPERTIES_FILE_NAME = ""

# Name of the root implementation
ROOT_NAME = ""

# default logging level
LOGGING_SYS_LEVEL = ""
LOGGING_TEST_LEVEL = ""
LOGGING_USER_LEVEL = ""
LOGGING_FNAME = "log/log_file.log"

# To stop after parsing, checking, fault injection, smv dumping,
# NuSMV to sigref translation, sigref to mrmc translation, etc.
# This is used mainly for debugging
STOP_AFTER = ""

# To enable the output of subprocess calls
DISPLAY_CALL_OUTPUT = False

# ----------------------------------------------------------------------
# Directories
# ----------------------------------------------------------------------

# top level directory, the root for all sources
TOP_LEVEL_DIR = top_dir

# place for resources
RESOURCES_DIR = os.path.join(TOP_LEVEL_DIR, "./resources")

# place for binary external executable files
BIN_DIR = os.path.join(RESOURCES_DIR, "bin")

# Name of a directory for output and logging files. If the empty
# string is given, the basename of the slim file will be used.  If
# empty and if a directory with the basename already exists, a
# numeric suffix will be added.
OUT_DIR_NAME = "log"


# ----------------------------------------------------------------------
# External tools
# ----------------------------------------------------------------------
DEGRADED_NUSMV = "NuSMV"
NUSMV = "esmc"
FTV = "ftv"
FTD = "ftd"

MEMOUT = -1  # expressed in MBytes (-1 means no limit)
TIMEOUT = -1  # expressed in seconds (-1 means no limit)


# ----------------------------------------------------------------------
# Style specific
# ----------------------------------------------------------------------
VIEW_INFO_LABEL_BG = "#d4e0d0"

# ----------------------------------------------------------------------
# SMV Specific
# ----------------------------------------------------------------------

# Size of words generated when encoding slim integers:
SMV_WORD_SIZE = 32

# use signed or unsigned words
SMV_WORD_SIGNED = True

# Enable smv file to contain debugging specifications
SMV_ENABLE_DEBUG = False

# Enable smv->slim name conversions of 'internal' names
# Note: this is needed by transition-based guided simulation
SMV_ENABLE_INTERNAL_SLIM_NAMES = True

# If time-transitions are forced to occur in a fair loop
SMV_FAIR_DELTA = True

# ----------------------------------------------------------------------
# NuSMV Specific
# ----------------------------------------------------------------------
# By default, use SAT-based Bounded Model Checking for LTL formulae
NUSMV_USE_BMC = False
NUSMV_BMC_USE_SBMC = True
NUSMV_SBMC_IS_COMPLETE = False
NUSMV_BMC_LENGTH = 10  # length (k) of BMC
NUSMV_SIM_LENGTH = 10  # length of simulation

# ----------------------------------------------------------------------
# FSAP Specific
# ----------------------------------------------------------------------
FSAP_FAULT_CONFIGURATION_CARDINALITY = 1
FSAP_KEEP_SMV_IDS = False
FSAP_PROPERTY_NUMBER = 0 
FSAP_USE_DYNAMIC = False

# change to True to have compact FMEA tables when possible (i.e. no
# dynamic table is requested)
FSAP_COMPACT_FMEA_TABLE = False

# Only for fault tree generation runscript:
FSAP_OUTPUT_FAULT_TREE_XML_AND_RATES = False


# ----------------------------------------------------------------------
# FDIR Specific
# ----------------------------------------------------------------------
PFDIR_ERROR_STATE = None
PFDIR_OBSERVABLE = None
PFDIR_UPPER_TIME_BOUND = 0
