# =============================================================================
#  This file is part of the ``HASDEL'' Toolset.
#  Copyright (C) 2014 by Fondazione Bruno Kessler
#
#
#  The HASDEL Toolset can be copied, modified and/or redistributed
#  under the terms of the HASDEL General Public License, either
#  version 1 of the License, or (at your option) any later version.
#
#  You should have received a copy of the HASDEL General Public
#  License along with this software. If not, please contact the
#  copyright owners.
#
#  The HASDEL Toolset is distributed in the hope that it will be
#  useful, but WITHOUT ANY WARRANTY; without even the implied warranty
#  of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  HASDEL General Public License for more details.
#
#  For more information on the HASDEL Project see the Project Web
#  site at <http://compassinternal.informatik.rwth-aachen.de/hasdel-site/> 
#  or email to <hasdel-support@lists.rwth-aachen.de>.  
#
#  Please report bugs to <hasdel-support@lists.rwth-aachen.de>.
# =============================================================================

import os
from datetime import datetime


def init():

    global tfpg_trace_writer
    tfpg_trace_writer = TfpgTraceWriter()



# -----------------------------------------------------------
class TfpgTraceWriter():

    def __init__(self):

        path = os.path.dirname(__file__)
        file_name = "tfpg_trace_description.txt"
        
        self.file = open(path + "/../../" + file_name, "w")

        return


    def start(self):

        self.file.write("%s: Start of trace execution\n\n" % (str(datetime.now())))
        return


    def write(self, text):

        self.file.write(text)
        return
        

    def node_activated(self, name, time):

        info = "* Node '%s' activated at time %s\n" % (name, time)
        self.write(info)
        return


    def edge_activated(self, name, time):

        info = "* Edge '%s' activated at time %s\n" % (name, time)
        self.write(info)
        return


    def terminate(self):

        self.file.write("%s: End of trace execution\n" % (str(datetime.now())))
        self.file.close()
        return

    
    pass  # end of class
