# =============================================================================
#  This file is part of the ``HASDEL'' Toolset.
#  Copyright (C) 2014 by Fondazione Bruno Kessler
#  Copyright (C) 2014 by RWTH Aachen University
#
#
#  The HASDEL Toolset can be copied, modified and/or redistributed
#  under the terms of the HASDEL General Public License, either
#  version 1 of the License, or (at your option) any later version.
#
#  You should have received a copy of the HASDEL General Public
#  License along with this software. If not, please contact the
#  copyright owners.
#
#  The HASDEL Toolset is distributed in the hope that it will be
#  useful, but WITHOUT ANY WARRANTY; without even the implied warranty
#  of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  HASDEL General Public License for more details.
#
#  For more information on the HASDEL Project see the Project Web
#  site at <http://compassinternal.informatik.rwth-aachen.de/hasdel-site/> 
#  or email to <hasdel-support@lists.rwth-aachen.de>.  
#
#  Please report bugs to <hasdel-support@lists.rwth-aachen.de>.
# =============================================================================


prog_ver = (1,0,0)
trace_fname = None

def setup_path():
    """Sets up the python include paths"""
    import os.path; import sys

    if sys.argv[0]:
        top_dir = os.path.dirname(os.path.abspath(sys.argv[0]))
        sys.path = [os.path.join(top_dir, ".."), 
                    os.path.join(top_dir, "../../resources/antlr/antlr-python-runtime-3.1.1/"), 
                    os.path.join(top_dir, "../gui"), 
                    os.path.join(top_dir, "../gui/src/local_root")] + sys.path
        pass
    return


def process_options():
    global trace_fname
    from optparse import OptionParser, OptionValueError

    optp = OptionParser("usage: %prog [options] trace-file.xml", 
                        version="%prog " + ".".join(map(str, prog_ver)))

    # sets the given options inside module options
    (opts, args) = optp.parse_args()
    if len(args) >= 1:
        trace_fname = args[0]
    return


def main():
    from src.ctrls.trace import TraceCtrl
    from src.views.trace import TraceView
    from src.models.trace import TraceViewModel
    
    mv = TraceViewModel()

    if trace_fname:
        #add only one trace in the set of possible trace
        m = mv.add_traces_from_file( trace_fname ) 
        mv.set_active_trace( m )

    v = TraceView()
    c = TraceCtrl(mv,v)

    import gtk
    gtk.main()
    return

if __name__ == "__main__":
    process_options()
    setup_path()
    main()

