# =============================================================================
#  This file is part of the ``HASDEL'' Toolset.
#  Copyright (C) 2014 by Fondazione Bruno Kessler
#  Copyright (C) 2014 by RWTH Aachen University
#
#
#  The HASDEL Toolset can be copied, modified and/or redistributed
#  under the terms of the HASDEL General Public License, either
#  version 1 of the License, or (at your option) any later version.
#
#  You should have received a copy of the HASDEL General Public
#  License along with this software. If not, please contact the
#  copyright owners.
#
#  The HASDEL Toolset is distributed in the hope that it will be
#  useful, but WITHOUT ANY WARRANTY; without even the implied warranty
#  of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  HASDEL General Public License for more details.
#
#  For more information on the HASDEL Project see the Project Web
#  site at <http://compassinternal.informatik.rwth-aachen.de/hasdel-site/> 
#  or email to <hasdel-support@lists.rwth-aachen.de>.  
#
#  Please report bugs to <hasdel-support@lists.rwth-aachen.de>.
# =============================================================================


import options
import utils.process
import glob
import subprocess
import tempfile
import os

# A possible value of the attribute "returncode" of Dft2MrmcProcess objects,
# indicating that the resulting model contains action transitions, so the
# output in MRMC format is not meaningful:
ACTION_TRANSITIONS_EXIST = 100

IMC_CONVERT_RESULT_FILENAME = "imc_convert_result.txt"

def start_model2imc(formulae_numbers, output_directory, output_formats=['mrmc','imca'], verbosity=2):
    # constructs a new instance of model2imc logging file
    fdlog, logfname = tempfile.mkstemp(suffix='.model2imc',
                                    dir=None, text=True)

    process = Model2ImcProcess(formulae_numbers, output_directory, output_formats, verbosity=2,
                               stdout=subprocess.PIPE,
                               stderr=subprocess.STDOUT,
                               dumpers=[os.fdopen(fdlog, "w")])
    return process

def run(formulae_numbers, output_directory, output_formats):
    process = start_model2imc(formulae_numbers, output_directory, output_formats)
    process.wait()
    return (process.returncode, (process.get_stdout_text(), process.get_stderr_text()))

class Model2ImcProcess (utils.process.InteractiveProcess):
    """Process handler for running dft2mrmc.
    """
    
    ## Save the output directory for file retrieval.
    _output_directory = None
    
    #working dir, formula_numbers, tools
    
    #TODO: Update this to use the passed formulas and according labels files which are
    #in the same order as the passed formulas 
    def __init__(self, formula_numbers, output_directory, output_formats=['mrmc','imca'], verbosity=0, 
                 stdout=None, stdin=None, stderr=None, dumpers=[]):
        
        outputcmd = ",".join(output_formats)
        
        path = utils.process.get_exec_name(options.MODEL2IMC)
        args = ["--working-directory",output_directory, "--output-types",outputcmd, "-v", str(verbosity)]

        for formula_number in formula_numbers:
            args.append("--property")
            args.append(str(formula_number))

        super(Model2ImcProcess, self).__init__(path, args, stdout=subprocess.PIPE, 
                                              stdin=stdin, stderr=subprocess.STDOUT, 
                                              dumpers=dumpers)
        self._output_directory = output_directory

    def get_outputfiles(self, suffix):
        return glob.glob(self._output_directory + "/*" + suffix )  
pass # end of class
