# =============================================================================
#  This file is part of the ``HASDEL'' Toolset.
#  Copyright (C) 2014 by Fondazione Bruno Kessler
#  Copyright (C) 2014 by RWTH Aachen University
#
#
#  The HASDEL Toolset can be copied, modified and/or redistributed
#  under the terms of the HASDEL General Public License, either
#  version 1 of the License, or (at your option) any later version.
#
#  You should have received a copy of the HASDEL General Public
#  License along with this software. If not, please contact the
#  copyright owners.
#
#  The HASDEL Toolset is distributed in the hope that it will be
#  useful, but WITHOUT ANY WARRANTY; without even the implied warranty
#  of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  HASDEL General Public License for more details.
#
#  For more information on the HASDEL Project see the Project Web
#  site at <http://compassinternal.informatik.rwth-aachen.de/hasdel-site/> 
#  or email to <hasdel-support@lists.rwth-aachen.de>.  
#
#  Please report bugs to <hasdel-support@lists.rwth-aachen.de>.
# =============================================================================


import os
import glob

import options
import utils.process


# A possible value of the attribute "returncode" of Dft2MrmcProcess objects,
# indicating that the resulting model contains action transitions, so the
# output in MRMC format is not meaningful:
ACTION_TRANSITIONS_EXIST = 100


IMC_CONVERT_RESULT_FILENAME = "imc_convert_result.txt"

#TODO merge this with dft2mrmc

def run_sigref2imc(xml_filename, formulae_numbers, output_directory, output_formats):
    process = Sigref2ImcProcess(xml_filename, output_directory, output_formats, formulae_numbers, verbosity=2)
    process.wait()

    return (process.returncode, process.get_stdout_text(), process.parse_imc_convert_result())

class Sigref2ImcProcess (utils.process.InteractiveProcess):
    """Process handler for running sigref2mrmc.
    """
    
    ## Save the output directory for file retrieval.
    _output_directory = None

    #TODO: Update this to use the passed formulas and according labels files which are
    #in the same order as the passed formulas 
    def __init__(self, xml_filename,
                 output_directory, output_formats=['mrmc','imca'], formulae_numbers=[], verbosity=0, 
                 stdout=None, stdin=None, stderr=None, dumpers=[]):        

        # The specified input files must exist.
        if not os.path.isfile(xml_filename):
            raise IOError("File not found: " + xml_filename)
        
        
        outputcmd = ",".join(output_formats)

        path = utils.process.get_exec_name(options.SIGREF2IMC)
        args = [xml_filename, output_directory, "--output-types",outputcmd, "-v", str(verbosity)]

        for formula_number in formulae_numbers:
            args.append("--property")
            args.append(formula_number)

        super(Sigref2ImcProcess, self).__init__(path, args, stdout=stdout, 
                                              stdin=stdin, stderr=stderr, 
                                              dumpers=dumpers)
        self._output_directory = output_directory

    def get_outputfiles(self, suffix):
        return glob.glob(self._output_directory + "/*" + suffix )
        
    def get_imc_convert_result_file(self):
        return self._output_directory+"/"+IMC_CONVERT_RESULT_FILENAME
    
    def parse_imc_convert_result(self):
        result_file_path = self.get_imc_convert_result_file()
        assert os.path.exists(result_file_path)
        result_file = open(result_file_path)
        lines = result_file.readlines()
        result_file.close()
        
        index2res_map = dict()
        for line in lines:
            splitted_line = line.rstrip("\n").split(", ")
            index = int(splitted_line[0])
            
            if index2res_map.has_key(index):
                index2res_map[index].append(tuple(splitted_line[1:]))
            else:
                index2res_map[index]=[tuple(splitted_line[1:])]
        
        return index2res_map
    pass # end of class
