# =============================================================================
#  This file is part of the ``HASDEL'' Toolset.
#  Copyright (C) 2014 by Fondazione Bruno Kessler
#  Copyright (C) 2014 by RWTH Aachen University
#
#
#  The HASDEL Toolset can be copied, modified and/or redistributed
#  under the terms of the HASDEL General Public License, either
#  version 1 of the License, or (at your option) any later version.
#
#  You should have received a copy of the HASDEL General Public
#  License along with this software. If not, please contact the
#  copyright owners.
#
#  The HASDEL Toolset is distributed in the hope that it will be
#  useful, but WITHOUT ANY WARRANTY; without even the implied warranty
#  of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  HASDEL General Public License for more details.
#
#  For more information on the HASDEL Project see the Project Web
#  site at <http://compassinternal.informatik.rwth-aachen.de/hasdel-site/> 
#  or email to <hasdel-support@lists.rwth-aachen.de>.  
#
#  Please report bugs to <hasdel-support@lists.rwth-aachen.de>.
# =============================================================================


import os
import options
import utils.process


# A possible value of the attribute "returncode" of SigrefProcess objects,
# indicating that the resulting model contains action transitions, so the
# output in MRMC format is not meaningful:
ACTION_TRANSITIONS_EXIST = 100


class SigrefProcess (utils.process.InteractiveProcess):
    """Process handler for running sigref2mrmc"""
    
    

    def __init__(self, f_xml, f_tra, f_lab, f_init,
                 args=[], stdout=None, stderr=None, dumpers=[]):

        # The specified input files must exist.
        if not os.path.isfile(f_xml):
            raise IOError("File not found: " + f_xml)                 
        
        # setting the Sigref binary
        executable_path = utils.process.get_exec_name(options.SIGREF2MRMC)
        # setting arguments
        args = [f_xml, f_tra, f_lab, f_init, "2"]
        
        super(SigrefProcess, self).__init__(executable_path, args, 
                                            stdout=stdout, stderr=stderr, 
                                            dumpers=dumpers)                                         
        return

pass # end of class
