# =============================================================================
#  This file is part of the ``HASDEL'' Toolset.
#  Copyright (C) 2014 by Fondazione Bruno Kessler
#  Copyright (C) 2014 by RWTH Aachen University
#
#
#  The HASDEL Toolset can be copied, modified and/or redistributed
#  under the terms of the HASDEL General Public License, either
#  version 1 of the License, or (at your option) any later version.
#
#  You should have received a copy of the HASDEL General Public
#  License along with this software. If not, please contact the
#  copyright owners.
#
#  The HASDEL Toolset is distributed in the hope that it will be
#  useful, but WITHOUT ANY WARRANTY; without even the implied warranty
#  of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  HASDEL General Public License for more details.
#
#  For more information on the HASDEL Project see the Project Web
#  site at <http://compassinternal.informatik.rwth-aachen.de/hasdel-site/> 
#  or email to <hasdel-support@lists.rwth-aachen.de>.  
#
#  Please report bugs to <hasdel-support@lists.rwth-aachen.de>.
# =============================================================================
import os

import options
import utils.process

class SlimsimProcess (utils.process.InteractiveProcess):
    """Process handler for running slimsim"""

    def __init__(self, f_xml, f_prop, step_bound, error_bound, confidence,
                 strategy, args=[], stdout=None, stdin=None, stderr=None, dumpers=[]):
        #assert strategy != 'input', "SlimsimProcess does not support the interactive input strategy"

        # The specified input files must exist.
        if not os.path.isfile(f_xml):
            raise IOError("Input XML file not found: " + f_xml)
        if not os.path.isfile(f_prop):
            raise IOError("Input properties file not found: " + f_prop)
                
        # Set runtime options
        path = utils.process.get_exec_name(options.SLIMSIM)
        threads = options.MC_THREADS
        args = [f_xml, f_prop, "--disallowdeadlock", "-b"+str(step_bound), "-e"+str(error_bound), "-c"+str(confidence), "-n0", "-t"+str(threads), "--strategy", strategy, "-lwarn"] + args
        super(SlimsimProcess, self).__init__(path, args, stdout=stdout, 
                                         stdin=stdin, stderr=stderr, 
                                         dumpers=dumpers)
        return    
    
pass # end of class
